Package: ggInterval
Type: Package
Title: Visualizing Interval-Valued Data Using 'ggplot2'
Version: 0.2.5
Date: 2026-04-28
Authors@R: c(
    person("Bo-Syue", "Jiang", email = "lukajiang1998@gmail.com", role = "aut"),
    person("Han-Ming", "Wu", email = "wuhm@g.nccu.edu.tw", role = "cre"))
Author: Bo-Syue Jiang [aut],
  Han-Ming Wu [cre]
Maintainer: Han-Ming Wu <wuhm@g.nccu.edu.tw>
Description: Extends 'ggplot2' for visualizing interval-valued data with
    scatter plots, histograms, index plots, boxplots, radar plots, PCA
    displays, and correlation heatmaps. The package also converts
    classical data tables into interval-valued data using clustering
    algorithms or user-defined groupings.
Depends: ggplot2, R (>= 4.4.0), tidyverse, RSDA
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, data.table,
        HistDAWass, MAINT.Data, TTR
Imports: rlang, R6, dplyr, tidyr, gridExtra, gtools, stringr, prodlim,
        ggforce, ggpubr, ggthemes, tibble, magrittr, vctrs
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/hanmingwu1103/ggInterval,
        https://CRAN.R-project.org/package=ggInterval
BugReports: https://github.com/hanmingwu1103/ggInterval/issues
NeedsCompilation: no
Packaged: 2026-04-28 11:19:27 UTC; CodexSandboxOffline
Repository: CRAN
Date/Publication: 2026-04-28 12:30:02 UTC
Built: R 4.6.0; ; 2026-04-28 15:50:00 UTC; unix

Package: ggInterval
Type: Package
Title: Visualizing Interval-Valued Data Using 'ggplot2'
Version: 0.2.5
Date: 2026-04-28
Authors@R: c(
    person("Bo-Syue", "Jiang", email = "lukajiang1998@gmail.com", role = "aut"),
    person("Han-Ming", "Wu", email = "wuhm@g.nccu.edu.tw", role = "cre"))
Author: Bo-Syue Jiang [aut],
  Han-Ming Wu [cre]
Maintainer: Han-Ming Wu <wuhm@g.nccu.edu.tw>
Description: Extends 'ggplot2' for visualizing interval-valued data with
    scatter plots, histograms, index plots, boxplots, radar plots, PCA
    displays, and correlation heatmaps. The package also converts
    classical data tables into interval-valued data using clustering
    algorithms or user-defined groupings.
Depends: ggplot2, R (>= 4.4.0), tidyverse, RSDA
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, data.table,
        HistDAWass, MAINT.Data, TTR
Imports: rlang, R6, dplyr, tidyr, gridExtra, gtools, stringr, prodlim,
        ggforce, ggpubr, ggthemes, tibble, magrittr, vctrs
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/hanmingwu1103/ggInterval,
        https://CRAN.R-project.org/package=ggInterval
BugReports: https://github.com/hanmingwu1103/ggInterval/issues
NeedsCompilation: no
Packaged: 2026-04-28 11:19:27 UTC; CodexSandboxOffline
Repository: CRAN
Date/Publication: 2026-04-28 12:30:02 UTC
Built: R 4.5.2; ; 2026-05-01 19:42:51 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(cor,default)
S3method(cor,symbolic_interval)
S3method(cor,symbolic_tbl)
S3method(cov,default)
S3method(cov,symbolic_interval)
S3method(cov,symbolic_tbl)
S3method(print,ggInterval_hist_result)
S3method(scale,default)
S3method(scale,symbolic_interval)
S3method(scale,symbolic_tbl)
S3method(sd,default)
S3method(sd,symbolic_interval)
S3method(sd,symbolic_tbl)
S3method(summary,default)
S3method(summary,symbolic_interval)
S3method(summary,symbolic_modal)
S3method(summary,symbolic_tbl)
export(RSDA2sym)
export(classic2sym)
export(cor)
export(cov)
export(ggInterval)
export(ggInterval_2Dhist)
export(ggInterval_2DhistMatrix)
export(ggInterval_3Dscatterplot)
export(ggInterval_CRplot)
export(ggInterval_MMplot)
export(ggInterval_PCA)
export(ggInterval_boxplot)
export(ggInterval_corrplot)
export(ggInterval_hist)
export(ggInterval_indexImage)
export(ggInterval_indexplot)
export(ggInterval_lineplot)
export(ggInterval_radarplot)
export(ggInterval_scatterMatrix)
export(ggInterval_scatterplot)
export(ggInterval_tsplot)
export(scale)
export(sd)
export(summary)
import(R6)
import(ggforce)
import(ggplot2)
import(ggthemes)
import(rlang)
import(stats)
import(tidyverse)
import(vctrs)
importFrom(RSDA,classic.to.sym)
importFrom(RSDA,is.sym.interval)
importFrom(dplyr,arrange)
importFrom(ggpubr,ggarrange)
importFrom(grDevices,gray.colors)
importFrom(grDevices,grey.colors)
importFrom(gridExtra,marrangeGrob)
importFrom(gtools,odd)
importFrom(magrittr,"%>%")
importFrom(prodlim,row.match)
importFrom(stringr,str_split)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,separate)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(warning = FALSE)
knitr::opts_chunk$set(message = FALSE)
knitr::opts_chunk$set(out.width = "100%")
knitr::opts_chunk$set(fig.align = "center")
library(knitr)
library(ggInterval)
library(RSDA)

## ----data-facedata------------------------------------------------------------
data(facedata)
facedata

## ----data-summary-------------------------------------------------------------
summary(facedata)

## ----classic2sym-species------------------------------------------------------
myIris <- classic2sym(iris, groupby = "Species")
myIris$intervalData

## ----classic2sym-kmeans-------------------------------------------------------
myIris_km <- classic2sym(iris, groupby = "kmeans", k = 5)
myIris_km$intervalData

## ----rsda2sym, eval = FALSE---------------------------------------------------
# mySym <- RSDA2sym(Cardiological)
# mySym$intervalData

## ----stat-summary-------------------------------------------------------------
summary(facedata)

## ----stat-cor-----------------------------------------------------------------
cor(facedata)

## ----stat-cov-----------------------------------------------------------------
cov(facedata)

## ----stat-scale---------------------------------------------------------------
facedata_scaled <- scale(facedata)
facedata_scaled

## ----indexplot, fig.width = 7, fig.height = 4---------------------------------
ggInterval_indexplot(facedata, aes(x = AD))

## ----indeximage-col, fig.width = 7, fig.height = 4----------------------------
ggInterval_indexImage(facedata, aes(AD),
                      column_condition = TRUE, full_strip = FALSE)

## ----indeximage-full, fig.width = 7, fig.height = 4---------------------------
ggInterval_indexImage(facedata, aes(AD),
                      column_condition = TRUE, full_strip = TRUE) +
  coord_flip()

## ----boxplot-single, fig.width = 7, fig.height = 4----------------------------
ggInterval_boxplot(facedata, aes(AD))

## ----boxplot-all, fig.width = 7, fig.height = 5-------------------------------
ggInterval_boxplot(facedata, plotAll = TRUE)

## ----hist-equal, fig.width = 7, fig.height = 4--------------------------------
ggInterval_hist(facedata, aes(x = AD), bins = 10,
                method = "equal-bin")$plot

## ----hist-unequal, fig.width = 7, fig.height = 4------------------------------
ggInterval_hist(facedata, aes(x = AD),
                method = "unequal-bin")$plot

## ----mmplot, fig.width = 7, fig.height = 4------------------------------------
ggInterval_MMplot(facedata, aes(AD))

## ----mmplot-all, fig.width = 7, fig.height = 5--------------------------------
ggInterval_MMplot(facedata, plotAll = TRUE)

## ----crplot, fig.width = 7, fig.height = 4------------------------------------
ggInterval_CRplot(facedata, aes(AD))

## ----crplot-all, fig.width = 7, fig.height = 5--------------------------------
ggInterval_CRplot(facedata, plotAll = TRUE)

## ----scatterplot, fig.width = 7, fig.height = 5-------------------------------
ggInterval_scatterplot(facedata, aes(x = AD, y = BC))

## ----hist2d, fig.width = 7, fig.height = 5------------------------------------
ggInterval_2Dhist(facedata, aes(x = AD, y = BC), xBins = 10, yBins = 10)

## ----hist2d-oils, fig.width = 7, fig.height = 5-------------------------------
data(oils)
ggInterval_2Dhist(oils, aes(x = GRA, y = FRE), xBins = 5, yBins = 5)

## ----scattermatrix, fig.width = 8, fig.height = 8-----------------------------
ggInterval_scatterMatrix(facedata[, 1:3])

## ----histmatrix, fig.width = 8, fig.height = 8--------------------------------
ggInterval_2DhistMatrix(oils, xBins = 5, yBins = 5)

## ----indeximage-heatmap, fig.width = 8, fig.height = 5------------------------
ggInterval_indexImage(facedata, plotAll = TRUE)

## ----radar-polygon, fig.width = 7, fig.height = 6-----------------------------
data(Environment)
ggInterval_radarplot(Environment[, 5:17],
                     plotPartial = 2,
                     showLegend = FALSE,
                     base_circle = TRUE,
                     base_lty = 2,
                     addText = FALSE) +
  labs(title = "Environment: radar plot (default)")

## ----radar-rect, fig.width = 7, fig.height = 6--------------------------------
ggInterval_radarplot(Environment[, 5:17],
                     plotPartial = 2,
                     type = "rect",
                     showLegend = FALSE,
                     base_circle = TRUE,
                     addText = FALSE) +
  labs(title = "Environment: radar plot (rect)")

## ----scatter3d, fig.width = 7, fig.height = 6---------------------------------
ggInterval_3Dscatterplot(facedata[1:5, ], aes(x = BC, y = EH, z = GH))

## ----pca, fig.width = 7, fig.height = 5---------------------------------------
pca_result <- ggInterval_PCA(facedata, plot = FALSE)
pca_result$ggplotPCA

## ----pca-poly, fig.width = 7, fig.height = 5----------------------------------
pca_poly <- ggInterval_PCA(facedata, poly = TRUE, plot = FALSE)
pca_poly$ggplotPCA

## ----pca-iris, fig.width = 7, fig.height = 5----------------------------------
myIris <- classic2sym(iris, groupby = "Species")
pca_iris <- ggInterval_PCA(myIris, plot = FALSE)
pca_iris$ggplotPCA

## ----ggplot2-theme, fig.width = 7, fig.height = 4-----------------------------
ggInterval_indexplot(facedata, aes(x = AD)) +
  theme_minimal() +
  labs(title = "Index plot of AD", x = "Observation", y = "AD")

## ----ggplot2-scale, fig.width = 7, fig.height = 4-----------------------------
p <- ggInterval_hist(facedata, aes(x = AD), bins = 10,
                     method = "equal-bin")$plot
p + scale_fill_manual(values = rainbow(10))

## ----ggplot2-ref, fig.width = 7, fig.height = 4-------------------------------
ggInterval_CRplot(facedata, aes(AD)) +
  geom_hline(yintercept = 5, linetype = "dashed", color = "red")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  echo = TRUE,
  warning = FALSE,
  message = FALSE,
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)

library(ggInterval)
library(ggplot2)

## ----built_in_data------------------------------------------------------------
data(facedata)
data(Environment)

facedata
Environment

Subjects <- substr(rownames(facedata), 1, 3)
Subjects

## ----ggplot2_workflow---------------------------------------------------------
ggInterval_scatterplot(
  facedata,
  aes(x = AD, y = BC, fill = Subjects),
  showLabels = FALSE,
  col = "black"
) +
  scale_fill_brewer(palette = "Set1") +
  labs(fill = "Subjects")

## ----aggregation_examples-----------------------------------------------------
breastData <- read.csv("data.csv")
breastData$X <- NULL
breastData <- dplyr::select(breastData, -id)

breastData_kmeans <- classic2sym(breastData, groupby = "kmeans", k = 5)
head(breastData_kmeans$intervalData[, 1:4])

breastData_diag <- classic2sym(breastData, groupby = "diagnosis")
head(breastData_diag$intervalData[, 1:4])

## ----descriptive_statistics---------------------------------------------------
mean(facedata)
sd(facedata)

cov(facedata$AD, facedata$BC, method = "BD")
cor(facedata$AD, facedata$BC, method = "BD")

summary(facedata$AD)
summary(Environment[1:3, ])

## ----index_plots--------------------------------------------------------------
ggInterval_indexplot(facedata, aes(x = AD))

ggInterval_indexplot(facedata, aes(x = AD, fill = Subjects)) +
  scale_fill_brewer(palette = "Set1") +
  geom_vline(xintercept = mean(facedata$AD), color = "darkgray") +
  labs(fill = "Subjects")

ggInterval_indexImage(facedata, aes(x = AD)) +
  coord_flip()

ggInterval_indexImage(facedata, aes(x = AD), full_strip = TRUE) +
  coord_flip()

## ----index_plot_ordering------------------------------------------------------
face.ip <- ggInterval_indexplot(facedata, aes(fill = Subjects), plotAll = TRUE) +
  scale_fill_brewer(palette = "Set1") +
  labs(x = "", y = "", fill = "Subjects")
face.ip

b <- ggInterval_indexplot(facedata, aes(fill = Subjects),
  plotAll = TRUE, row_order = "c") +
  scale_fill_brewer(palette = "Set1") +
  labs(x = "", y = "", fill = "Subjects")
b

cc <- ggInterval_indexplot(facedata, aes(fill = Subjects),
  plotAll = TRUE, row_order = "r") +
  scale_fill_brewer(palette = "Set1") +
  labs(x = "", y = "", fill = "Subjects")
cc

## ----boxplots_histograms------------------------------------------------------
ggInterval_boxplot(facedata, plotAll = TRUE, width_type = "violin-like") +
  theme(legend.position = "bottom", axis.text.x = element_blank())

ggInterval_boxplot(facedata, plotAll = TRUE, width_type = "side-by-side") +
  theme(legend.position = "bottom", axis.text.x = element_blank())

ggInterval_boxplot(facedata, aes(x = AD), width_type = "quantile-depth")

hist.obj.equal <- ggInterval_hist(facedata, plotAll = TRUE, bins = 10)
print(hist.obj.equal)
hist.obj.equal$`Table AD`

hist.obj.unequal <- ggInterval_hist(
  facedata,
  plotAll = TRUE,
  method = "unequal-bin"
)
print(hist.obj.unequal)

ggInterval_hist(facedata, aes(x = AD), method = "equal-bin", bins = 10)

ggInterval_hist(facedata, aes(x = AD), method = "unequal-bin")

## ----lineplot_stocklike-------------------------------------------------------
if (!requireNamespace("TTR", quietly = TRUE)) {
  stop("Package 'TTR' is required for this example.")
}

data("ttrc", package = "TTR")
stock.data <- subset(
  ttrc[, c("Date", "Close", "Low", "High")],
  format(Date, "%Y-%m") %in% c("1985-01", "1985-02", "1985-03")
)
stock.data$Month <- factor(
  month.abb[as.integer(format(stock.data$Date, "%m"))],
  levels = month.abb[1:3]
)
stock.data$Day <- as.integer(format(stock.data$Date, "%d"))

stock.data.LH <- classic2sym(
  stock.data,
  groupby = "customize",
  minData = stock.data$Low,
  maxData = stock.data$High
)

ggInterval_lineplot(stock.data.LH, aes(y = V1, x = Day), barWidth = 0.6) +
  geom_point(aes(y = Close), shape = 21, fill = "#D95F02",
             color = "black", size = 1.6, stroke = 0.2) +
  coord_cartesian(xlim = c(1, 31), expand = FALSE) +
  facet_wrap(~Month, ncol = 1, scales = "free_y") +
  scale_x_continuous(breaks = c(1, 8, 15, 22, 29)) +
  labs(title = "Interval-valued Line Plot",
       x = "Day of month", y = "Price") +
  ggthemes::theme_economist() +
  theme(strip.text = element_text(face = "bold"))

## ----minmax_plot--------------------------------------------------------------
mm.plot <- function(x) {
  plot.var <<- names(facedata)[x]
  ggInterval_MMplot(facedata, aes(facedata[[plot.var]], size = 2)) +
    coord_fixed(ratio = 1) +
    theme(legend.position = "none")
}

mm.plot.list <- lapply(seq_along(facedata), mm.plot)
gridExtra::marrangeGrob(mm.plot.list, nrow = 2, ncol = 3, top = "")

## ----centerrange_plot---------------------------------------------------------
cr_aspect_ratio <- 5

ggInterval_CRplot(facedata, aes(size = 1.5), plotAll = TRUE) +
  coord_fixed(ratio = cr_aspect_ratio)

facedata.scale <- scale(facedata)$intervalData
ggInterval_CRplot(facedata.scale, aes(size = 1.5), plotAll = TRUE) +
  coord_fixed(ratio = cr_aspect_ratio)

## ----scatterplot_labels-------------------------------------------------------
ggInterval_scatterplot(
  facedata,
  aes(x = BC, y = AD, fill = Subjects),
  showLabels = TRUE,
  labelSize = 2.6,
  labelPosition = "topright",
  labelNudgeX = -0.2,
  labelNudgeY = 0.15,
  checkOverlap = FALSE,
  color = "black"
) +
  scale_fill_brewer(palette = "Set1") +
  scale_x_continuous(expand = expansion(mult = c(0.08, 0.12))) +
  scale_y_continuous(expand = expansion(mult = c(0.06, 0.10))) +
  labs(fill = "Subjects")

## ----hist2d_equal-------------------------------------------------------------
face.2dh.equal <- ggInterval_2Dhist(
  facedata,
  aes(x = BC, y = AD, col = "white"),
  method = "equal-bin",
  xBins = 10,
  yBins = 10,
  display = "p",
  palette = "Blues",
  direction = 1,
  cell_labels = TRUE
)$plot

face.2dh.equal +
  coord_fixed(ratio = 1)

## ----hist2d_unequal-----------------------------------------------------------
face.2dh.unequal <- ggInterval_2Dhist(
  facedata,
  aes(x = BC, y = AD, col = "white"),
  method = "unequal-bin",
  display = "p",
  palette = "Blues",
  direction = 1,
  tau = 0.5
)$plot

face.2dh.unequal +
  coord_fixed(ratio = 1)

## ----scatter_matrix-----------------------------------------------------------
ggInterval_scatterMatrix(facedata)

ggInterval_scatterMatrix(facedata, aes(fill = "steelblue", alpha = 0.3))

## ----hist2d_matrix_equal------------------------------------------------------
ggInterval_2DhistMatrix(
  facedata,
  aes(col = "white"),
  method = "equal-bin",
  xBins = 10,
  yBins = 10,
  display = "p",
  removeZero = TRUE,
  palette = "Blues",
  direction = 1,
  cell_labels = FALSE
)

## ----hist2d_matrix_unequal----------------------------------------------------
ggInterval_2DhistMatrix(
  facedata,
  aes(col = "white"),
  method = "unequal-bin",
  display = "p",
  palette = "Blues",
  direction = 1,
  tau = 0.5,
  removeZero = TRUE,
  cell_labels = FALSE
)

## ----image_plots--------------------------------------------------------------
ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = TRUE,
  column_condition = FALSE
) +
  scale_colour_distiller(palette = "Blues", direction = 1) +
  labs(x = "Subjects") +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank())

ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = TRUE
) +
  scale_colour_distiller(palette = "Spectral") +
  labs(x = "Subjects")

ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = FALSE,
  column_condition = FALSE
) +
  scale_colour_distiller(palette = "Blues", direction = 1) +
  labs(x = "Subjects") +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank())

ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = FALSE
) +
  scale_colour_distiller(palette = "Spectral") +
  labs(x = "Subjects")

## ----radar_plots--------------------------------------------------------------
library(ggthemes)

ggInterval_radarplot(
  Environment,
  plotPartial = c(4, 6),
  showLegend = FALSE,
  base_circle = FALSE,
  base_lty = 1,
  addText = FALSE,
  addText_modal = FALSE
) +
  scale_fill_manual(values = c("darkred", "darkblue")) +
  scale_color_manual(values = c("darkred", "darkblue")) +
  labs(title = "") +
  theme_hc()

ggInterval_radarplot(
  Environment,
  plotPartial = c(4, 6),
  showLegend = FALSE,
  base_circle = FALSE,
  base_lty = 1,
  addText = FALSE,
  addText_modal = FALSE,
  type = "rect"
) +
  scale_fill_manual(values = c("darkred", "darkblue")) +
  scale_color_manual(values = c("darkred", "darkblue")) +
  labs(title = "") +
  theme_hc()

ggInterval_radarplot(
  facedata,
  base_circle = FALSE,
  base_lty = 1,
  type = "quantile",
  quantileNum = 5,
  showLegend = TRUE,
  Drift = 0
) +
  scale_fill_brewer(palette = "Greys") +
  labs(title = "", fill = "Quantiles") +
  theme_hc()

ggInterval_radarplot(
  Environment,
  base_circle = FALSE,
  base_lty = 1,
  type = "quantile",
  quantileNum = 5,
  showLegend = TRUE,
  Drift = 0
) +
  scale_fill_brewer(palette = "Greys") +
  labs(title = "", fill = "Quantiles") +
  theme_hc()

## ----customization_examples---------------------------------------------------
set.seed(1234567890)
facedata.tmp <- facedata
facedata.tmp$cluster <- RSDA::sym.kmeans(facedata.tmp, k = 3)$cluster

ggInterval_indexplot(facedata.tmp, aes(y = AD, fill = Subjects)) +
  geom_text(
    aes(x = seq_len(nrow(facedata.tmp)), y = .data$AD$min,
        label = rownames(facedata.tmp)),
    vjust = 1.5,
    size = 2
  ) +
  scale_fill_brewer(palette = "Set1") +
  labs(title = "Customized Index Plot Using K-Means Clusters",
       fill = "Subjects") +
  facet_grid(cols = vars(cluster), scales = "free_x", space = "free_x") +
  theme_economist_white() +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank())

## ----histdawass_example, eval = requireNamespace("HistDAWass", quietly = TRUE)----
BLOOD <- HistDAWass::BLOOD
blood.min <- HistDAWass::get.MatH.stats(BLOOD, stat = "min")
blood.max <- HistDAWass::get.MatH.stats(BLOOD, stat = "max")
blood <- data.frame(blood.min, blood.max)

myBLOOD <- classic2sym(
  blood,
  groupby = "customize",
  minData = blood[, 2:4],
  maxData = blood[, 6:8]
)

colnames(myBLOOD$intervalData) <- HistDAWass::get.MatH.main.info(BLOOD)$varnames
ggInterval_PCA(myBLOOD$intervalData, plot = FALSE)$ggplotPCA

## ----maint_data_example, eval = requireNamespace("MAINT.Data", quietly = TRUE)----
AbaloneIdt <- MAINT.Data::AbaloneIdt

AbaloneRange <- exp(AbaloneIdt@LogR)
AbaloneMid <- AbaloneIdt@MidP
AbaloneBounds <- data.frame(
  AbaloneMid - AbaloneRange / 2,
  AbaloneMid + AbaloneRange / 2
)

myAbalone <- classic2sym(
  AbaloneBounds,
  groupby = "customize",
  minData = AbaloneBounds[, 1:7],
  maxData = AbaloneBounds[, 8:14]
)

colnames(myAbalone$intervalData) <- AbaloneIdt@VarNames
ggInterval_PCA(myAbalone$intervalData, plot = FALSE)$ggplotPCA

## ----pca_example--------------------------------------------------------------
pca.results <- RSDA::sym.pca(facedata, method = "tops")
rownames(pca.results$Sym.Components) <- rownames(facedata)

ggInterval_scatterplot(
  pca.results$Sym.Components,
  aes(Dim.1, Dim.2, fill = as.factor(Subjects)),
  labelSize = 2.6,
  labelPosition = "topright",
  labelNudgeX = -0.2,
  labelNudgeY = 0.15,
  checkOverlap = TRUE,
  col = "black"
) +
  scale_fill_brewer(palette = "Set1") +
  scale_x_continuous(expand = expansion(mult = c(0.08, 0.12))) +
  scale_y_continuous(expand = expansion(mult = c(0.06, 0.10))) +
  coord_fixed(ratio = 1) +
  labs(x = "PCA-1", y = "PCA-2", fill = "Subjects")

ggInterval_PCA(facedata, poly = TRUE, concepts_group = as.factor(Subjects)) +
  coord_fixed(ratio = 1) +
  labs(x = "PCA-1", y = "PCA-2")

## ----corrplot_example---------------------------------------------------------
ggInterval_corrplot(facedata, method = "BG", triangle = "lower")

## ----session_info-------------------------------------------------------------
sessionInfo()

#' @name classic2sym
#' @title Convert classical data frame into a symbolic data.
#' @description  A function for converting a classical data, which may
#' present as a data frame or a matrix with one entry one value, into
#' a symbolic data object, which is represented as an interval or a set in an entry.
#' Object after converting is ggInterval class containing interval
#' data and raw data(if it exist) and typically statistics.
#' @import rlang stats vctrs
#' @importFrom RSDA classic.to.sym
#' @importFrom stringr str_split
#' @importFrom tibble as_tibble rownames_to_column column_to_rownames
#' @importFrom prodlim row.match
#' @param data A classical data frame that you want to be converted into
#' interval-valued data
#' @param groupby A way to aggregate. It can be either a clustering method
#' or a variable name which exist in input data (necessary factor type)
#' . Default "kmeans".
#' @param k A number of group,which is used by clustering. Default k = 5.
#' @param minData if choose groupby parameter as 'customize',user need to define
#' which data is min data or max data.
#' @param maxData if choose groupby parameter as 'customize',user need to define
#' which data is min data or max data.
#' @param modalData list, each cell of list contain a set of column index of its
#' modal multi-valued data of the input data. the value of it is a proportion
#' presentation, and sum of each row in these column must be equal to 1.
#' ex 0,1,0  or  0.2,0.3,0.5. the input type of modalData for example is
#' modalData[[1]] = c(2, 3), modalData[[2]] = c(7:10), that 2, 3, 7, 8, 9, 10
#' columns are modal type of the data. Note: the option is only valid when
#' groupby == "customize".
#' @usage classic2sym(data=NULL,groupby = "kmeans",k=5,minData=NULL,maxData=NULL,
#' modalData = NULL)
#' @return classic2sym returns an object of class "ggInterval",which
#' has interval-valued data and related outputs as follows.
#' \itemize{
#'   \item intervalData - The Interval data after converting also known
#'   as a RSDA object.
#'   \item rawData - Classical data that user input.
#'   \item clusterResult - Cluster results .If the groupby method is
#'   a clustering method then it will exist.
#'   \item statisticsDF - A list contains data frame including some
#'   typically statistics in each group.
#' }
#' @examples
#' #classical data to symbolic data
#' classic2sym(iris)
#' classic2sym(mtcars, groupby = "kmeans", k = 10)
#' classic2sym(iris, groupby = "hclust", k = 7)
#' classic2sym(iris, groupby = "Species")
#'
#' x1<-runif(10, -30, -10)
#' y1<-runif(10, -10, 30)
#' x2<-runif(10, -5, 5)
#' y2<-runif(10, 10, 50)
#' x3<-runif(10, -50, 30)
#' y3<-runif(10, 31, 60)
#'
#' d<-data.frame(min1=x1,max1=y1,min2=x2,max2=y2,min3=x3,max3=y3)
#' classic2sym(d,groupby="customize",minData=d[,c(1,3,5)],maxData=d[,c(2,4,6)])
#' classic2sym(d,groupby="customize",minData=d$min1,maxData=d$min2)
#'
#'
#' #example for build modal data
#' #for the first modal data proportion
#' a1 <- runif(10, 0,0.4) %>% round(digits = 1)
#' a2 <- runif(10, 0,0.4) %>% round(digits = 1)
#'
#' #for the second modal data proportion
#' b1 <- runif(10, 0,0.4) %>% round(digits = 1)
#' b2 <- runif(10, 0,0.4) %>% round(digits = 1)
#'
#' #for interval-valued data
#' c1 <- runif(10, 10, 20) %>% round(digits = 0)
#' c2 <- runif(10, -50, -10) %>% round(digits = 0)
#'
#' #build simulated data
#' d <- data.frame(a1 = a1, a2 = a2, a3 = 1-(a1+a2),
#'                 c1 = c1, c2 = c2,
#'                 b1 = b1, b2 = b2, b3 = 1-(b1+b2))
#'
#' #transformation
#' classic2sym(d, groupby = "customize",
#'             minData = d$c2,
#'             maxData = d$c1,
#'             modalData = list(1:3, 6:8))#two modal data
#'
#' #extract the data
#' symObj<-classic2sym(iris)
#' symObj$intervalData       #interval data
#' symObj$rawData            #raw data
#' symObj$clusterResult      #cluster result
#' symObj$statisticsDF       #statistics

#' @export
classic2sym <- function(data = NULL,
                        groupby = "kmeans",
                        k = 5,
                        minData = NULL,
                        maxData = NULL,
                        modalData = NULL) {
  if ("ggInterval" %in% class(data) ||
      "symbolic_tbl" %in% class(data)) {
    stop("ERROR : please using classical data frame.")
  }
  pkg.env <- new.env()
  pkg.env$rawData <- data
  pkg.env$modal <- NULL
  pkg.env$statistics <- c("min", "median", "max", "mean")
  pkg.env$statisticsDF <- NULL
  pkg.env$result <- NULL
  pkg.env$intervalData <- NULL
  data <- as.data.frame(data)
  
  if (groupby == "customize" & !is.null(modalData)) {
    #1. test each of col of modalData > 2
    this.modal <- buildModal(data, modalData)
    
    #data remain numeric data
    non.modal.index <- (1:ncol(data))[-unlist(modalData)]
    
    #if no data remain numeric then return directly
    if (length(non.modal.index) == 0) {
      #there is no numeric data
      symObj <- ggInterval$new(
        rawData = pkg.env$rawData,
        statisticsDF = pkg.env$statisticsDF,
        intervalData = this.modal,
        clusterResult = pkg.env$result
      )
      return(symObj)
    } else{
      pkg.env$modal <- this.modal
    }
    #merge in the end
  }
  
  groupby <- substitute(groupby)
  numericData <- unlist(lapply(data.frame(data[1:dim(data)[2]]) , FUN = is.numeric))
  #start debug version0825-1
  for (i in c(1:dim(data)[2])[!numericData]) {
    if (!inherits(data[[i]], c("Date", "POSIXct", "POSIXt"))) {
      data[[i]] <- as.factor(data[[i]])
    }
  }
  #end debug
  numericData <- data.frame(data[, which(numericData)])
  if (isNaExist(data)[[1]]) {
    warning(paste(isNaExist(data)[[2]], "NA observations will be omit."))
    numericData <- na.omit(numericData)
    data <- na.omit(data)
  }
  pkg.env$rawData <- data
  switch(
    toString(groupby),
    kmeans = {
      tryCatch({
        pkg.env$result <- stats::kmeans(numericData, centers = k)
        cluster <- as.factor(pkg.env$result$cluster)
        d <- as.data.frame(cbind(data, cluster = as.factor(pkg.env$result$cluster)))
        
        pkg.env$statisticsDF <- lapply(
          pkg.env$statistics,
          FUN = function(x)
            aggrByGroup(
              df = d,
              group = cluster,
              method = x
            )
        )
        names(pkg.env$statisticsDF) <- pkg.env$statistics
        pkg.env$intervalData <- RSDA::classic.to.sym(d, concept = cluster)
      }, warning = function(war) {
        print(paste("WARNING in kmeans:  ", war))
      }, error = function(err) {
        print(paste("ERROR in kmeans:  ", err))
        print("Automatically using RSDA::classic.to.sym to improve.")
        finallySol(data, numericData)
      })
    },
    #case2 hirechical
    hclust = {
      tryCatch({
        pkg.env$result <- stats::hclust(stats::dist(numericData))
        cluster <- stats::cutree(pkg.env$result, k = k)
        cluster <- as.factor(as.integer(cluster))
        d <- as.data.frame(cbind(data, cluster = cluster))
        pkg.env$statisticsDF <- lapply(
          pkg.env$statistics,
          FUN = function(x)
            aggrByGroup(
              df = d,
              group = cluster,
              method = x
            )
        )
        names(pkg.env$statisticsDF) <- pkg.env$statistics
        pkg.env$intervalData <- RSDA::classic.to.sym(d, concept = cluster)
      }, warning = function(war) {
        print(paste("WARNING in hclust:  ", war))
      }, error = function(err) {
        print(paste("ERROR in hclust:  ", err))
        print("Automatically using RSDA::classic.to.sym to improve.")
        finallySol(data, numericData)
      })
    },
    customize = {
      idata <- customize(numericData, minData, maxData)
      if (is.null(pkg.env$modal)) {
        pkg.env$intervalData <- idata
      } else{
        pkg.env$intervalData <- cbind(idata, pkg.env$modal)
      }
      pkg.env$statisticsDF <- buildStatsDf(numericData = idata)
      names(pkg.env$statisticsDF) <- c("min", "median", "max")
      
    },
    #default
    {
      if (length(as.character(groupby)) > 1) {
        groupby <- as.character(groupby)[-1]
      } else{
        groupby <- as.character(groupby)[1]
      }
      
      d <- RSDA::classic.to.sym(data, groupby)
      d <- buildRowName(data, d, groupby)
      #d <- tibble::as_tibble(d)

      numericData <- unlist(lapply(data.frame(tibble::as_tibble(d)[, 1:dim(d)[2]]) , FUN = RSDA::is.sym.interval))
      numericData <- d[, numericData]
      
      pkg.env$statisticsDF <- buildStatsDf(numericData = numericData)
      names(pkg.env$statisticsDF) <- c("min", "median", "max")
      pkg.env$intervalData <- d
    }
  )
  #final
  if (!("symbolic_tbl" %in% class(data))) {
    class(pkg.env$intervalData) <- c("symbolic_tbl", class(pkg.env$intervalData))
  }
  
  symObj <- ggInterval$new(
    rawData = pkg.env$rawData,
    statisticsDF = pkg.env$statisticsDF,
    intervalData = pkg.env$intervalData,
    clusterResult = pkg.env$result
  )
  
  return(symObj)
}




aggrByGroup <- function(df = NULL,
                        group = NULL,
                        method = "mean") {
  group <- rlang::get_expr(group)#trans string to variable
  #1. select numeric data
  numericData <- unlist(lapply(df[, 1:dim(df)[2]] , FUN = is.numeric))
  nData <- as.data.frame(cbind(df[, numericData], group = group))
  #2. aggrate by method
  p <- dim(nData)[2]
  aggData <- data.frame(NULL)
  for (g in levels(group)) {
    temp <- round(t(unlist(lapply(
      as.data.frame(nData[nData$group == g, 1:p - 1]), FUN = method
    ))), 2)
    aggData <- rbind(aggData, temp)
    #aggData<-rbind(aggData,cbind(temp,cluster=g))
  }
  rownames(aggData) <- levels(group)
  return(aggData)
}

isNaExist <- function(data) {
  if (sum(is.na(data) * 1) == 0) {
    #no na
    return(FALSE)
  } else{
    N <- dim(data)[1]
    completeN <- sum(complete.cases(data) * 1)
    return(list(TRUE, N - completeN))
  }
}

finallySol <- function(data, numericData) {
  pkg.env$intervalData <- RSDA::classic.to.sym(data)
  d <- RSDA::classic.to.sym(numericData)
  mind <- data.frame(matrix(sapply(d, min), nrow = dim(d)[1]))
  maxd <- data.frame(matrix(sapply(d, max), nrow = dim(d)[1]))
  colnames(mind) <- colnames(d)
  colnames(maxd) <- colnames(d)
  pkg.env$statisticsDF <- list(mind, maxd)
  names(pkg.env$statisticsDF) <- c("min", "max")
}

buildRowName <- function(rData, iData, attrList) {
  #change to factor type
  for (i in attrList) {
    rData[, i] <- as.factor(rData[, i])
  }
  
  #all possible combination
  #1. build all attributes levels in a list
  attrLevel <- NULL
  for (i in attrList) {
    attrLevel <- append(attrLevel, list(levels(rData[, i])))
  }
  #2 find combination by attributes levels list
  combination <- NULL
  if (length(attrLevel) > 1) {
    for (i in (length(attrLevel) - 1):1) {
      temp <- NULL
      for (u in 1:length(attrLevel[[i]])) {
        if (i == (length(attrLevel) - 1)) {
          temp <- append(temp, paste(attrLevel[[i]][u], attrLevel[[i + 1]], sep =
                                       ","))
        } else{
          temp <- append(temp, paste(attrLevel[[i]][u], combination, sep = ","))
        }
      }
      combination <- temp
    }
  } else{
    combination <- attrLevel[[1]]
    iData[, "rowname"] <- combination
    iData <- tibble::column_to_rownames(iData, var = "rowname")
    return(iData)
  }
  
  #data frame combination
  df <- rData[!duplicated(rData[, attrList]), attrList]
  
  #which combination exactly exist
  notIn <- NULL
  for (comb in combination) {
    temp <- stringr::str_split(comb, ",")
    #if not exist
    if (is.na(prodlim::row.match(unlist(temp), df))) {
      notIn <- append(notIn, comb)
    }
  }
  idx <- which(unlist(lapply(combination, function(x) {
    x %in% notIn
  })))
  
  if (length(idx) > 0) {
    combination <- combination[-idx]
  }
  
  
  #build rownames by combination
  rownames(iData) <- combination
  iData <- tibble::rownames_to_column(iData)
  iData <- tibble::column_to_rownames(iData, var = "rowname")
  return(iData)
}

customize <- function(nData = NULL,
                      minList = NULL,
                      maxList = NULL) {
  if (is.null(nData)) {
    stop("ERROR : Missing data input.")
  }
  nData <- as.data.frame(nData)
  if (is.null(minList) || is.null(maxList)) {
    #default
    if (dim(nData)[2] == 2) {
      minList <- nData[, 1]
      maxList <- nData[, 2]
      warning(
        "warning : Missing min data or max data.Automatically
              convert column 1 to min data,column 2 to max data."
      )
    } else{
      stop("ERROR : Missing min data or max data.")
    }
  }
  
  minList <- as.data.frame(minList)
  maxList <- as.data.frame(maxList)
  #test whether minList len == maxList
  if (dim(minList)[1] != dim(maxList)[1] ||
      dim(minList)[2] != dim(maxList)[2]) {
    stop("ERROR : Dimension Error,cannot match min and max.")
  }
  
  #test all min smaller than all max
  if (!all(minList <= maxList)) {
    stop("ERROR : min values must smaller than all max values.")
  }
  
  p <- length(minList)
  n <- dim(minList)[1]
  allVar <- lapply(
    1:p,
    FUN = function(x)
      cbind(minList[x], maxList[x])
  )
  finalData <- matrix(NA, nrow = n)
  for (i in 1:p) {
    temp <- unlist(lapply(
      1:n,
      FUN = function(x)
        cbind(allVar[[i]][[1]][x], allVar[[i]][[2]][x])
    ))
    g = as.factor(rep(1:n, each = 2))
    data <- data.frame(temp, g)
    data <- RSDA::classic.to.sym(data, g)
    colnames(data) <- paste0("V", i)
    finalData <- cbind(finalData, data)
  }
  finalData <- tibble::as_tibble(finalData)
  finalData <- finalData[, 2:(p + 1)]
  return(finalData)
}

buildStatsDf <- function(numericData = NULL) {
  mind <- data.frame(matrix(sapply(numericData, min), nrow = dim(numericData)[1]))
  maxd <- data.frame(matrix(sapply(numericData, max), nrow = dim(numericData)[1]))
  mediand <- (mind + maxd) / 2
  colnames(mind) <- colnames(numericData)
  colnames(maxd) <- colnames(numericData)
  colnames(mediand) <- colnames(numericData)
  return(list(mind, mediand, maxd))
}

buildModal <- function(data, modalData) {
  n <- nrow(data)
  result <- data.frame(none = matrix(0, nrow = n, ncol = 1))
  if (!is.list(modalData)) {
    stop("modalData must be a list containing the column index of data.")
  }
  
  u <- 1 # for name index
  for (i in modalData) {
    if (!all(apply(data[, i], 1, sum) == 1)) {
      warning("Break off: sum of observation in each row must equal to 1.")
      return(result[, -1])
    }
    if (length(i) < 2) {
      warning("select length of column of modalData must >= 2")
      return(result[, -1])
    }
    if (!is.numeric(i)) {
      warning("select column of modalData must be index (numeric type)")
      return(result[, -1])
    }
    
    propName <- colnames(data)[i]
    modal <- list(NULL)
    for (obs in 1:n) {
      modal[[obs]] <- list(var = names(data[obs, i]), prop = as.numeric(data[obs, i]))
    }
    modal <- dplyr::tibble(vctrs::new_vctr(modal, class = "symbolic_modal"))
    colnames(modal) <- paste0("modal", u)
    u <- u + 1
    result <- cbind(result, modal)
  }
  return(dplyr::tibble(result[, -1]))
}

#' @name ggInterval_scatterplot
#' @title Scatter plot for two continuous interval variables
#' @description Visualize the distribution of two continuous
#' interval-valued variables with rectangles whose widths and heights
#' represent the corresponding intervals.
#' @import rlang ggplot2
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object.It can also be either RSDA object or
#' classical data frame, which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' @param showLabels Logical. If \code{TRUE}, add row labels to the rectangles.
#' @param labelSize Numeric text size used when \code{showLabels = TRUE}.
#' @param labelPosition Character label position used when
#' \code{showLabels = TRUE}. One of \code{"topright"}, \code{"topleft"},
#' \code{"bottomright"}, or \code{"bottomleft"}.
#' @param labelNudgeX Numeric horizontal adjustment applied to label positions.
#' @param labelNudgeY Numeric vertical adjustment applied to label positions.
#' @param checkOverlap Logical. Passed to \code{geom_text()} to suppress
#' overlapping labels when needed.
#' @param ... Others in ggplot2.
#' @return Return a ggplot2 object.
#' @usage ggInterval_scatterplot(data = NULL,mapping = aes(NULL),
#' showLabels = TRUE, labelSize = 3,
#' labelPosition = "topright", labelNudgeX = 0, labelNudgeY = 0,
#' checkOverlap = FALSE, ...)
#' @examples
#' Subjects <- substr(rownames(facedata), 1, 3)
#' ggInterval_scatterplot(facedata, aes(x = AD, y = BC))
#' ggInterval_scatterplot(
#'   facedata,
#'   aes(x = AD, y = BC, fill = Subjects),
#'   showLabels = TRUE,
#'   labelSize = 2.6,
#'   labelPosition = "topright",
#'   labelNudgeX = 0.8,
#'   labelNudgeY = 0.15,
#'   checkOverlap = TRUE,
#'   col = "black"
#' )
#' p <- ggInterval_scatterplot(facedata[1:10, ], aes(x = AD, y = BC, alpha = 0.2))
#' p + scale_fill_manual(
#'   labels = rownames(facedata)[1:10],
#'   values = rainbow(10),
#'   name = "Group"
#' )
#' @export
ggInterval_scatterplot <- function(data = NULL,
                                   mapping = aes(NULL),
                                   showLabels = TRUE,
                                   labelSize = 3,
                                   labelPosition = "topright",
                                   labelNudgeX = 0,
                                   labelNudgeY = 0,
                                   checkOverlap = FALSE,
                                   ...) {
  #data preparing
  . <- NULL
  caller_env <- parent.frame()
  labelPosition <- match.arg(
    labelPosition,
    choices = c("topright", "topleft", "bottomright", "bottomleft")
  )
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  p <- dim(data)[2]
  n <- dim(iData)[1]
  myRowNames <- rownames(iData)
  
  #test big o
  if (n > 150 & n <= 2000) {
    warning("It is not recommended for too many observations.")
  } else if (n > 2000) {
    stop("Out of time limits.")
  }
  
  #start process
  with(data, {
    resolve_aes_value <- function(expr, n_expected) {
      if (is.null(expr)) {
        return(NULL)
      }
      value <- tryCatch(
        eval(expr, envir = data, enclos = caller_env),
        error = function(err) NULL
      )
      if (is.null(value)) {
        return(NULL)
      }
      if (!(is.atomic(value) || is.factor(value))) {
        return(NULL)
      }
      if (length(value) == 1) {
        return(rep(value, n_expected))
      }
      if (length(value) != n_expected) {
        stop("ERROR : Aesthetic values must have length 1 or match the number of observations.")
      }
      value
    }
    
    #get attrs
    attr1 <- which(unlist(lapply(
      data[, 1:p], FUN = identical, x = eval(this.x)
    )))
    attr1 <- names(attr1)
    attr2 <- which(unlist(lapply(
      data[, 1:p], FUN = identical, x = eval(this.y)
    )))
    attr2 <- names(attr2)
    #if cannot find attr
    if (length(attr1) == 0 || length(attr2) == 0) {
      stop("ERROR : Missing attributes x or y in data frame.")
    }
    
    #test attribute illegal
    if (all((!is.numeric(data[[attr1]])) , !RSDA::is.sym.interval(data[[attr1]]))
        ||
        all((!is.numeric(data[[attr2]])), !RSDA::is.sym.interval(data[[attr2]]))) {
      stop("ERROR : Variables in Scatter Plot can only be numeric.")
    }
    
    #build data.frame for ggplot
    d <- data.frame(
      x1 = iData[[attr1]]$min,
      x2 = iData[[attr1]]$max,
      y1 = iData[[attr2]]$min,
      y2 = iData[[attr2]]$max
    )
    
    #build Aesthetic (mapping)
    xyLocation <- c(which(names(mapping) == "x"), which(names(mapping) == "y"))
    if (length(xyLocation) != 0) {
      usermapping <- mapping[-xyLocation] #Aesthetic without x,y
    } else{
      usermapping <- mapping
    }
    #add facets
    d <- addFactor(rawData = data, iData = d)
    
    
    fill_expr <- args$fill
    colour_expr <- args$colour
    if (is.null(colour_expr)) {
      colour_expr <- args$color
    }
    if (is.null(colour_expr)) {
      colour_expr <- args$col
    }
    fcLocation <- which(names(usermapping) %in% c("fill", "colour", "color", "col"))
    if (length(fcLocation) != 0) {
      usermapping <- usermapping[-fcLocation] #Aesthetic without fill, col
    }
    fill_value <- resolve_aes_value(fill_expr, n)
    if (!is.null(fill_value)) {
      d$rect_fill <- fill_value
    }
    colour_value <- resolve_aes_value(colour_expr, n)
    if (!is.null(colour_value)) {
      d$rect_colour <- colour_value
    }
    
    mapping_rect <- aes(
      xmin = x1,
      xmax = x2,
      ymin = y1,
      ymax = y2,
      alpha = 0.5
    )
    if (length(usermapping) != 0) {
      mapping_rect <- structure(c(mapping_rect, usermapping), class = "uneval")
    }
    if (!is.null(fill_value)) {
      mapping_rect$fill <- rlang::expr(.data$rect_fill)
    }
    if (!is.null(colour_value)) {
      mapping_rect$colour <- rlang::expr(.data$rect_colour)
    }
    
    #ggplot(data=d,aes(x = x1, y = y1))+
    #  do.call(geom_rect,ss)
    
    #start plot
    base <- ggplot(data = d, aes(x = x1, y = y1)) +
      geom_rect(mapping = mapping_rect, ...) +
      guides(colour = "none", alpha = "none") +
      labs(x = attr1, y = attr2)
    if (!is.null(fill_value)) {
      base <- base + labs(fill = rlang::as_label(fill_expr))
    }
    
    if (showLabels) {
      labelSpec <- switch(
        labelPosition,
        topright = list(x = d$x2, y = d$y2, hjust = 0, vjust = 0),
        topleft = list(x = d$x1, y = d$y2, hjust = 1, vjust = 0),
        bottomright = list(x = d$x2, y = d$y1, hjust = 0, vjust = 1),
        bottomleft = list(x = d$x1, y = d$y1, hjust = 1, vjust = 1)
      )
      d$label_x <- labelSpec$x + labelNudgeX
      d$label_y <- labelSpec$y + labelNudgeY
      d$label_text <- myRowNames
      base <- base +
        geom_text(
          data = d,
          mapping = aes(
            x = .data$label_x,
            y = .data$label_y,
            label = .data$label_text
          ),
          inherit.aes = FALSE,
          size = labelSize,
          check_overlap = checkOverlap,
          hjust = labelSpec$hjust,
          vjust = labelSpec$vjust
        )
    }
    
    base
    
  })
}

#' @name ggInterval_MMplot
#' @title A min-max plot for interval data
#' @description  Visualize the range of the variables of each observations
#' by marking minimal and maximal point.
#' @import ggplot2
#' @importFrom dplyr arrange
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame,which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' @param scaleXY default "local", which means limits of x-axis and y-axis
#' depend on their own variable. "global" means limits of them depend on all
#' variables that user input.
#' @param plotAll booleans, if TRUE, plot all variable together
#' @return Return a ggplot2 object.
#' @usage ggInterval_MMplot(data = NULL,mapping = aes(NULL),
#'           scaleXY = "local",plotAll=FALSE)
#' @examples
#' mydata <- ggInterval::facedata
#' ggInterval_MMplot(mydata, aes(x = AD))
#' ggInterval_MMplot(mydata, aes(x = AD, size = 3))
#' ggInterval_MMplot(mydata, plotAll = TRUE, scaleXY = "global") +
#'   theme_classic()
#' @export
ggInterval_MMplot <- function(data = NULL,
                              mapping = aes(NULL),
                              scaleXY = "local",
                              plotAll = FALSE) {
  #@param sort if FALSE, it will not be sort by min data,default TRUE.
  sort = TRUE
  if (!(scaleXY %in% c("local", "global"))) {
    warning(
      paste0(
        "There is no method called scaleXY = ",
        scaleXY,
        " , please set 'global' or 'local'."
      )
    )
    scaleXY <- "local"
  }
  if (plotAll) {
    if (scaleXY == "local") {
      warning("In order to fix axis, scaleXY must be global when plotAll = T.")
      scaleXY <- "global" #need to fix xy
    }
  }
  
  #data preparing
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  testXY(iData, this.x, this.y)
  p <- dim(iData)[2]
  n <- dim(iData)[1]
  #test big o
  if (dim(iData)[1] > 3000) {
    stop("Out of time limits")
  } else if (dim(iData)[1] > 200) {
    warning("It is not recommended for too many observations.")
  }
  
  #start process
  with(data, {
    if (plotAll) {
      #get numerical data
      numericData <- unlist(lapply(data.frame(iData[1:dim(iData)[2]]) , FUN = is.sym.interval))
      iData <- iData[, which(numericData)]
      attr <- colnames(iData)
      d <- NULL
      #build data frame
      for (i in 1:length(attr)) {
        #build data frame for ggplot
        if (scaleXY == "local") {
          temp <- data.frame(
            min = iData[[attr[i]]]$min,
            max = iData[[attr[i]]]$max,
            var = attr[i],
            min.limits = min(iData[[attr[i]]]$min),
            max.limits = max(iData[[attr[i]]]$max)
          )
        } else{
          temp <- data.frame(
            min = iData[[attr[i]]]$min,
            max = iData[[attr[i]]]$max,
            var = attr[i],
            min.limits = min(ggSymData$statisticsDF$min),
            max.limits = max(ggSymData$statisticsDF$max)
          )
        }
        d <- rbind(d, temp)
      }
    } else{
      #get attr
      if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.x)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.x)
        )))
        attr <- names(attr)
      } else if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.y)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.y)
        )))
        attr <- names(attr)
      } else{
        stop("ERROR : Cannot find variables in aes(...)")
      }
      if (p == 1) {
        attr = colnames(data)
      }
      #test attribute illegal
      if (all(!is.numeric(data[[attr]]) , !RSDA::is.sym.interval(data[[attr]]))) {
        stop("ERROR : Variables in Min-Max Plot can only be numeric.")
      }
      
      
      #build data frame for ggplot
      if (scaleXY == "local") {
        d <- data.frame(
          min = iData[[attr]]$min,
          max = iData[[attr]]$max,
          var = attr,
          min.limits = min(iData[[attr]]$min),
          max.limits = max(iData[[attr]]$max)
        )
      } else{
        d <- data.frame(
          min = iData[[attr]]$min,
          max = iData[[attr]]$max,
          var = attr,
          min.limits = min(ggSymData$statisticsDF$min),
          max.limits = max(ggSymData$statisticsDF$max)
        )
      }
    }
    #if(sort){
    newd <- dplyr::arrange(d, min)
    #}else{newd <- d}
    #build Aesthetic (mapping)
    xyLocation <- c(which(names(mapping) == "x"), which(names(mapping) == "y"))
    if (length(xyLocation) != 0) {
      usermapping <- mapping[-xyLocation] #Aesthetic without x,y
    } else{
      usermapping <- mapping
    }
    #if(sort){
    mymapping <- list(mapping = aes(
      x = min,
      y = max,
      size = 3,
      col = "Max"
    ))
    # }
    # else{
    #   mymapping <- list(mapping=aes(x=1:nrow(newd),y=max,size=4,col="Max"))
    # }
    allmapping <- as.list(structure(as.expression(c(
      usermapping, mymapping
    )), class = "uneval"))
    
    mymapping2 <- list(mapping = aes(size = 3, col = "Min"))
    allmapping2 <- as.list(structure(as.expression(c(
      usermapping, mymapping2
    )), class = "uneval"))
    #plot
    
    #always sort = T
    #if(sort){
    #add facets
    if (!plotAll) {
      newd <- addFactor(rawData = data, iData = newd)
    }
    
    
    base  <- ggplot(newd, aes(x = min, y = min, group = var)) +
      do.call(geom_point, allmapping2) +
      do.call(geom_point, allmapping) +
      geom_segment(aes(
        x = min,
        y = min,
        xend = min,
        yend = max
      )) +
      geom_segment(aes(
        x = min.limits,
        y = min.limits,
        xend = max.limits,
        yend = max.limits
      ),
      lty = 2) +
      guides(size = "none", fill = "none")
    
    if (plotAll) {
      base <- base + labs(x = "", y = "", title = "MM Plot") +
        facet_grid(. ~ var)
    } else{
      base <- base + scale_x_continuous(limits = c(mean(d$min.limits), mean(d$max.limits))) +
        scale_y_continuous(limits = c(mean(d$min.limits), mean(d$max.limits)))
      base <- base + labs(x = "",
                          y = "",
                          title = paste0("MM Plot - ", attr))
    }
    return(base)
    # }
    # else{
    #   ggplot(newd,aes(x=1:nrow(newd),y=min))+
    #     do.call(geom_point,allmapping2)+
    #     do.call(geom_point,allmapping)+
    #     geom_segment(aes(x=1:nrow(newd),y=min,xend=1:nrow(newd),yend=max))+
    #     labs(x="",y=attr,title = "Min-Max Plot")+
    #     guides(size=FALSE,fill=FALSE)+
    #     scale_x_continuous(breaks=1:nrow(newd),labels=rownames(iData))
    # }
    
  })
}

#' @name ggInterval_2DhistMatrix
#' @title 2-Dimension histogram matrix
#' @description Visualize all continuous interval-valued variables with a
#' matrix of 2D histograms. Each off-diagonal panel shows a 2D histogram for
#' a pair of variables, and each diagonal panel displays the variable name.
#' @import tidyverse rlang ggplot2
#' @importFrom RSDA is.sym.interval
#' @importFrom magrittr %>%
#' @param data A ggInterval object. It can also be either an RSDA object or
#' a classical data frame, which will be automatically converted to
#' ggInterval data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit.aes = TRUE (the default), it is combined with
#' the default mapping at the top level of the plot. It is the same as the
#' mapping of ggplot2. This function ignores \code{x} and \code{y} mappings
#' and plots all continuous interval-valued variables.
#' @param method Histogram partition method. Use \code{"equal-bin"} for
#' equally spaced bins or \code{"unequal-bin"} for non-equidistant bins
#' defined by the observed interval endpoints.
#' @param xBins Number of x-axis bins used when \code{method = "equal-bin"}.
#' @param yBins Number of y-axis bins used when \code{method = "equal-bin"}.
#' @param display Metric shown in the cells. Use \code{"p"} for relative
#' frequency, \code{"f"} for weighted frequency, or \code{"h"} for density.
#' @param palette ColorBrewer palette passed to \code{scale_fill_distiller()}.
#' @param direction Direction passed to \code{scale_fill_distiller()}.
#' @param tau Non-negative tolerance used when \code{method = "unequal-bin"}.
#' Consecutive breakpoints whose gaps are smaller than \code{tau} are merged.
#' @param removeZero Whether remove cells whose frequency is equal to zero.
#' @param cell_labels Logical. If \code{TRUE}, add cell labels.
#' @param label_rule Rule used when \code{cell_labels = TRUE}. The default
#' \code{"above-mean"} shows only cells whose displayed values exceed the
#' mean value within the matrix. Alternatives are \code{"nonzero"} and
#' \code{"all"}.
#' @param addFreq Deprecated alias for \code{cell_labels}.
#' @return Return a ggplot2 object.
#' @usage ggInterval_2DhistMatrix(data = NULL, mapping = aes(NULL),
#' method = "equal-bin", xBins = 8, yBins = 8,
#' display = "p", palette = "Blues", direction = 1, tau = 0,
#' removeZero = FALSE, cell_labels = FALSE, label_rule = "above-mean",
#' addFreq = NULL)
#'
#' @examples
#' ggInterval_2DhistMatrix(
#'   oils,
#'   xBins = 5,
#'   yBins = 5,
#'   display = "p",
#'   palette = "Blues",
#'   cell_labels = TRUE
#' )
#'
#' ggInterval_2DhistMatrix(
#'   oils,
#'   method = "unequal-bin",
#'   display = "p",
#'   palette = "Blues",
#'   tau = 0.5
#' )
#'
#' @export
ggInterval_2DhistMatrix <- function(data = NULL,
                                    mapping = aes(NULL),
                                    method = "equal-bin",
                                    xBins = 8,
                                    yBins = 8,
                                    display = "p",
                                    palette = "Blues",
                                    direction = 1,
                                    tau = 0,
                                    removeZero = FALSE,
                                    cell_labels = FALSE,
                                    label_rule = "above-mean",
                                    addFreq = NULL) {
  . <- NULL
  method <- match_hist2d_method(method)
  display <- match_hist2d_display(display)
  label_rule <- match.arg(label_rule, c("above-mean", "nonzero", "all"))
  if (!is.null(addFreq)) {
    cell_labels <- addFreq
  }

  if (method == "equal-bin" && xBins + yBins > 100) {
    stop("ERROR : Bins are too large to calculate. Suggest both axes use fewer than 50 bins.")
  }

  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  n <- nrow(iData)

  if (n > 50) {
    stop("Out of time limits. Suggested number of observations should be less than 50.")
  }

  numericData <- unlist(lapply(iData, FUN = RSDA::is.sym.interval))
  iData <- iData[, numericData, drop = FALSE]

  if (ncol(iData) < 2) {
    stop("At least two numeric interval-valued variables are required.")
  }

  if (ncol(iData) < dim(ggSymData$intervalData)[2]) {
    warning("Ignore non-numeric data.")
  }

  p <- ncol(iData)
  if (n * p > 200) {
    stop("Out of time limits. Suggested dimension should be less than (50 x 4).")
  }

  x_bin_sizes <- vapply(
    seq_len(p),
    FUN.VALUE = numeric(1),
    FUN = function(i) {
      length(build_hist2d_breaks(iData[[i]], xBins, method, tau = tau)) - 1
    }
  )
  y_bin_sizes <- vapply(
    seq_len(p),
    FUN.VALUE = numeric(1),
    FUN = function(i) {
      length(build_hist2d_breaks(iData[[i]], yBins, method, tau = tau)) - 1
    }
  )
  complexity <- n * p * p * max(x_bin_sizes) * max(y_bin_sizes)
  if (complexity >= 2000000 && complexity < 8000000) {
    warning("It is not recommended number of variables and bins be too large.")
  } else if (complexity >= 8000000) {
    stop("Out of time limits. The sample size, dimensions, and bins are too large.")
  }

  usermapping <- mapping[setdiff(names(mapping), c("x", "y"))]
  freq.matrix <- NULL
  for (i in seq_len(p)) {
    var_x <- colnames(iData)[i]
    x_breaks <- build_hist2d_breaks(iData[[i]], xBins, method, tau = tau)
    x_mid <- (min(x_breaks) + max(x_breaks)) / 2

    for (u in seq_len(p)) {
      var_y <- colnames(iData)[u]

      if (i == u) {
        diag_row <- data.frame(
          f = 0,
          p = 0,
          h = 0,
          x1 = min(x_breaks),
          x2 = max(x_breaks),
          y1 = min(x_breaks),
          y2 = max(x_breaks),
          area = (max(x_breaks) - min(x_breaks))^2,
          xmid = x_mid,
          ymid = x_mid,
          display_value = 0,
          display_metric = display,
          xv = var_x,
          yv = var_y,
          isPlot = FALSE,
          textXY = x_mid
        )
        freq.matrix <- rbind(
          freq.matrix,
          diag_row
        )
      } else {
        pair_data <- build_hist2d_data(
          iData = iData,
          attr1 = var_x,
          attr2 = var_y,
          xBins = xBins,
          yBins = yBins,
          method = method,
          tau = tau
        )$data
        pair_data <- hist2d_assign_display(pair_data, display = display)
        pair_data$xv <- var_x
        pair_data$yv <- var_y
        pair_data$isPlot <- TRUE
        pair_data$textXY <- NA_real_
        freq.matrix <- rbind(freq.matrix, pair_data)
      }
    }
  }

  if (removeZero) {
    freq.matrix <- rbind(
      freq.matrix[freq.matrix$display_value != 0 | !freq.matrix$isPlot, , drop = FALSE]
    )
  }

  plot_cells <- freq.matrix[freq.matrix$isPlot, , drop = FALSE]
  border_linewidth <- if (method == "unequal-bin") 0.15 else 0.3

  mymapping <- list(
    data = . %>% dplyr::filter(.data$isPlot),
    mapping = aes(
      xmin = .data$x1,
      xmax = .data$x2,
      ymin = .data$y1,
      ymax = .data$y2,
      fill = .data$display_value
    ),
    alpha = 0.5,
    linewidth = border_linewidth
  )
  allmapping <- as.list(structure(as.expression(c(
    usermapping, mymapping
  )), class = "uneval"))

  base <- ggplot(data = freq.matrix, aes(.data$x1, .data$y1)) +
    do.call(geom_rect, allmapping) +
    geom_text(
      data = . %>% dplyr::filter(!.data$isPlot),
      aes(
        x = .data$textXY,
        y = .data$textXY,
        label = .data$xv
      ),
      size = 12
    ) +
    facet_grid(.data$yv ~ .data$xv, scales = "free") +
    scale_fill_distiller(
      name = hist2d_legend_name(display),
      palette = palette,
      direction = direction,
      limits = c(0, max(plot_cells$display_value))
    ) +
    labs(x = "", y = "") +
    theme_bw()

  if (cell_labels) {
    label_data <- switch(
      label_rule,
      "above-mean" = plot_cells[plot_cells$display_value > mean(plot_cells$display_value), , drop = FALSE],
      "nonzero" = plot_cells[plot_cells$display_value != 0, , drop = FALSE],
      "all" = plot_cells
    )
    if (nrow(label_data) != 0) {
      label_data$cell_label <- format_hist2d_labels(label_data$display_value)
      base <- base + geom_text(
        data = label_data,
        aes(
          x = .data$xmid,
          y = .data$ymid,
          label = .data$cell_label
        ),
        inherit.aes = FALSE
      )
    }
  }

  base
}

#' @name ggInterval_indexplot
#' @title Plot the range of each observations
#' @description  Visualize the range of the variables of each observations
#' by using a kind of margin bar that indicate the minimal and maximal of
#' observations.
#' @import rlang ggplot2
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame,which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' It is the same as the mapping of ggplot2.
#' @param plotAll plot all variables
#' @param row_order Row ordering used when \code{plotAll = TRUE}: \code{"o"}
#' for the original order, \code{"c"} for ascending interval centers within
#' each variable panel, \code{"r"} for ascending interval ranges within each
#' variable panel, and \code{"u"} for a user-defined permutation supplied
#' through \code{user_order}.
#' @param user_order User-defined row permutation used when
#' \code{row_order = "u"} and \code{plotAll = TRUE}. Supply either an integer
#' permutation of \code{1:nrow(data)} or a character permutation of the row
#' names.
#' @param labels Logical. When \code{TRUE} and \code{plotAll = TRUE}, repeat the
#' row labels inside each variable panel. This is especially useful for ordered
#' displays with \code{row_order = "c"} or \code{"r"}. For ordered displays,
#' the outer left-side row labels are suppressed by default.
#' @return Return a ggplot2 object.
#' @usage ggInterval_indexplot(data = NULL,mapping = aes(NULL),
#' plotAll = FALSE, row_order = "o", user_order = NULL, labels = FALSE)
#' @examples
#' mydata <- ggInterval::facedata
#' Subjects <- substr(rownames(mydata), 1, 3)
#' ggInterval_indexplot(mydata, aes(x = AD))
#' ggInterval_indexplot(mydata, aes(x = AD, fill = Subjects))
#' ggInterval_indexplot(mydata, plotAll = TRUE, row_order = "c")
#' custom_order <- c(19:21, 22:24, 1:18, 25:27)
#' ggInterval_indexplot(mydata["AD"], plotAll = TRUE,
#'                      row_order = "u", user_order = custom_order)
#' ggInterval_indexplot(mydata, plotAll = TRUE, row_order = "c", labels = TRUE)
#'
#' @export
ggInterval_indexplot <- function(data = NULL,
                             mapping = aes(NULL),
                             plotAll = FALSE,
                             row_order = "o",
                             user_order = NULL,
                             labels = FALSE) {
  row_order <- match.arg(row_order, c("o", "c", "r", "u"))
  if (!is.logical(labels) || length(labels) != 1 || is.na(labels)) {
    stop("labels must be either TRUE or FALSE.")
  }
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  this.fill <- args$fill
  this.group <- args$group
  
  ggSymData <- testData(data)#test if symdata
  iData <- ggSymData$intervalData
  if (plotAll) {
    if (!is.null(this.x) | !is.null(this.y)) {
      warning("Using plotAll presentation cannot specify variables.")
    }
  } else{
    if (row_order != "o") {
      warning("row_order is only used when plotAll = TRUE.")
    }
    if (!is.null(user_order)) {
      warning("user_order is only used when plotAll = TRUE and row_order = 'u'.")
    }
    if (labels) {
      warning("labels is only used when plotAll = TRUE.")
    }
    testXY(iData, this.x, this.y)
  }
  
  p <- dim(iData)[2]
  n <- dim(iData)[1]
  n.groups <- 1
  default_index_labels <- FALSE
  
  #adjust
  this.fill <- eval(this.fill)
  this.group <- eval(this.group)
  
  if (!is.null(this.group)) {
    if (length(this.group) == n | length(this.group) == 1) {
      #yes
      
      this.group <- as.factor(this.group)
      #test concepts_group illegal
      if (length(unique(table(this.group))) != 1) {
        stop("Each group of concepts must be equal length.")
      }
      
      if (length(this.group) == n) {
        n.concepts <- unique(table(this.group))
        n.groups <- length(table(this.group))
      }
    } else{
      #no
      
      stop(paste0("Length of fill must be equal to data (", n, ") or 1."))
    }
    
  }
  
  myRowNames <- rownames(iData)
  if (is.null(myRowNames)) {
    myRowNames <- as.character(seq_len(n))
  }
  if (plotAll && row_order != "u" && !is.null(user_order)) {
    warning("user_order is only used when row_order = 'u'.")
  }

  #test big o
  if (dim(iData)[1] >= 3000) {
    stop("Out of time limits.")
  } else if (dim(iData)[1] > 200 & dim(iData)[1] < 3000) {
    warning("It is not recommended for too many observations.")
  }
  
  isPlotX <- TRUE
  #temp<-c(which(names(args)=="x"),which(names(args)=="y"))
  #args.noXY<-args[-temp]
  with(data, {
    if (plotAll) {
      #isPlotX = FALSE is OK too
      isPlotX <- TRUE
      d <- NULL
      label_size <- if (n <= 20) {
        3
      } else if (n <= 35) {
        2.4
      } else if (n <= 60) {
        1.9
      } else{
        1.5
      }
      for (i in 1:p) {
        row_index <- reorder_plotall_rows(
          iData,
          row_order = row_order,
          user_order = user_order,
          row_names = myRowNames,
          variable = i
        )
        local_min <- iData[[i]]$min[row_index]
        local_max <- iData[[i]]$max[row_index]
        temp <- data.frame(
          myx = (local_min + local_max) / 2,
          myy = 1:n,
          min = local_min,
          max = local_max,
          g = colnames(iData)[[i]],
          row_label = myRowNames[row_index]
        )
        if (!is.null(this.fill)) {
          if (length(this.fill) == n) {
            temp$plot_fill <- this.fill[row_index]
          } else{
            temp$plot_fill <- rep(this.fill, n)
          }
        }
        if (!is.null(this.group)) {
          if (length(this.group) == n) {
            temp$plot_group <- this.group[row_index]
          } else{
            temp$plot_group <- rep(this.group, n)
          }
        }
        span <- max(temp$max, na.rm = TRUE) - min(temp$min, na.rm = TRUE)
        pad <- if (is.finite(span) && span > 0) {
          0.04 * span
        } else{
          0.5
        }
        temp$label_x <- temp$max + pad
        d <- rbind(d, temp)
      }
      d$g <- as.factor(d$g)
      
    } else{
      #autogenerate variable pretent user forget
      if (!is.null(this.group) & length(this.group) == n) {
        if (is.null(this.x)) {
          this.x <- rep(1:n.concepts, n.groups)
        } else if (is.null(this.y)) {
          this.y <- rep(1:n.concepts, n.groups)
        }
      } else{
        if (is.null(this.x)) {
          this.x <- 1:dim(iData)[1]
          default_index_labels <- TRUE
        } else if (is.null(this.y)) {
          this.y <- 1:dim(iData)[1]
          default_index_labels <- TRUE
        }
      }
      
      if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.x)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.x)
        )))
        attr <- names(attr)
        #adjust data number from user set
        if (!is.null(this.fill) & length(this.fill) == n) {
          #not change
          iData <- iData
        } else{
          iData <- iData[eval(this.y), ]
        }
      } else if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.y)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.y)
        )))
        attr <- names(attr)
        isPlotX <- FALSE
        if (!is.null(this.fill) & length(this.fill) == n) {
          #not change
          iData <- iData
        } else{
          iData <- iData[eval(this.x), ]
        }
      } else{
        stop("ERROR : Cannot find variables in aes(...)")
      }
      if (p == 1) {
        attr = colnames(data)
      }
      #test attribute illegal
      if (all(!is.numeric(data[[attr]]) , !RSDA::is.sym.interval(data[[attr]]))) {
        stop("ERROR : Variables in index Plot can only be numeric.")
      }
      
      mid <- (iData[[attr]]$min + iData[[attr]]$max) / 2
    }
    
    mymapping <- mapping
    
    #start plot
    if (plotAll) {
      mymapping$x <- d$myx
      mymapping$y <- d$myy
      if (!is.null(this.fill)) {
        mymapping$fill <- d$plot_fill
      }
      if (!is.null(this.group)) {
        mymapping$group <- d$plot_group
      }
      p <- plotAllFun(d, mymapping, this.fill, this.group,
                      labels = labels, label_size = label_size)
      if (row_order == "o") {
        return(p + scale_y_continuous(breaks = c(1:n), labels = myRowNames))
      }
      return(
        p +
          scale_y_continuous(breaks = c(1:n), labels = rep("", n)) +
          theme(axis.text.y = element_blank(), axis.ticks.y = element_blank())
      )
    }
    #deal with facets
    #iData <- addFactor(rawData = data, iData = iData)
    
    
    if (isPlotX) {
      mymapping$x <- mid
      mymapping$y <- this.y
      if (!is.null(this.group) & length(this.group) == n) {
        errorBar <- quote(geom_errorbar(
          aes(
            xmin = iData[[attr]]$min,
            xmax = iData[[attr]]$max,
            fill = this.group
          ),
          width = 0.2
        ))
        crossBar <- quote(geom_crossbar(
          aes(
            xmin = iData[[attr]]$min,
            xmax = iData[[attr]]$max,
            fill = this.group
          ),
          width = 0.5,
          fatten = 0.5
        ))
      } else{
        errorBar <- quote(geom_errorbar(aes(
          xmin = iData[[attr]]$min, xmax = iData[[attr]]$max
        ), width = 0.2))
        crossBar <- quote(geom_crossbar(aes(
          xmin = iData[[attr]]$min, xmax = iData[[attr]]$max
        ), width = 0.5, fatten = 0.5))
      }
      if (default_index_labels) {
        scale_xy <- scale_y_continuous(breaks = this.y, labels = myRowNames)
      } else {
        temp <- paste0("scale_y_continuous(breaks=c(", list(this.y), "))")
        scale_xy <- eval(parse(text = temp))
      }
      #temp2 <- paste0("scale_x_continuous(labels = myRowNames)+theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))")
      myLabs <- eval(parse(
        text = paste0(
          "labs(title='Index Plot',y='Observations',x='",
          attr,
          "')"
        )
      ))
    } else{
      mymapping$y <- mid
      mymapping$x <- this.x
      if (!is.null(this.group) & length(this.group) == n) {
        errorBar <- quote(geom_errorbar(
          aes(
            ymin = iData[[attr]]$min,
            ymax = iData[[attr]]$max,
            fill = this.group
          ),
          width = 0.2
        ))
        crossBar <- quote(geom_crossbar(
          aes(
            ymin = iData[[attr]]$min,
            ymax = iData[[attr]]$max,
            fill = this.group
          ),
          width = 0.5,
          fatten = 0.5
        ))
      } else{
        errorBar <- quote(geom_errorbar(aes(
          ymin = iData[[attr]]$min, ymax = iData[[attr]]$max
        ), width = 0.2))
        crossBar <- quote(geom_crossbar(aes(
          ymin = iData[[attr]]$min, ymax = iData[[attr]]$max
        ), width = 0.5, fatten = 0.5))
      }
      if (default_index_labels) {
        scale_xy <- scale_x_continuous(breaks = this.x, labels = myRowNames)
      } else {
        temp <- paste0("scale_x_continuous(breaks=c(", list(this.x), "))")
        scale_xy <- eval(parse(text = temp))
      }
      #temp2 <- paste0("scale_y_continuous(labels = myRowNames)")
      myLabs <- eval(parse(
        text = paste0(
          "labs(title='Index Plot',x='Observations',y='",
          attr,
          "')"
        )
      ))
    }
    mymapping$fill <- this.fill
    mymapping$group <- this.group
    p <- ggplot(iData, mapping = mymapping) +
      geom_point() +
      guides(alpha = "none") +
      scale_xy + myLabs
    if (!is.null(this.fill) | !is.null(this.group)) {
      p + eval(crossBar)
    } else{
      p + eval(errorBar)
    }
  })
}

reorder_plotall_rows <- function(iData,
                                 row_order = "o",
                                 user_order = NULL,
                                 row_names = rownames(iData),
                                 variable = NULL) {
  row_order <- match.arg(row_order, c("o", "c", "r", "u"))
  if (row_order == "o") {
    return(seq_len(nrow(iData)))
  }
  if (row_order == "u") {
    return(validate_user_order(user_order, nrow(iData), row_names))
  }
  if (is.null(variable) || length(variable) != 1) {
    stop("A single variable index must be supplied for row_order = 'c' or 'r'.")
  }
  row_summary <- switch(
    row_order,
    c = (iData[[variable]]$min + iData[[variable]]$max) / 2,
    r = iData[[variable]]$max - iData[[variable]]$min
  )
  order(row_summary)
}

validate_user_order <- function(user_order, n, row_names = NULL) {
  if (is.null(user_order)) {
    stop("user_order must be supplied when row_order = 'u'.")
  }
  if (is.factor(user_order)) {
    user_order <- as.character(user_order)
  }
  if (is.numeric(user_order)) {
    user_order_int <- as.integer(user_order)
    if (length(user_order_int) != n ||
        anyNA(user_order_int) ||
        !all(user_order == user_order_int) ||
        !setequal(user_order_int, seq_len(n))) {
      stop("user_order must be a permutation of 1:nrow(data).")
    }
    return(user_order_int)
  }
  if (is.character(user_order)) {
    if (is.null(row_names) || anyNA(row_names)) {
      stop("Character user_order requires non-missing row names.")
    }
    if (length(user_order) != n ||
        anyNA(user_order) ||
        anyDuplicated(user_order) ||
        !setequal(user_order, row_names)) {
      stop("Character user_order must be a permutation of the row names.")
    }
    return(match(user_order, row_names))
  }
  stop("user_order must be an integer or character permutation.")
}

plotAllFun <- function(d = NULL,
                       mymapping = NULL,
                       this.fill = NULL,
                       this.group = NULL,
                       labels = FALSE,
                       label_size = 2.4) {
  if (!is.null(this.fill) | !is.null(this.group)) {
    p <- ggplot(data = d, mapping = mymapping) +
      guides(alpha = "none") +
      geom_crossbar(aes(xmin = .data$min, xmax = .data$max),
                    width = 0.5, fatten = 0.5) +
      facet_grid(. ~ g, scales = "free")
  } else{
    p <- ggplot(data = d, mapping = mymapping) +
      guides(alpha = "none") +
      geom_errorbar(aes(xmin = .data$min, xmax = .data$max), width = 0.2) +
      facet_grid(. ~ g, scales = "free")
  }
  if (labels) {
    p <- p + geom_text(
      data = d,
      mapping = aes(x = .data$label_x, y = .data$myy, label = .data$row_label),
      inherit.aes = FALSE,
      hjust = 0,
      size = label_size,
      show.legend = FALSE
    )
  }
  p
}

#' @name ggInterval_lineplot
#' @aliases ggInterval_tsplot
#' @title Interval-valued line plot
#' @description Visualize interval-valued data along an ordered horizontal
#' axis. A line can connect the centers of intervals at each position, with
#' crossbars or errorbars indicating the interval range. When the horizontal
#' axis is time, the display becomes an interval-valued time-series plot.
#' @import rlang ggplot2
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame, which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' Must include x for the ordered variable and y for the interval variable.
#' Optional aesthetics include group and fill for multiple line series.
#' It is the same as the mapping of ggplot2.
#' @param barWidth The width of the crossbar or errorbar indicating
#' the interval range at each x position. Default is 0.5.
#' @param add_line Logical; if TRUE (default), connect interval centers with a
#' line. Set FALSE when adding a customized line layer with
#' \code{geom_line(...)}.
#' @return Return a ggplot2 object.
#' @usage ggInterval_lineplot(data = NULL, mapping = aes(NULL),
#' barWidth = 0.5, add_line = TRUE)
#' @usage ggInterval_tsplot(data = NULL, mapping = aes(NULL),
#' barWidth = 0.5, add_line = TRUE)
#' @examples
#' \donttest{
#' if (requireNamespace("TTR", quietly = TRUE)) {
#'   data("ttrc", package = "TTR")
#'   stock.data <- subset(
#'     ttrc[, c("Date", "Close", "Low", "High")],
#'     format(Date, "%Y-%m") %in% c("1985-01", "1985-02", "1985-03")
#'   )
#'   stock.data$Month <- factor(
#'     month.abb[as.integer(format(stock.data$Date, "%m"))],
#'     levels = month.abb[1:3]
#'   )
#'   stock.data$Day <- as.integer(format(stock.data$Date, "%d"))
#'   stock.i <- classic2sym(
#'     stock.data,
#'     groupby = "customize",
#'     minData = stock.data$Low,
#'     maxData = stock.data$High
#'   )
#'   ggInterval_lineplot(stock.i, aes(y = V1, x = Day), barWidth = 0.6) +
#'     geom_point(aes(y = Close), shape = 21, fill = "#D95F02",
#'                color = "black", size = 1.6, stroke = 0.2) +
#'     coord_cartesian(xlim = c(1, 31), expand = FALSE) +
#'     facet_wrap(~Month, ncol = 1, scales = "free_y") +
#'     scale_x_continuous(breaks = c(1, 8, 15, 22, 29)) +
#'     labs(x = "Day of month", y = "Price") +
#'     ggthemes::theme_economist() +
#'     theme(strip.text = element_text(face = "bold"))
#' }
#' }
#' @export
ggInterval_lineplot <- function(data = NULL,
                                mapping = aes(NULL),
                                barWidth = 0.5,
                                add_line = TRUE) {
  args_num <- length(mapping)
  args <- lapply(mapping[1:args_num], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  this.fill <- args$fill
  this.group <- args$group

  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  plotData <- .prepare_lineplot_data(ggSymData)

  num_vars <- ncol(iData)
  n <- nrow(iData)

  if (is.null(this.x) || is.null(this.y)) {
    stop("ERROR : Line plot requires both x (ordered variable) and y (interval) in aes().")
  }

  x_value <- rlang::eval_tidy(this.x, data = plotData)
  if (!(length(x_value) %in% c(1, n))) {
    stop(paste0("Length of x must be equal to data (", n, ") or 1."))
  }
  if (length(x_value) == 1) {
    x_value <- rep(x_value, n)
  }

  y_value <- rlang::eval_tidy(this.y, data = plotData)
  attr <- NULL
  match_y <- unlist(lapply(
    as.data.frame(iData[, 1:num_vars, drop = FALSE]),
    FUN = identical,
    x = y_value
  ))
  if (any(match_y)) {
    attr <- names(match_y)[which(match_y)[1]]
  } else {
    stop("ERROR : Cannot find interval variable y in data.")
  }

  if (all(!is.numeric(iData[[attr]]), !RSDA::is.sym.interval(iData[[attr]]))) {
    stop("ERROR : Variable y in line plot must be numeric or interval type.")
  }

  plotData$.lineplot_x <- x_value
  plotData$.lineplot_ymin <- iData[[attr]]$min
  plotData$.lineplot_ymax <- iData[[attr]]$max
  plotData$.lineplot_mid <- (plotData$.lineplot_ymin + plotData$.lineplot_ymax) / 2

  if (!is.null(this.group)) {
    group_value <- rlang::eval_tidy(this.group, data = plotData)
    if (!(length(group_value) %in% c(1, n))) {
      stop(paste0("Length of group must be equal to data (", n, ") or 1."))
    }
    if (length(group_value) == 1) {
      group_value <- rep(group_value, n)
    }
    plotData$.lineplot_group <- as.factor(group_value)
  }

  if (!is.null(this.fill)) {
    fill_value <- rlang::eval_tidy(this.fill, data = plotData)
    if (!(length(fill_value) %in% c(1, n))) {
      stop(paste0("Length of fill must be equal to data (", n, ") or 1."))
    }
    if (length(fill_value) == 1) {
      fill_value <- rep(fill_value, n)
    }
    plotData$.lineplot_fill <- fill_value
  } else if (!is.null(this.group)) {
    plotData$.lineplot_fill <- plotData$.lineplot_group
  }

  mymapping <- aes(x = .lineplot_x, y = .lineplot_mid)
  if (!is.null(this.group)) {
    mymapping$group <- quote(.lineplot_group)
  }

  myplot <- ggplot(plotData, mapping = mymapping) +
    guides(alpha = "none") +
    labs(title = "Interval-valued Line Plot")

  if (isTRUE(add_line)) {
    myplot <- myplot + geom_line()
  }

  myplot <- myplot + geom_point()

  if (".lineplot_fill" %in% names(plotData)) {
    myplot + geom_crossbar(
      aes(
        ymin = .lineplot_ymin,
        ymax = .lineplot_ymax,
        fill = .lineplot_fill
      ),
      width = barWidth,
      linewidth = 0.25
    )
  } else {
    myplot + geom_errorbar(
      aes(
        ymin = .lineplot_ymin,
        ymax = .lineplot_ymax
      ),
      width = barWidth * 0.4,
      linewidth = 0.25
    )
  }
}

#' @rdname ggInterval_lineplot
#' @export
ggInterval_tsplot <- function(data = NULL,
                              mapping = aes(NULL),
                              barWidth = 0.5,
                              add_line = TRUE) {
  ggInterval_lineplot(
    data = data,
    mapping = mapping,
    barWidth = barWidth,
    add_line = add_line
  )
}

.prepare_lineplot_data <- function(ggSymData) {
  plotData <- as.data.frame(ggSymData$intervalData)
  rawData <- ggSymData$rawData

  if (!is.null(rawData) &&
      is.data.frame(rawData) &&
      nrow(rawData) == nrow(plotData)) {
    extra_names <- setdiff(names(rawData), names(plotData))
    if (length(extra_names) > 0) {
      plotData <- dplyr::bind_cols(plotData, rawData[, extra_names, drop = FALSE])
    }
  }

  plotData
}

if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".lineplot_fill",
    ".lineplot_group",
    ".lineplot_mid",
    ".lineplot_x",
    ".lineplot_ymax",
    ".lineplot_ymin"
  ))
}

library(ggInterval)

is_ggplot_object <- function(x) {
  inherits(x, "ggplot") || inherits(x, "ggplot2::ggplot")
}

test_that("classic2sym returns a ggInterval object", {
  iris_sym <- classic2sym(iris, groupby = "Species")
  expect_s3_class(iris_sym, "ggInterval")
  expect_true("intervalData" %in% names(iris_sym))
})

test_that("scatterplot label controls return a ggplot object", {
  Subjects <- substr(rownames(facedata), 1, 3)
  p <- ggInterval_scatterplot(
    facedata,
    aes(x = AD, y = BC, fill = Subjects),
    showLabels = TRUE,
    labelSize = 2.5,
    labelPosition = "topright",
    labelNudgeX = 0.2,
    labelNudgeY = 0.1,
    checkOverlap = TRUE
  )

  expect_true(is_ggplot_object(p))

  built <- ggplot_build(p)
  expect_equal(built$data[[2]]$x[1], facedata$AD$max[1] + 0.2)
  expect_equal(built$data[[2]]$y[1], facedata$BC$max[1] + 0.1)
})

test_that("scatterplot matrix returns a ggplot object and honors borderLinewidth", {
  p <- ggInterval_scatterMatrix(
    facedata[, 1:3],
    borderLinewidth = 0.15
  )

  expect_true(is_ggplot_object(p))
  built <- ggplot_build(p)
  expect_true(any(abs(built$data[[1]]$linewidth - 0.15) < 1e-8))
})

test_that("histogram variants return plots", {
  h_equal <- ggInterval_hist(facedata, aes(x = AD), method = "equal-bin", bins = 8)
  h_unequal <- ggInterval_hist(facedata, aes(x = AD), method = "unequal-bin")

  expect_s3_class(h_equal, "ggInterval_hist_result")
  expect_s3_class(h_unequal, "ggInterval_hist_result")
  expect_true(is_ggplot_object(h_equal$plot))
  expect_true(is_ggplot_object(h_unequal$plot))
})

test_that("2D histogram variants return plots and tables", {
  h_equal <- ggInterval_2Dhist(
    facedata,
    aes(x = BC, y = AD),
    xBins = 8,
    yBins = 8,
    display = "p",
    palette = "Blues",
    cell_labels = TRUE
  )
  h_unequal <- ggInterval_2Dhist(
    facedata,
    aes(x = BC, y = AD),
    method = "unequal-bin",
    display = "h",
    tau = 0.5
  )

  expect_true(is_ggplot_object(h_equal$plot))
  expect_true(is_ggplot_object(h_unequal$plot))
  expect_true(any(grepl("^Table", names(h_equal))))
  expect_true(any(grepl("^Table", names(h_unequal))))
  expect_true(any(vapply(
    h_equal$plot$layers,
    function(layer) inherits(layer$geom, "GeomText"),
    logical(1)
  )))
})

test_that("2D histogram matrices support equal-bin and unequal-bin methods", {
  m_equal <- ggInterval_2DhistMatrix(
    facedata,
    method = "equal-bin",
    xBins = 6,
    yBins = 6,
    display = "p",
    palette = "Blues",
    cell_labels = TRUE
  )
  m_unequal <- ggInterval_2DhistMatrix(
    facedata,
    method = "unequal-bin",
    display = "h",
    palette = "Blues",
    tau = 0.5,
    cell_labels = FALSE
  )

  expect_true(is_ggplot_object(m_equal))
  expect_true(is_ggplot_object(m_unequal))
})

test_that("PCA label controls return a ggplot object", {
  pca_fit <- ggInterval_PCA(
    facedata,
    plot = FALSE,
    concepts_group = substr(rownames(facedata), 1, 3),
    showLabels = TRUE,
    labelSize = 2.5,
    checkOverlap = TRUE
  )

  expect_true(is_ggplot_object(pca_fit$ggplotPCA))
})

test_that("correlation heatmap returns a ggplot object", {
  p <- ggInterval_corrplot(facedata, method = "BG", triangle = "lower")
  expect_true(is_ggplot_object(p))
})

test_that("boxplot width types return ggplot objects", {
  p_violin <- ggInterval_boxplot(facedata, plotAll = TRUE, width_type = "violin-like")
  p_depth <- ggInterval_boxplot(facedata, aes(x = AD), width_type = "quantile-depth")
  p_side <- ggInterval_boxplot(facedata, plotAll = TRUE, width_type = "side-by-side")

  expect_true(is_ggplot_object(p_violin))
  expect_true(is_ggplot_object(p_depth))
  expect_true(is_ggplot_object(p_side))
})

test_that("index plots support built-in row ordering for plotAll", {
  p_original <- ggInterval_indexplot(facedata, plotAll = TRUE)
  p_centers <- ggInterval_indexplot(
    facedata,
    plotAll = TRUE,
    row_order = "c"
  )
  p_ranges <- ggInterval_indexplot(
    facedata,
    plotAll = TRUE,
    row_order = "r"
  )
  custom_order <- c(19:21, 22:24, 1:18, 25:27)
  p_user <- ggInterval_indexplot(
    facedata,
    plotAll = TRUE,
    row_order = "u",
    user_order = custom_order
  )

  split_labels <- function(plot_obj) {
    split(plot_obj$data$row_label, plot_obj$data$g)
  }
  centers_by_panel <- split_labels(p_centers)
  ranges_by_panel <- split_labels(p_ranges)
  original_by_panel <- split_labels(p_original)
  user_by_panel <- split_labels(p_user)

  expect_identical(p_original$scales$get_scales("y")$labels, rownames(facedata))
  expect_true(all(vapply(
    original_by_panel,
    identical,
    logical(1),
    y = rownames(facedata)
  )))
  expect_true(all(vapply(
    names(centers_by_panel),
    function(var_name) identical(
      centers_by_panel[[var_name]],
      rownames(facedata)[order((facedata[[var_name]]$min + facedata[[var_name]]$max) / 2)]
    ),
    logical(1)
  )))
  expect_true(all(vapply(
    names(ranges_by_panel),
    function(var_name) identical(
      ranges_by_panel[[var_name]],
      rownames(facedata)[order(facedata[[var_name]]$max - facedata[[var_name]]$min)]
    ),
    logical(1)
  )))
  expect_true(all(vapply(
    user_by_panel,
    identical,
    logical(1),
    y = rownames(facedata)[custom_order]
  )))
  expect_true(all(p_centers$scales$get_scales("y")$labels == ""))
  expect_true(all(p_ranges$scales$get_scales("y")$labels == ""))
  expect_true(all(p_user$scales$get_scales("y")$labels == ""))
})

test_that("ordered index plots can repeat row labels within each panel", {
  p_labeled <- ggInterval_indexplot(
    facedata,
    plotAll = TRUE,
    row_order = "c",
    labels = TRUE
  )

  expect_true(any(vapply(
    p_labeled$layers,
    function(layer) inherits(layer$geom, "GeomText"),
    logical(1)
  )))
})

test_that("line plot aliases return ggplot objects with raw-data columns", {
  dates <- as.Date("2021-01-15") + c(0, 4, 5, 6, 7, 10)
  stock.data <- data.frame(
    Date = rep(dates, 2),
    Company = factor(rep(c("A", "B"), each = length(dates))),
    Close = c(100, 101.5, 102, 98.5, 99.5, 100.8,
              150, 151.2, 152.4, 153.5, 154.1, 155.3),
    Low = c(98.9, 100.2, 100.8, 96.9, 98.1, 99.4,
            148.7, 149.8, 151.0, 152.0, 152.7, 154.0),
    High = c(101.3, 102.6, 103.0, 100.3, 100.8, 102.0,
             151.5, 152.5, 153.6, 154.7, 155.2, 156.5)
  )
  stock.i <- classic2sym(
    stock.data,
    groupby = "customize",
    minData = stock.data$Low,
    maxData = stock.data$High
  )

  p_line <- ggInterval_lineplot(
    stock.i,
    aes(y = V1, x = Date, group = Company, fill = Company),
    add_line = FALSE
  ) +
    geom_line(aes(y = Close, color = Company, group = Company), linewidth = 0.4) +
    facet_wrap(~Company, ncol = 1, scales = "free_y")
  p_alias <- ggInterval_tsplot(
    stock.i,
    aes(y = V1, x = Date, group = Company, fill = Company)
  )

  expect_true(is_ggplot_object(p_line))
  expect_true(is_ggplot_object(p_alias))
  expect_true(all(c("Company", "Close", "Date") %in% names(p_line$data)))
})

#' @name ggInterval_boxplot
#' @title A interval Box plot
#' @description  Visualize the one continuous variable distribution using one of
#' three interval-aware boxplot styles.
#' @import rlang ggplot2
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame, which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' It is the same as the mapping of ggplot2.
#' @param plotAll booleans, if TRUE, plot all variable together
#' @param width_type Box-width style. Use \code{"violin-like"} (default) to let
#' the rectangle widths reflect local interval concentration, 
#' \code{"quantile-depth"} for progressively narrower quantile boxes, or
#' \code{"side-by-side"} for conventional boxplots of the lower and upper
#' bounds shown next to each other.
#' @return Return a ggplot2 object.
#' @usage ggInterval_boxplot(data = NULL,mapping = aes(NULL),plotAll=FALSE,
#' width_type = "violin-like")
#' @examples
#' mydata <- ggInterval::facedata
#' ggInterval_boxplot(mydata, aes(x = AD))
#' ggInterval_boxplot(mydata, aes(x = AD), width_type = "quantile-depth")
#' ggInterval_boxplot(mydata, plotAll = TRUE, width_type = "side-by-side")
#'
#' @export
ggInterval_boxplot <- function(data = NULL,
                               mapping = aes(NULL),
                               plotAll = FALSE,
                               width_type = "violin-like") {
  width_type <- match.arg(width_type, c("quantile-depth", "violin-like", "side-by-side"))
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  if (!plotAll) {
    testXY(iData, this.x, this.y)
  }
  p <- dim(iData)[2]
  
  with(data, {
    # build box
    if (plotAll) {
      #get numerical data
      numericData <- unlist(lapply(data.frame(iData[1:dim(iData)[2]]) , FUN = is.sym.interval))
      iData <- iData[, which(numericData)]
      attr <- colnames(iData)
    } else{
      #get attr
      if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.x)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.x)
        )))
        attr <- names(attr)
      } else if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.y)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.y)
        )))
        attr <- names(attr)
      } else{
        stop("ERROR : Cannot find variables in aes(...)")
      }
      if (p == 1) {
        attr = colnames(data)
      }
      #test attribute illegal
      if (all(!is.numeric(data[[attr]]) , !RSDA::is.sym.interval(data[[attr]]))) {
        stop("ERROR : Variables in Box Plot can only be numeric.")
      }
    }

    #plot
    if (is.null(this.x) || is.null(this.y)) {
      mymapping <- mapping[-1]
    } else{
      mymapping <- mapping[-c(1, 2)]
    }

    if (width_type == "side-by-side") {
      d <- build_side_by_side_box_data(iData, attr)
      arg1 <- list(
        mapping = aes(
          x = endpoint,
          y = value,
          fill = endpoint
        ),
        width = 0.6
      )
      arg2 <- mymapping
      ggArgs <- as.list(structure(as.expression(c(arg2, arg1)), class = "uneval"))

      base <- ggplot(data = d) + do.call(geom_boxplot, ggArgs) +
        scale_fill_manual(
          name = "bound",
          values = c("min" = "gray35", "max" = "gray80"),
          labels = c("min" = "Minimum", "max" = "Maximum")
        ) +
        guides(colour = "none", alpha = "none") +
        labs(y = "Values")

      if (plotAll) {
        base <- base + facet_grid(. ~ var)
      } else{
        base <- base + labs(x = attr)
      }
      return(base)
    }

    d <- build_quantile_box_data(iData, attr, width_type)
    d$group <- as.factor(d$group)

    arg1 <- list(
      mapping = aes(
        xmin = x1,
        xmax = x2,
        ymin = y1,
        ymax = y2,
        fill = group
      ),
      col = "black"
    )
    arg2 <- mymapping
    ggArgs <- as.list(structure(as.expression(c(arg2, arg1)), class = "uneval"))

    base  <- ggplot(data = d) + do.call(geom_rect, ggArgs) +
      scale_fill_manual(
        name = "quantile level",
        values = gray.colors(5, start = 0.25, end = 0.85),
        labels = c("0%", "25%", "50%", "75%", "100%")
      ) +
      guides(colour = "none", alpha = "none") +
      scale_x_continuous(limits = c(-0.5, 2.5)) +
      labs(y = "Values")

    if (plotAll) {
      base <- base + facet_grid(. ~ var)
    } else{
      base <- base + labs(x = attr)
    }
    return(base)
  })
}

build_quantile_box_data <- function(iData, attr, width_type = c("quantile-depth", "violin-like")) {
  width_type <- match.arg(width_type)
  quantileN <- 5
  xmid <- rep(1, quantileN)
  d <- NULL

  for (i in seq_along(attr)) {
    mins <- iData[[attr[i]]]$min
    maxs <- iData[[attr[i]]]$max
    y1 <- as.numeric(quantile(mins))
    y2 <- as.numeric(quantile(maxs))
    widths <- switch(
      width_type,
      "quantile-depth" = seq(0.5, 0.3, length.out = quantileN),
      "violin-like" = compute_violin_like_widths(mins, maxs, y1, y2)
    )
    temp <- data.frame(
      x1 = xmid - widths,
      x2 = xmid + widths,
      y1 = y1,
      y2 = y2,
      var = attr[i],
      group = seq_len(quantileN)
    )
    d <- rbind(d, temp)
  }

  d
}

build_side_by_side_box_data <- function(iData, attr) {
  d <- NULL
  for (i in seq_along(attr)) {
    temp <- data.frame(
      value = c(iData[[attr[i]]]$min, iData[[attr[i]]]$max),
      endpoint = factor(
        c(rep("min", length(iData[[attr[i]]]$min)), rep("max", length(iData[[attr[i]]]$max))),
        levels = c("min", "max")
      ),
      var = attr[i]
    )
    d <- rbind(d, temp)
  }
  d
}

compute_violin_like_widths <- function(mins, maxs, y1, y2,
                                       display_min_width = 0.3,
                                       display_max_width = 0.5) {
  scores <- mapply(
    FUN = function(lower, upper) average_interval_density(mins, maxs, lower, upper),
    lower = y1,
    upper = y2
  )
  rescale_box_widths(
    scores,
    display_min_width = display_min_width,
    display_max_width = display_max_width
  )
}

average_interval_density <- function(mins, maxs, lower, upper, grid_n = 25) {
  if (!is.finite(lower) || !is.finite(upper)) {
    return(NA_real_)
  }
  if (lower == upper) {
    return(interval_density(lower, mins, maxs))
  }
  xs <- seq(lower, upper, length.out = grid_n)
  mean(vapply(xs, interval_density, numeric(1), mins = mins, maxs = maxs))
}

interval_density <- function(value, mins, maxs) {
  lens <- pmax(maxs - mins, sqrt(.Machine$double.eps))
  sum((value >= mins & value <= maxs) / lens)
}

rescale_box_widths <- function(scores,
                               display_min_width = 0.3,
                               display_max_width = 0.5) {
  finite_scores <- scores[is.finite(scores)]
  if (length(finite_scores) == 0 || diff(range(finite_scores)) == 0) {
    return(rep((display_min_width + display_max_width) / 2, length(scores)))
  }
  scaled <- (scores - min(finite_scores)) / diff(range(finite_scores))
  scaled[!is.finite(scaled)] <- 0.5
  display_min_width + scaled * (display_max_width - display_min_width)
}

#' @name ggInterval_corrplot
#' @title Correlation heatmap for interval-valued data
#' @description Visualize pairwise symbolic correlations between
#' interval-valued variables with a heatmap. This plot is especially useful
#' for summarizing multivariate dependence structures before more formal
#' modeling or dimension reduction.
#' @import ggplot2
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object. It can also be either an RSDA object or
#' a classical data frame, which will be automatically converted to ggInterval
#' data.
#' @param method Correlation method for interval-valued variables. It must be
#' one of \code{"centers"} (interval centers), \code{"B"} (Billard),
#' \code{"BD"} (Billard-Diday), or \code{"BG"} (Bertrand-Goupil).
#' @param triangle Which part of the symmetric correlation matrix to display.
#' It can be \code{"lower"}, \code{"upper"}, or \code{"full"}.
#' @param showValues Logical. If \code{TRUE}, print the correlation values in
#' each cell.
#' @param digits Number of digits used when \code{showValues = TRUE}.
#' @param showLegend Logical. If \code{TRUE}, display the fill legend.
#' @references
#' Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
#' Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
#' \emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
#' Springer.
#'
#' Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.
#'
#' Billard, Lynne (2008). Sample covariance functions for complex quantitative
#' data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
#' Yokohama, Japan.
#' @return Return a ggplot2 object.
#' @examples
#' ggInterval_corrplot(facedata)
#'
#' ggInterval_corrplot(facedata, method = "BG", triangle = "full")
#' @export
ggInterval_corrplot <- function(data = NULL,
                                method = c("centers", "B", "BD", "BG"),
                                triangle = c("lower", "upper", "full"),
                                showValues = TRUE,
                                digits = 2,
                                showLegend = TRUE) {
  method <- match.arg(method)
  triangle <- match.arg(triangle)
  
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  numericData <- unlist(lapply(as.data.frame(iData), FUN = RSDA::is.sym.interval))
  iData <- iData[, numericData, drop = FALSE]
  
  if (ncol(iData) < 2) {
    stop("At least two interval-valued variables are required.")
  }
  
  corrMat <- as.matrix(cor(iData, method = method))
  corrMat[] <- pmax(-1, pmin(1, corrMat))
  diag(corrMat) <- 1
  varNames <- colnames(corrMat)
  
  plotData <- expand.grid(
    Var1 = varNames,
    Var2 = varNames,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  plotData$i <- match(plotData$Var1, varNames)
  plotData$j <- match(plotData$Var2, varNames)
  plotData$Correlation <- corrMat[cbind(plotData$j, plotData$i)]
  
  if (triangle == "lower") {
    plotData <- plotData[plotData$i <= plotData$j, , drop = FALSE]
  } else if (triangle == "upper") {
    plotData <- plotData[plotData$i >= plotData$j, , drop = FALSE]
  }
  
  plotData$Var1 <- factor(plotData$Var1, levels = varNames)
  plotData$Var2 <- factor(plotData$Var2, levels = rev(varNames))
  plotData$Label <- formatC(plotData$Correlation, digits = digits, format = "f")
  
  base <- ggplot(
    plotData,
    aes(x = .data$Var1, y = .data$Var2, fill = .data$Correlation)
  ) +
    geom_tile(color = "white", linewidth = 0.4) +
    scale_fill_gradient2(
      low = "#2166ac",
      mid = "white",
      high = "#b2182b",
      midpoint = 0,
      limits = c(-1, 1),
      name = paste0("Correlation\n(", method, ")")
    ) +
    coord_equal() +
    labs(x = "", y = "") +
    theme_minimal() +
    theme(
      panel.grid = element_blank(),
      axis.text.x = element_text(angle = 45, hjust = 1),
      axis.title = element_blank()
    )
  
  if (showValues) {
    base <- base + geom_text(aes(label = .data$Label), size = 3)
  }
  
  if (!showLegend) {
    base <- base + guides(fill = "none")
  }
  
  base
}

#' @name ggInterval
#' @title A symbolic object by R6 class for interval analysis and ggplot
#' @description  This is an object that will be used to make a ggplot
#' object.A ggInterval object contains both classic data that user have
#' and interval data which we transform.More over,some basic statistics
#' from row data will also be recorded in this object,and the interval
#' data which is from RSDA transformation will still contain RSDA
#' properties.
#' @import R6
#'
#' @export
ggInterval <- R6::R6Class(
  classname = "ggInterval",
  public = list(
    #' @field rawData the data from user.
    rawData = "data.frame",
    
    #' @field statisticsDF contains min max mean median dataframe for each group of symbolic data
    statisticsDF = "list",
    
    #' @field intervalData interval data from RSDA type
    intervalData = "data.frame",
    
    #' @field clusterResult clustering result
    clusterResult = "list",
    
    #' @description
    #' initialize all data, check whether satisfy theirs form
    initialize = function(rawData = NULL,
                          statisticsDF = NULL,
                          intervalData = NULL,
                          clusterResult = NULL) {
      self$rawData <- rawData
      self$statisticsDF <- statisticsDF
      self$intervalData <- intervalData
      self$clusterResult <- clusterResult
      
      if (private$invalidDataType()) {
        stop("Object type error in statisticsDF")
      }
    }
  ),
  private = list(
    invalidDataType = function() {
      n <- length(self$statisticsDF)
      isAllDF <- all(unlist(lapply(self$statisticsDF[1:n], FUN = is.data.frame)))
      return(!isAllDF)
    }
  )
)

#test whether data can be used for ggplot
testData <- function(data) {
  if ("ggInterval" %in% class(data)) {
    # if ggInterval class?
    return(data)
  } else{
    if (("symbolic_tbl" %in% class(data))) {
      #if RSDA class?
      return(RSDA2sym(data))
    } else{
      warning("Automatically transform a classical data to symbolic data")
      return(classic2sym(data))
    }
  }
}

testXY <- function(iData, this.x, this.y) {
  p <- dim(iData)[2]
  with(iData, {
    isVarX <- any(unlist(lapply(
      iData[, 1:p], FUN = identical, x = eval(this.x)
    )))
    isVarY <- any(unlist(lapply(
      iData[, 1:p], FUN = identical, x = eval(this.y)
    )))
    if (isVarX && isVarY) {
      stop("ERROR : This plot must have exactly one variable")
    }
  })
}


addFactor <- function(rawData, iData) {
  tryCatch({
    test <- unlist(lapply(rawData, is.factor))
    if (any(test)) {
      factorIndex <- which(test)
      if (dim(rawData)[1] == dim(iData)[1]) {
        #OK
        for (i in factorIndex) {
          iData <- dplyr::bind_cols(iData, tmp = rawData[[i]])
          colnames(iData)[dim(iData)[2]] <- colnames(rawData)[i]
        }
      }
    }
    return(iData)
  }, error = function(err) {
    return(iData)
  })
}

#' @name ggInterval_CRplot
#' @title Center-range plot for interval data
#' @description  Visualize the relation between interval centers and ranges,
#' with mean reference lines and an optional ellipse overlay.
#' @import ggplot2
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame,which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' It is the same as the mapping of ggplot2.
#' @param plotAll booleans, if TRUE, plot all variable together
#' @param addEllipse logical, if TRUE (default), add a shaded ellipse layer.
#' @param ellipseFill fill color of the ellipse layer. Default is "blue".
#' @param ellipseAlpha alpha level of the ellipse layer. Default is 0.3.
#' @return Return a ggplot2 object.
#' @usage ggInterval_CRplot(data = NULL,mapping = aes(NULL),plotAll=FALSE,
#'                          addEllipse = TRUE,
#'                          ellipseFill = "blue",
#'                          ellipseAlpha = 0.3)
#' @examples
#' mydata <- ggInterval::facedata
#' ggInterval_CRplot(mydata, aes(x = AD, col = "blue", shape = 2))
#' ggInterval_CRplot(mydata, plotAll = TRUE)
#' ggInterval_CRplot(mydata, plotAll = TRUE, addEllipse = FALSE)
#' @export
ggInterval_CRplot <- function(data = NULL,
                              mapping = aes(NULL),
                              plotAll = FALSE,
                              addEllipse = TRUE,
                              ellipseFill = "blue",
                              ellipseAlpha = 0.3) {
  #data preparing
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  testXY(iData, this.x, this.y)
  p <- dim(iData)[2]
  
  #start process
  with(data, {
    if (plotAll) {
      #get numerical data
      numericData <- unlist(lapply(data.frame(iData[1:dim(iData)[2]]) , FUN = is.sym.interval))
      iData <- iData[, which(numericData)]
      attr <- colnames(iData)
      d <- NULL
      #build data frame
      for (i in 1:length(attr)) {
        #build data frame for ggplot
        center <- (iData[[attr[i]]]$min + iData[[attr[i]]]$max) / 2
        r <- (iData[[attr[i]]]$max - iData[[attr[i]]]$min)
        temp <- data.frame(
          cbind(center, r),
          minCenter = min(center),
          maxCenter = max(center),
          minRange = min(r),
          maxRange = max(r),
          meanC = mean(center),
          meanR = mean(r),
          var = attr[i]
        )
        d <- rbind(d, temp)
        #xmid <- xmid + 1
      }
    } else{
      #get attr
      if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.x)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.x)
        )))
        attr <- names(attr)
      } else if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.y)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.y)
        )))
        attr <- names(attr)
      } else{
        stop("ERROR : Cannot find variables in aes(...)")
      }
      if (p == 1) {
        attr = colnames(data)
      }
      #test attribute illegal
      if (all(!is.numeric(data[[attr]]) , !RSDA::is.sym.interval(data[[attr]]))) {
        stop("ERROR : Variables in Min-Max Plot can only be numeric.")
      }
      
      #build data frame for ggplot
      center <- (iData[[attr]]$min + iData[[attr]]$max) / 2
      r <- (iData[[attr]]$max - iData[[attr]]$min)
      d <- data.frame(cbind(center, r), var = attr)
      #figure limits
      minCenter <- min(d$center)
      maxCenter <- max(d$center)
      minRange <- min(d$r)
      maxRange <- max(d$r)
      meanC <- mean(d$center)
      meanR <- mean(d$r)
    }
    
    #build Aesthetic (mapping)
    xyLocation <- c(which(names(mapping) == "x"), which(names(mapping) == "y"))
    if (length(xyLocation) != 0) {
      usermapping <- mapping[-xyLocation] #Aesthetic without x,y
    } else{
      usermapping <- mapping
    }
    #usermapping <- mapping[-1] #Aesthetic without x,y
    mymapping <- list(mapping = aes(size = 4))
    allmapping <- as.list(structure(as.expression(c(
      usermapping, mymapping
    )), class = "uneval"))
    
    #add facets
    if (!plotAll) {
      d <- addFactor(rawData = data, iData = d)
    }
    
    #plot
    base <- ggplot(d, aes(center, r, group = var)) +
      do.call(geom_point, allmapping) +
      geom_vline(aes(xintercept = meanC), lty = 2) +
      geom_hline(aes(yintercept = meanR), lty = 2) +
      labs(x = "center", y = "range") +
      guides(size = "none")
    if (isTRUE(addEllipse)) {
      base <- base + stat_ellipse(
        geom = "polygon",
        fill = ellipseFill,
        colour = ellipseFill,
        alpha = ellipseAlpha,
        linewidth = 0.35
      )
    }
    if (plotAll) {
      base <- base + facet_grid(. ~ var)
    }
    return(base)
  })
}

#' Generic function for the correlation
#' @name cor
#' @aliases cor
#' @description This function compute the symbolic correlation
#' @param x First symbolic variables.
#' @param y Second symbolic variables.
#' @param use an optional character string giving a method for computing
#' correlation in the presence of missing values. This must be (an abbreviation of)
#'  one of the strings 'everything', 'all.obs', 'complete.obs', 'na.or.complete',
#'  or 'pairwise.complete.obs'.
#' @param method The method to use.
#' @param ... As in R cor function.
#'
#' @return Return a real number in [-1, 1].
#' @details
#' Supported interval-valued methods are:
#' \itemize{
#'   \item \code{"centers"}: correlation of interval centers.
#'   \item \code{"B"}: Billard correlation.
#'   \item \code{"BD"}: Billard-Diday correlation.
#'   \item \code{"BG"}: Bertrand-Goupil correlation.
#' }
#' For \code{"B"}, \code{"BD"}, and \code{"BG"}, the denominator uses the
#' corresponding method-matched standard deviation.
#' @references
#' Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
#' Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
#' \emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
#' Springer.
#'
#' Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.
#'
#' Billard, Lynne (2008). Sample covariance functions for complex quantitative
#' data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
#' Yokohama, Japan.
#'
#' Rodriguez-Rojas, Oldemar (2000). \emph{Classification et modeles lineaires
#' en analyse des donnees symboliques}. PhD thesis, Universite Paris IX
#' Dauphine.
#'
#' @keywords Symbolic correlation
#' @export
cor <- function(x, ...) {
  UseMethod("cor", x)
}

#' @rdname cor
#' @export
cor.default <- function(x,
                        y = NULL,
                        use = "everything",
                        method = c("pearson", "kendall", "spearman"),
                        ...) {
  stats::cor(x, y, use, method)
}

#' @rdname cor
#' @export
cor.symbolic_tbl <- function(x, ...) {
  iData <- x
  
  isnumericData <- unlist(lapply(data.frame(iData[1:dim(iData)[2]]) , FUN = is.sym.interval))
  numericData <- data.frame(iData[, which(isnumericData)])
  p <- ncol(numericData)
  
  d <- sapply(1:p, function(a)
    sapply(1:p, function(b)
      cor(numericData[[a]], numericData[[b]], ...)))
  d <- as.data.frame(d)
  colnames(d) <- colnames(iData[, which(isnumericData)])
  rownames(d) <- colnames(iData[, which(isnumericData)])
  return(d)
  
}

#' @rdname cor
#' @export
cor.symbolic_interval <- function(x,
                                  y,
                                  method = c("centers", "B", "BD", "BG"),
                                  ...) {
  method <- match.arg(method)
  if (method == "centers") {
    out <- stats::cor((min(x) + max(x)) / 2, (min(y) + max(y)) / 2)
  } else{
    out <- cov(x, y, method, ...) / (sd(x, method = method) * sd(y, method = method))
  }
  return(out)
}



#' Generic function for the covariance
#' @name cov
#' @aliases cov
#' @description This function compute the symbolic covariance.
#' @param x First symbolic variables.
#' @param y Second symbolic variables.
#' @param use an optional character string giving a method for computing
#' covariances in the presence of missing values. This must be (an abbreviation of)
#'  one of the strings 'everything', 'all.obs', 'complete.obs', 'na.or.complete',
#'  or 'pairwise.complete.obs'.
#' @param method The method to use.
#' @param na.rm As in R cov function.
#' @param ... As in R cov function.
#'
#' @return Return a real number.
#' @details
#' Supported interval-valued methods are:
#' \itemize{
#'   \item \code{"centers"}: covariance of interval centers.
#'   \item \code{"B"}: Billard covariance.
#'   \item \code{"BD"}: Billard-Diday covariance.
#'   \item \code{"BG"}: Bertrand-Goupil covariance.
#' }
#' @references
#' Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
#' Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
#' \emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
#' Springer.
#'
#' Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.
#'
#' Billard, Lynne (2008). Sample covariance functions for complex quantitative
#' data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
#' Yokohama, Japan.
#'
#' Rodriguez-Rojas, Oldemar (2000). \emph{Classification et modeles lineaires
#' en analyse des donnees symboliques}. PhD thesis, Universite Paris IX
#' Dauphine.
#'
#' @keywords Symbolic Covariance
#' @export
cov <- function(x, ...) {
  UseMethod("cov", x)
}

#' @rdname cov
#' @export
cov.default <- function(x,
                        y = NULL,
                        use = "everything",
                        method = c("pearson", "kendall", "spearman"),
                        ...) {
  stats::cov(x, y, use, method)
}

#' @rdname cov
#' @export
cov.symbolic_tbl <- function(x, ...) {
  iData <- x
  
  isnumericData <- unlist(lapply(data.frame(iData[1:dim(iData)[2]]) , FUN = is.sym.interval))
  numericData <- data.frame(iData[, which(isnumericData)])
  p <- ncol(numericData)
  
  d <- sapply(1:p, function(a)
    sapply(1:p, function(b)
      cov(numericData[[a]], numericData[[b]], ...)))
  d <- as.data.frame(d)
  colnames(d) <- colnames(iData[, which(isnumericData)])
  rownames(d) <- colnames(iData[, which(isnumericData)])
  return(d)
  
}

#' @rdname cov
#' @export
cov.symbolic_interval <- function(x,
                                  y = NULL,
                                  method = c("centers", "B", "BD", "BG"),
                                  na.rm = FALSE,
                                  ...) {
  m <- length(x)
  Gj <- function(a, b, vmean) {
    if ((a + b) / 2 <= vmean) {
      return(-1)
    } else {
      return(1)
    }
  }
  Qj <- function(a, b, vmean) {
    return((a - vmean)^2 + (a - vmean) * (b - vmean) + (b - vmean)^2)
  }
  method <- match.arg(method)
  if (method == "centers") {
    out <- cov((min(x) + max(x)) / 2, (min(y) + max(y)) / 2)
    return(out)
  }
  if (method == "BD") {
    ss <- 0
    vmean.x <- mean(x, method = "centers")
    vmean.y <- mean(y, method = "centers")
    
    for (i in seq_len(length(x))) {
      ss <- ss + Gj(min(x[i]), max(x[i]), vmean.x) * Gj(min(y[i]), max(y[i]), vmean.y) * sqrt(Qj(min(x[i]), max(x[i]), vmean.x) *
                                                                                                Qj(min(y[i]), max(y[i]), vmean.y))
    }
    return((1 / (3 * length(x))) * ss)
  }
  if (method == "BG") {
    a <- sum((min(x) + max(x)) * (min(y) + max(y))) / (4 * m)
    b <- (sum((min(x) + max(x))) * sum((min(y) + max(y)))) / (4 * m^2)
    return(a - b)
  }
  if (method == "B") {
    x_bar <- mean(x)
    y_bar <- mean(y)
    a <- 2 * (min(x) - x_bar) * (min(y) - y_bar)
    b <- (min(x) - x_bar) * (max(y) - y_bar)
    c <- (max(x) - x_bar) * (min(y) - y_bar)
    d <- 2 * (max(x) - x_bar) * (max(y) - y_bar)
    return(sum(a + b + c + d) / (6 * m))
  }
}


#' Generic function for the standard deviation
#' @name sd
#' @aliases sd
#' @description This function compute the symbolic standard deviation.
#' @param x First symbolic variables.
#' @param method The method to use.
#' @param na.rm As in R sd function.
#' @param ... As in R sd function.
#'
#' @return Return a real number.
#' @details
#' Supported interval-valued methods are:
#' \itemize{
#'   \item \code{"billard"}: standard deviation based on the Billard
#'   univariate variance formula.
#'   \item \code{"centers"}: standard deviation of interval centers.
#'   \item \code{"interval"}: interval-valued standard deviation obtained by
#'   standardizing lower and upper bounds separately.
#'   \item \code{"B"}: method-matched standard deviation defined by
#'   \eqn{\sqrt{C_B(X, X)}}.
#'   \item \code{"BD"}: method-matched standard deviation defined by
#'   \eqn{\sqrt{C_{BD}(X, X)}}.
#'   \item \code{"BG"}: method-matched standard deviation defined by
#'   \eqn{\sqrt{C_{BG}(X, X)}}.
#' }
#' @references
#' Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
#' Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
#' \emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
#' Springer.
#'
#' Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.
#'
#' Billard, Lynne (2008). Sample covariance functions for complex quantitative
#' data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
#' Yokohama, Japan.
#'
#' @keywords Symbolic standard deviation
#' @export
sd <- function(x, ...) {
  UseMethod("sd", x)
}

#' @rdname sd
#' @export
sd.default <- function(x, na.rm = FALSE, ...) {
  stats::sd(x, na.rm = na.rm, ...)
}

#' @rdname sd
#' @export
sd.symbolic_tbl <- function(x,
                            method = c("billard", "centers", "interval", "B", "BD", "BG"),
                            na.rm = FALSE,
                            ...) {
  method <- match.arg(method)
  out <- lapply(seq_len(ncol(x)), function(i) {
    col <- x[[i]]
    if (RSDA::is.sym.interval(col)) {
      sd(col, method = method, na.rm = na.rm, ...)
    } else if (is.numeric(col)) {
      stats::sd(col, na.rm = na.rm, ...)
    } else {
      NA_real_
    }
  })
  names(out) <- colnames(x)
  tibble::as_tibble(out)
}

#' @rdname sd
#' @export
sd.symbolic_interval <- function(x,
                                 method = c("billard", "centers", "interval", "B", "BD", "BG"),
                                 na.rm = FALSE,
                                 ...) {
  method <- match.arg(method)
  if (method == "centers") {
    out <- stats::sd((min(x) + max(x)) / 2, na.rm = na.rm)
    return(out)
  }
  if (method == "interval") {
    out <- utils::getFromNamespace("new.sym.intreval", "RSDA")(
      stats::sd(min(x), na.rm = na.rm),
      stats::sd(max(x), na.rm = na.rm)
    )
    return(out)
  }
  if (method == "BG") {
    return(sqrt(cov(x, x, method = "BG", na.rm = na.rm, ...)))
  }
  out <- sqrt((1 / (3 * length(x))) * sum(min(x)^2 + (min(x) * max(x)) + max(x)^2) -
                (1 / (4 * (length(x))^2)) * sum(min(x) + max(x))^2)
  return(out)
}


#' @name RSDA2sym
#' @title RSDA object to symbolic object for ggplot
#' @description  It will be a good way to unify all symbolic data
#' object in R that collects all useful symbolic analysis tools
#' such like RSDA into the same class for management. In this way,
#' user who wants to do some study in symbolic data will be more
#' convenient for searching packages.Thus,RSDA2sym collecting RSDA
#' object into ggInterval object will do for plot(ggplot) and
#' RSDA's analysis.
#' @import stats
#' @param data an interval data, which may transfrom by RSDA::classic.to.sym
#' .Note:data is a necessary parameter,and must have symbolic_tbl class.
#' @param rawData rawData, which can be transformed to interval data,
#' must be a data frame and match to data.
#' @return Return an object of class "ggInterval", which
#' has interval-valued data and related outputs as follows.
#' \itemize{
#'   \item intervalData - The Interval data after converting also known
#'   as a RSDA object.
#'   \item rawData - Classical data that user input.
#'   \item clusterResult - Cluster results .If the groupby method is
#'   a clustering method then it will exist.
#'   \item statisticsDF - A list contains data frame including some
#'   typically statistics in each group.
#' }#'
#' @usage RSDA2sym(data=NULL,rawData=NULL)
#'
#' @examples
#' r<-ggInterval::Cardiological
#' mySym<-RSDA2sym(r)
#' mySym$intervalData
#'
#' @export
RSDA2sym <- function(data = NULL, rawData = NULL) {
  if (!("symbolic_tbl" %in% class(data))) {
    stop("data must be a RSDA object. Missing \"symbolic_tbl\"")
  }
  pkg.env$intervalData <- data
  #if having row data
  if ((!is.null(rawData))) {
    if (!is.data.frame(rawData)) {
      stop("rawData must be a data frame")
    }
    if (!isMatch(data, rawData)) {
      stop("Cannot match data and rawData")
    }
  }
  numericData <- unlist(lapply(as.data.frame(data[, 1:dim(data)[2]]) , FUN = RSDA::is.sym.interval))
  numericData <- data[, numericData]
  
  pkg.env$statisticsDF <- buildStatsDf(numericData)
  names(pkg.env$statisticsDF) <- c("min", "median", "max")
  
  symObj <- ggInterval$new(
    rawData = rawData,
    statisticsDF = pkg.env$statisticsDF,
    intervalData = pkg.env$intervalData,
    clusterResult = pkg.env$result
  )
  return(symObj)
}

isMatch <- function(D, rowD) {
  a <- dim(rowD)[2] >= dim(D)[2]
  b <- colnames(D) %in% colnames(rowD)
  if (a && b) {
    return(TRUE)
  }
  return(FALSE)
}
pkg.env <- new.env()
pkg.env$statistics <- c("min", "median", "max", "mean")
pkg.env$statisticsDF <- NULL
pkg.env$result <- NULL
pkg.env$intervalData <- NULL
pkg.env$rawData <- NULL



#' @name scale
#' @aliases scale
#' @title scale for symbolic data table
#' @description scale for symbolic data table
#' @importFrom RSDA is.sym.interval
#' @param x A ggInterval object. It can also be either RSDA object or
#' classical data frame, which will be automatically convert to ggInterval
#' data.
#' @param center same as base::scale, either a logical value or numeric-alike vector of length equal to the number of columns of x, where nmeric-alike means that as.numeric(.) will be applied successfully if is.numeric(.) is not true.
#' @param scale same as base::scale, either a logical value or a numeric-alike vector of length equal to the number of columns of x.
#' @param ... Used by other R function.
#' @return Return a scale ggInterval object.
#' @examples
#'
#' #For all interval-valued
#' scale(facedata)
#'
#' #For both interval-valued and modal multi-valued
#' scale(mtcars.i)
#'
#' @keywords Symbolic scale
#' @export
scale <- function(x, ...) {
  UseMethod("scale")
}

#' @rdname scale
#' @export
scale.default <- function(x,
                          center = TRUE,
                          scale = TRUE,
                          ...) {
  base::scale.default(x, center, scale)
}

#' @rdname scale
#' @export
scale.symbolic_tbl <- function(x, ...) {
  #test data illegal
  ggSymData <- testData(x)
  iData <- ggSymData$intervalData
  
  
  #get interval-valued col
  interval.index <- lapply(
    1:ncol(iData),
    FUN = function(x)
      RSDA::is.sym.interval(iData[[x]])
  )
  none.interval.index <- which(!unlist(interval.index))
  interval.index <- which(unlist(interval.index))
  if (length(interval.index) == 0)
    stop("Cannot find interval-valued variables to scale.")
  n <- dim(iData)[1]
  
  #scale
  temp1 <- sapply(
    interval.index,
    FUN = function(x)
      unlist(data.frame(iData[[x]]))
  )
  temp2 <- apply(temp1, 2, scale)
  newd <- data.frame(temp2[1:n, ], temp2[(n + 1):(n * 2), ])
  myd <- classic2sym(newd,
                     groupby = "customize",
                     minData = temp2[1:n, ],
                     maxData = temp2[(n + 1):(n * 2), ])
  
  #initial result & merge interval-valued and modal
  result <- data.frame(matrix(NA, nrow = nrow(iData), ncol = ncol(iData)))
  result[, interval.index] <- myd$intervalData
  if (length(none.interval.index) != 0)
    result[, none.interval.index] <- iData[, none.interval.index]
  myd$intervalData <- result
  colnames(myd$intervalData) <- colnames(iData)
  rownames(myd$intervalData) <- rownames(iData)
  
  if (!("symbolic_tbl" %in% class(myd$intervalData))) {
    class(myd$intervalData) <- c(class(myd$intervalData), "symbolic_tbl")
  }
  
  return(myd)
}

#' @rdname scale
#' @export
scale.symbolic_interval <- function(x, ...) {
  s <- RSDA::sd(x)
  m <- mean(x)
  m1 <- (min(x) - m) / s
  m2 <- (max(x) - m) / s
  d <- data.frame(m1 = m1, m2 = m2)
  d2 <- classic2sym(d,
                    groupby = "customize",
                    minData = d$m1,
                    maxData = d$m2)
  
  return(d2$intervalData$V1)
}



#' @name summary
#' @aliases summary
#' @title summary for symbolic data table
#' @description summary for symbolic data table
#' @param object an object for which a summary is desired.
#' @param summary_fun only works when the symbolic_modal class input, it determine which summary function be applied for each modal.
#' @param ... additional arguments affecting the summary produced.
#' @return Return a summary table.
#' @examples
#'
#' #For all interval-valued
#' summary(facedata)
#'
#' #For both interval-valued and modal multi-valued
#' summary(Environment)
#'
#' summary(Environment$URBANICITY, summary_fun = "quantile")
#'
#'
#' @keywords Symbolic summary
#' @export
summary <- function(object, ...) {
  UseMethod("summary")
}

#' @rdname summary
#' @export
summary.default <- function(object, ...) {
  tryCatch({
    eval(parse(text = paste0("base::summary.", class(object)[1])))(object, ...)
  }, error = function(err1) {
    tryCatch({
      base::summary(object, ...)
    }, error = function(err2) {
      base::summary.default(object, ...)
    })
  })
}

#' @rdname summary
#' @export
summary.symbolic_tbl <- function(object, ...) {
  pkg.env$inPackage <- TRUE
  symbolic_interval <- NULL
  symbolic_modal <- NULL
  iData.boolean <- unlist(lapply(object, RSDA::is.sym.interval))
  mData.boolean <- unlist(lapply(object, RSDA::is.sym.modal))
  if (!all(iData.boolean | mData.boolean)) {
    stop(
      "Non-symbolic object detected. Please use classic2sym() to transform data into symbolic_tbl."
    )
  }
  result <- list(symbolic_interval = NULL, symbolic_modal = NULL)
  
  #For interval-valued data
  iData_ind <- which(iData.boolean)
  if (length(iData_ind) > 0) {
    tmp <- data.frame(matrix(0, nrow = 7, ncol = 1))#7: summary interval data will return 7 measures
    for (i in iData_ind) {
      tmp <- cbind(tmp, summary.symbolic_interval(object[, i][[1]]))
    }
    tmp <- tmp[, -1]
    if (class(tmp)[1] == "symbolic_interval") {
      tmp <- data.frame(dplyr::tibble(tmp))
      rownames(tmp) <- c("Min.",
                         "1st Qu.",
                         "Median",
                         "Mean",
                         "3rd Qu.",
                         "Max.",
                         "Std.")
    }
    colnames(tmp) <- colnames(object)[iData_ind]
    result$symbolic_interval <- tmp
  } else{
    result <- within(result, rm(symbolic_interval))
  }
  
  #For modal-multi valued data
  mData_ind <- which(mData.boolean)
  if (length(mData_ind) > 0) {
    tmp <- list(NULL)
    for (i in 1:length(mData_ind)) {
      tmp[[i]] <- summary.symbolic_modal(object[, mData_ind[i]][[1]], ...)
    }
    myMax <- max(unlist(lapply(tmp, length)))
    myMat <- matrix("", ncol = length(mData_ind), nrow = myMax)
    
    for (i in 1:length(tmp)) {
      myMat[1:nrow(tmp[[i]]), i] <- tmp[[i]]
    }
    colnames(myMat) <- colnames(object)[mData_ind]
    result$symbolic_modal <- noquote(myMat)
  } else{
    result <- within(result, rm(symbolic_modal))
  }
  pkg.env$inPackage <- FALSE
  return(result)
  
}

#' @rdname summary
#' @export
summary.symbolic_interval <- function(object, ...) {
  result <- NULL
  x <- object
  m1 <- min(x)
  m2 <- max(x)
  d <- data.frame(
    min_ = c(quantile(m1), mean(x), RSDA::sd(x)),
    max_ = c(quantile(m2), mean(x), RSDA::sd(x))
  )
  
  d2 <- classic2sym(
    d,
    groupby = "customize",
    minData = d$min_,
    maxData = d$max_
  )
  myNames <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max.", "Std.")
  if (pkg.env$inPackage) {
    result <- as.data.frame(d2$intervalData[c(1:3, 6, 4:5, 7), ])
    rownames(result) <- myNames
    class(result) <- c("symbolic_tbl", class(result))
  } else{
    result <- c(d2$intervalData[c(1:3, 6, 4:5, 7), ])[[1]]
    names(result) <- myNames
  }
  
  return(result)
  #summary.symbolic_modal(object, ...) works
}

#' @rdname summary
#' @export
summary.symbolic_modal <- function(object, summary_fun = "mean", ...) {
  x <- object
  d <- data.frame(NULL)
  for (i in 1:length(x)) {
    d <- rbind(d, x[[i]]$prop)
  }
  colnames(d) <- x[[1]]$var
  result <- round(apply(d, 2, eval(parse(text = summary_fun))), 2)
  if (pkg.env$inPackage) {
    if (!is.null(dim(result))) {
      result <- round(apply(d, 2, mean, 2))
      warning("Dimension Error in summary_fun input. Autoadjust to mean summary.")
    }
    result <- paste(names(result), sprintf(result, fmt = "%.2f"), sep = ": ")
    result <- noquote(cbind(result))
  }
  #colnames(result) <- deparse(as.list(match.call())$object)
  return(result)
}
pkg.env <- new.env()
pkg.env$inPackage <- FALSE

make_test_intervals <- function() {
  classic2sym(
    data.frame(
      x_min = c(1, 2, 3, 4),
      x_max = c(3, 5, 6, 8),
      y_min = c(2, 1, 6, 7),
      y_max = c(5, 4, 8, 10)
    ),
    groupby = "customize",
    minData = data.frame(
      x = c(1, 2, 3, 4),
      y = c(2, 1, 6, 7)
    ),
    maxData = data.frame(
      x = c(3, 5, 6, 8),
      y = c(5, 4, 8, 10)
    )
  )$intervalData
}

cov_bg_expected <- function(x, y) {
  n <- length(x)
  sum((min(x) + max(x)) * (min(y) + max(y))) / (4 * n) -
    (sum(min(x) + max(x)) * sum(min(y) + max(y))) / (4 * n^2)
}

cov_bd_expected <- function(x, y) {
  n <- length(x)
  x_center_mean <- mean(x, method = "centers")
  y_center_mean <- mean(y, method = "centers")

  gj <- function(a, b, center_mean) {
    ifelse((a + b) / 2 <= center_mean, -1, 1)
  }

  qj <- function(a, b, center_mean) {
    (a - center_mean)^2 +
      (a - center_mean) * (b - center_mean) +
      (b - center_mean)^2
  }

  sum(
    gj(min(x), max(x), x_center_mean) *
      gj(min(y), max(y), y_center_mean) *
      sqrt(
        qj(min(x), max(x), x_center_mean) *
          qj(min(y), max(y), y_center_mean)
      )
  ) / (3 * n)
}

cov_b_expected <- function(x, y) {
  n <- length(x)
  x_bar <- mean(x)
  y_bar <- mean(y)

  a <- 2 * (min(x) - x_bar) * (min(y) - y_bar)
  b <- (min(x) - x_bar) * (max(y) - y_bar)
  c <- (max(x) - x_bar) * (min(y) - y_bar)
  d <- 2 * (max(x) - x_bar) * (max(y) - y_bar)

  sum(a + b + c + d) / (6 * n)
}

sd_billard_expected <- function(x) {
  sqrt((1 / (3 * length(x))) * sum(min(x)^2 + min(x) * max(x) + max(x)^2) -
         (1 / (4 * length(x)^2)) * sum(min(x) + max(x))^2)
}

test_that("symbolic covariance methods match their published formulas", {
  intervals <- make_test_intervals()
  x <- intervals[[1]]
  y <- intervals[[2]]

  expect_equal(cov(x, y, method = "centers"), stats::cov((min(x) + max(x)) / 2, (min(y) + max(y)) / 2))
  expect_equal(cov(x, y, method = "BD"), cov_bd_expected(x, y))
  expect_equal(cov(x, y, method = "BG"), cov_bg_expected(x, y))
  expect_equal(cov(x, y, method = "B"), cov_b_expected(x, y))
})

test_that("symbolic correlation methods use the matching covariance definitions", {
  intervals <- make_test_intervals()
  x <- intervals[[1]]
  y <- intervals[[2]]

  expect_equal(
    cor(x, y, method = "BD"),
    cov_bd_expected(x, y) / (sd_billard_expected(x) * sd_billard_expected(y))
  )
  expect_equal(
    cor(x, y, method = "BG"),
    cov_bg_expected(x, y) / sqrt(cov_bg_expected(x, x) * cov_bg_expected(y, y))
  )
  expect_equal(
    cor(x, y, method = "B"),
    cov_b_expected(x, y) / (sd_billard_expected(x) * sd_billard_expected(y))
  )
})

test_that("symbolic standard deviation defaults to the Billard formula", {
  intervals <- make_test_intervals()
  x <- intervals[[1]]

  expect_equal(sd(x), sd_billard_expected(x))
  expect_equal(sd(x, method = "B"), sd_billard_expected(x))
  expect_equal(sd(x, method = "BD"), sd_billard_expected(x))
  expect_equal(sd(x, method = "BG"), sqrt(cov_bg_expected(x, x)))
})

test_that("symbolic self-correlations are equal to one", {
  intervals <- make_test_intervals()
  x <- intervals[[1]]

  expect_equal(cor(x, x, method = "centers"), 1)
  expect_equal(cor(x, x, method = "B"), 1)
  expect_equal(cor(x, x, method = "BD"), 1)
  expect_equal(cor(x, x, method = "BG"), 1)
})

test_that("correlation heatmap values stay within [-1, 1] and keep a unit diagonal", {
  intervals <- make_test_intervals()
  plot_obj <- ggInterval_corrplot(intervals, method = "BG", triangle = "full")
  plot_data <- ggplot2::ggplot_build(plot_obj)$plot$data

  expect_true(all(plot_data$Correlation >= -1))
  expect_true(all(plot_data$Correlation <= 1))

  diag_data <- plot_data[plot_data$Var1 == plot_data$Var2, , drop = FALSE]
  expect_true(all(diag_data$Correlation == 1))
})

test_that("triangle selection matches the displayed lower and upper halves", {
  intervals <- make_test_intervals()

  lower_data <- ggplot2::ggplot_build(
    ggInterval_corrplot(intervals, method = "BG", triangle = "lower")
  )$plot$data
  upper_data <- ggplot2::ggplot_build(
    ggInterval_corrplot(intervals, method = "BG", triangle = "upper")
  )$plot$data

  expect_true(all(lower_data$i <= lower_data$j))
  expect_true(all(upper_data$i >= upper_data$j))
})

#' @name ggInterval_hist
#' @title Histogram for symbolic data with equal-bin or unequal-bin.
#' @description  Visualize the continuous variable distribution
#' by dividing the x axis into bins,and calculating the frequency
#' of observation interval in each bin.
#' @import rlang ggplot2 tidyverse
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object.It can also be either RSDA object or
#' classical data frame,which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' It is the same as the mapping of ggplot2.
#' @param method It can be \code{"equal-bin"} (default) or
#' \code{"unequal-bin"}. Equal-bin means the histogram uses equally spaced
#' intervals. Unequal-bin means the bin widths are determined by the data,
#' so the argument \code{bins} is unused in that case.
#' @param bins x axis bins,which mean how many partials the variable
#' @param plotAll boolean, whether plot all variables, default FALSE.
#' will be separate into.
#' @param position "stack" or "identity"
#' @param alpha fill alpha
#' @return An object of class \code{ggInterval_hist_result}. A direct call
#' displays the histogram, and the returned object stores the ggplot object in
#' \code{$plot} together with the accompanying frequency tables.
#' @usage ggInterval_hist(data = NULL,mapping = aes(NULL),method="equal-bin",bins=10,
#'  plotAll = FALSE, position = "identity", alpha = 0.5)
#' @examples
#' mydata <- ggInterval::facedata
#' ggInterval_hist(mydata, aes(x = AD), bins = 10)
#'
#' hist_obj <- ggInterval_hist(mydata, plotAll = TRUE, bins = 10)
#' hist_obj
#' hist_obj$`Table AD`
#'
#' ggInterval_hist(mydata, aes(x = AD), method = "unequal-bin")
#'
#' @export
ggInterval_hist <- function(data = NULL,
                            mapping = aes(NULL),
                            method = "equal-bin",
                            bins = 10,
                            plotAll = FALSE,
                            position = "identity",
                            alpha = 0.5) {
  #data preparing
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  this.fill <- args$fill
  this.group <- args$group
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  if (plotAll) {
    if (!is.null(this.x) | !is.null(this.y)) {
      warning("Using plotAll presentation cannot specify variables.")
    }
  } else{
    testXY(iData, this.x, this.y)
  }
  p <- dim(iData)[2]
  n <- dim(iData)[1]
  resultSet <- NULL
  base <- NULL
  
  #test figure clearly
  if (method == "equal-bin" & bins > 200 & bins <= 1000) {
    warning("Bins are too large that will lead the figure to be fuzzy.")
  } else if (method == "equal-bin" & bins > 1000) {
    stop("Bins are too large that will lead the figure to be fuzzy.")
  }
  
  # group for hist adjust
  this.fill <- eval(this.fill)
  this.group <- eval(this.group)
  isGroup <- FALSE
  myGroup <- 1 # no group (initial)
  fill.group <- rep(1, n) # no group(initial)
  if (length(this.group) == n | length(this.fill) == n) {
    if (method == "equal-bin") {
      isGroup <- TRUE
      if (length(this.fill) == n) {
        myGroup <- unique(this.fill)
        fill.group <- this.fill
      } else{
        myGroup <- unique(this.group)
        fill.group <- this.group
      }
    } else{
      warning("Ignore group of concepts, since the method is not support for group.")
    }
  }
  
  
  if ("ggInterval" %in% class(data)) {
    data <- iData
  }
  #start process
  with(data, {
    plotVarNum <- NULL
    mmd <- data.frame(NULL) #min and max data frame
    if (plotAll) {
      #get numerical data
      numericData <- unlist(lapply(data.frame(iData[1:dim(iData)[2]]) , FUN = is.sym.interval))
      iData <- iData[, which(numericData)]
      attr <- colnames(iData)
      
      #adjust for group
      for (i in 1:length(attr)) {
        for (g in myGroup) {
          select.row <- which(fill.group == g)
          temp <- data.frame(
            minimal = min(iData[select.row, i][[1]]$min),
            maximal = max(iData[select.row, i][[1]]$max),
            fill.group = g,
            this.attr = i
          )
          mmd <- rbind(mmd, temp)
        }
      }
    } else{
      #get attr
      if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.x)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.x)
        )))
        attr <- names(attr)
      } else if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.y)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.y)
        )))
        attr <- names(attr)
      } else{
        stop("ERROR : Cannot find variables in aes(...)")
      }
      if (p == 1) {
        attr = colnames(data)
      }
      
      #test attribute illegal
      if (all(!is.numeric(data[[attr]]) , !RSDA::is.sym.interval(data[[attr]]))) {
        stop("ERROR : Variables in histogram can only be numeric.")
      }
      
      for (g in myGroup) {
        select.row <- which(fill.group == g)
        temp <- data.frame(
          minimal = min(iData[select.row, attr][[1]]$min),
          maximal = max(iData[select.row, attr][[1]]$max),
          fill.group = g,
          this.attr = 1
        )
        mmd <- rbind(mmd, temp)
      }
    }#end plotAll
    
    #unequal-bin cannot grouping
    if (method == "unequal-bin") {
      plotData <- NULL
      intervalDf = NULL
      for (u in 1:nrow(mmd)) {
        d <- as.data.frame(iData[[attr[u]]])
        temp <- seq(mmd[u, "minimal"], mmd[u, "maximal"], 0.01)
        f <- matrix(0, nrow = length(temp), ncol = n + 1)
        f[, 1] <- temp
        
        #calculate fraquency
        for (i in 1:n) {
          #start debug version0825-10
          #raw code :
          #start<-which(round(f[,1],2)==round(d[i,1],2))
          #end<-which(round(f[,1],2)==round(d[i,2],2))
          #new code :
          start <- length(which(round(f[, 1], 2) <= round(d[i, 1], 2)))
          end <- length(which(round(f[, 1], 2) <= round(d[i, 2], 2)))
          #end debug
          
          f[start:end, i + 1] <- 1
        }
        
        f <- cbind(temp, apply(f[, -1], 1, FUN = sum))
        
        x <- f[1, 2]
        y <- 1
        for (i in 1:(dim(f)[1] - 1)) {
          if (f[i, 2] != f[i + 1, 2]) {
            y <- append(y, f[i + 1, 2])
            x <- append(x, i)
            x <- append(x, i + 1)
          }
        }
        x <- append(x, length(temp))
        x <- t(matrix(x, nrow = 2))
        x1 <- f[x[, 1], 1]
        x2 <- f[x[, 2], 1]
        plotData <- rbind(plotData, data.frame(
          x1,
          x2,
          y1 = 0,
          y2 = y,
          group = factor(attr[u])
        ))
        
        tempDf <- data.frame(start = rep(NA, length(x1)),
                             end = rep(NA, length(x1)))
        tempDf[1:length(x1), "start"] <- round(x1, 2)
        tempDf[1:length(x1), "end"] <- round(x2, 2)
        intervalDf <- rbind(intervalDf, tempDf)
      }
      
      intervalDf <- intervalDf[order(intervalDf$start), ]
      temp <- paste(intervalDf$start, intervalDf$end, sep = ":")
      nameList <- paste0("[", temp, "]")
      
      midP <- (plotData$x1 + plotData$x2) / 2
      #build Aesthetic (mapping)
      xyLocation <- c(which(names(mapping) == "x"), which(names(mapping) == "y"))
      if (length(xyLocation) != 0) {
        usermapping <- mapping[-xyLocation] #Aesthetic without x,y
      } else{
        usermapping <- mapping
      }
      mymapping <- list(
        mapping = aes(
          xmin = x1,
          xmax = x2,
          ymin = y1,
          ymax = y2
        ),
        fill = "grey",
        col = "black",
        linewidth = 0.2,
        alpha = alpha
      )
      allmapping <- as.list(structure(as.expression(c(
        usermapping, mymapping
      )), class = "uneval"))
      
      #plot
      base <- ggplot(data = plotData, aes(x1, y1)) +
        do.call(geom_rect, allmapping) +
        theme(axis.text.x = element_text(
          angle = 90,
          vjust = 0.5,
          hjust = 1
        ))
      
      if (plotAll) {
        base <- base + facet_grid(group ~ .) +
          labs(x = "", y = "frequency") +
          scale_x_continuous(n.breaks = 8)
      } else{
        base <- base + labs(x = attr, y = "frequency") +
          scale_x_continuous(breaks = midP, labels = nameList)
      }
      
      #make table
      for (var in unique(plotData$group)) {
        this.myhist <- dplyr::filter(plotData, group == var)
        this.nameList <- paste0("[",
                                paste(this.myhist$x1, this.myhist$x2, sep = ":"),
                                "]")
        myName <- paste0("Table ", var)
        resultSet[[myName]] <- data.frame(
          "Interval" = unique(this.nameList),
          "Cumulative Concepts" = this.myhist$y2
        )
        
      }
      
      resultSet[["plot"]] <- base
      return(as_ggInterval_hist_result(resultSet))
      
    } else if (method == "equal-bin") {
      myhist <- NULL
      intervalDf <- NULL
      
      #adjust for groups
      #mmd : ith row : the ith variable's minimal and maximal
      for (i in 1:nrow(mmd)) {
        #adjust for group
        select.row <- which(mmd[i, "fill.group"] == fill.group)
        work.attr <- mmd[i, "this.attr"]
        mmd.temp <- dplyr::filter(mmd, this.attr == work.attr)
        #seperate the attribute into bins
        dist <- (max(mmd.temp[, "maximal"]) - min(mmd.temp[, "minimal"])) /
          bins
        #original:, till 2022 4 12
        interval <- seq(min(mmd.temp[, "minimal"]), max(mmd.temp[, "maximal"]), dist)
        
        # adjust interval: not finish yet
        # interval<-seq(min(mmd.temp[, "minimal"]) - dist/2,
        #               max(mmd.temp[, "maximal"]) + dist/2, dist)
        
        
        
        
        #test for ch3, p.84, figure3.3
        #interval <- c(0, 4, 7, 10, 13, 16, 19)
        #calculate frequency
        f <- NULL
        
        f <- matrix(nrow = nrow(iData[select.row, attr[work.attr]]), ncol =
                      bins)
        
        for (obs in 1:nrow(iData[select.row, attr[work.attr]])) {
          a <- iData[select.row, attr[work.attr]][[1]][obs]$min
          b <- iData[select.row, attr[work.attr]][[1]][obs]$max
          for (area in 1:bins) {
            headIn <- a %>% between(interval[area], interval[area + 1])
            tailIn <- b %>% between(interval[area], interval[area + 1])
            contain <- interval[area] %>% between(a, b)
            if (headIn | tailIn | contain) {
              temp <- sort(c(a, b, interval[area], interval[area + 1]))
              f[obs, area] <- (temp[3] - temp[2]) / (b - a)
            } else{
              f[obs, area] <- 0
            }
          }
        }
        
        nInt <- length(interval)
        interval <- round(interval, 2)
        tempDf <- data.frame(start = rep(NA, nInt - 1),
                             end = rep(NA, nInt - 1))
        tempDf[1:(nInt - 1), "start"] <- interval[1:(nInt - 1)]
        tempDf[1:(nInt - 1), "end"] <- interval[2:nInt]
        intervalDf <- rbind(intervalDf, tempDf)
        
        #build data frame and plot
        myhist <- rbind(
          myhist,
          data.frame(
            interval = interval[1:bins],
            frequency = apply(f, 2, FUN = sum),
            group = factor(attr[work.attr]),
            myFill = mmd[i, "fill.group"]
          )
        )
        
      }
      myhist <- cbind(myhist, intervalDf)
      myhist$myFill <- as.factor(myhist$myFill)
      
      intervalDf <- intervalDf[order(intervalDf$start), ]
      temp <- paste(intervalDf$start, intervalDf$end, sep = ":")
      #temp<-mapply(intervalDf$start,intervalDf$end,FUN=function(x,y) paste(interval[x],interval[y],sep = ":"))
      nameList <- paste0("[", temp, "]")
      
      # if(!isGroup){
      #   myhist$frequency <- myhist$frequency/n
      # }
      
      #build Aesthetic (mapping)
      #
      xyLocation <- c(which(names(mapping) == "x"), which(names(mapping) == "y"))
      if (length(xyLocation) != 0) {
        usermapping <- mapping[-xyLocation] #Aesthetic without x,y
      } else{
        usermapping <- mapping
      }
      
      
      mymapping <- list(
        stat = "identity",
        mapping = aes(alpha = alpha, fill = gray.colors(bins *
                                                          length(attr))),
        col = "black"
      )
      
      
      allmapping <- as.list(structure(as.expression(c(
        usermapping, mymapping
      )), class = "uneval"))
      
      
      #plot
      if (plotAll) {
        if (isGroup) {
          if (position == "identity") {
            base <- ggplot(data = myhist, aes(fill = myhist$myFill)) +
              geom_rect(
                mapping = aes(
                  xmin = start,
                  xmax = end,
                  ymin = 0,
                  ymax = frequency
                ),
                col = "black",
                alpha = alpha
              ) +
              theme(axis.text.x = element_text(
                angle = 90,
                vjust = 0.5,
                hjust = 1
              )) +
              labs(x = "",
                   fill = "Group",
                   y = "Frequency") +
              scale_x_continuous(n.breaks = 8) +
              guides(colour = "none", alpha = "none") +
              facet_grid(group ~ .)
          } else if (position == "stack") {
            myhist <- stackFreq(myhist, myGroup)
            base <- ggplot(data = myhist, aes(fill = myhist$myFill)) +
              geom_rect(
                mapping = aes(
                  xmin = start,
                  xmax = end,
                  ymin = ystart,
                  ymax = yend
                ),
                col = "black",
                alpha = alpha
              ) +
              theme(axis.text.x = element_text(
                angle = 90,
                vjust = 0.5,
                hjust = 1
              )) +
              labs(x = "",
                   fill = "Group",
                   y = "Frequency") +
              scale_x_continuous(n.breaks = 8) +
              guides(colour = "none", alpha = "none") +
              facet_grid(group ~ .)
          } else{
            stop("The position is not implemented.")
          }
          
          #make table
          for (var in unique(myhist$group)) {
            this.var.myhist <- dplyr::filter(myhist, group == var)
            this.nameList <- paste0("[",
                                    paste(this.var.myhist$start, this.var.myhist$end, sep = ":"),
                                    "]")
            for (i in unique(myhist$myFill)) {
              this.myhist <- dplyr::filter(this.var.myhist, myFill == i)
              myName <- paste("Table ", paste(unique(this.myhist$group), i, sep = ": "))
              resultSet[[myName]] <- data.frame(
                "Interval" = unique(this.nameList),
                "Observed.Frequency" = this.myhist$frequency,
                "Relative.Frequency" = this.myhist$frequency /
                  sum(this.myhist$frequency)
              )
            }
          }
        } else{
          mymapping <- list(
            mapping = aes(
              xmin = start,
              xmax = end,
              ymin = 0,
              ymax = frequency
            ),
            fill = "grey",
            col = "black"
          )
          allmapping <- as.list(structure(as.expression(c(
            usermapping, mymapping
          )), class = "uneval"))
          base <- ggplot(data = myhist) +
            do.call(geom_rect, allmapping) +
            facet_grid(group ~ .) +
            labs(x = "", y = "frequency") +
            scale_x_continuous(n.breaks = 8)
          
          #make table
          for (var in unique(myhist$group)) {
            this.myhist <- dplyr::filter(myhist, group == var)
            this.nameList <- paste0("[",
                                    paste(this.myhist$start, this.myhist$end, sep = ":"),
                                    "]")
            myName <- paste0("Table ", var)
            resultSet[[myName]] <- data.frame(
              "Interval" = unique(this.nameList),
              "Observed Frequency" = this.myhist$frequency,
              "Relative Frequency" = this.myhist$frequency /
                sum(this.myhist$frequency)
            )
            
          }
        }
      } else{
        #not plot all
        if (isGroup) {
          if (position == "identity") {
            base <- ggplot(data = myhist, aes(fill = myhist$myFill)) +
              geom_rect(
                mapping = aes(
                  xmin = start,
                  xmax = end,
                  ymin = 0,
                  ymax = frequency
                ),
                col = "black",
                alpha = alpha
              ) +
              theme(axis.text.x = element_text(
                angle = 90,
                vjust = 0.5,
                hjust = 1
              )) +
              labs(x = attr,
                   fill = "Group",
                   y = "Frequency") +
              scale_x_continuous(breaks = (myhist[myhist$myFill == myGroup[1], "start"] + myhist[myhist$myFill == myGroup[1], "end"]) /
                                   2,
                                 labels = unique(nameList)) +
              guides(colour = "none", alpha = "none")
          } else if (position == "stack") {
            myhist <- stackFreq(myhist, myGroup)
            base <- ggplot(data = myhist, aes(fill = myhist$myFill)) +
              geom_rect(
                mapping = aes(
                  xmin = start,
                  xmax = end,
                  ymin = ystart,
                  ymax = yend
                ),
                col = "black",
                alpha = alpha
              ) +
              theme(axis.text.x = element_text(
                angle = 90,
                vjust = 0.5,
                hjust = 1
              )) +
              labs(x = attr,
                   fill = "Group",
                   y = "Frequency") +
              scale_x_continuous(breaks = (myhist[myhist$myFill == myGroup[1], "start"] + myhist[myhist$myFill == myGroup[1], "end"]) /
                                   2,
                                 labels = unique(nameList)) +
              guides(colour = "none", alpha = "none")
          } else{
            stop("The position is not implemented.")
          }
          #make table
          for (i in unique(myhist$myFill)) {
            this.myhist <- dplyr::filter(myhist, myFill == i)
            myName <- paste("Table ", paste(unique(this.myhist$group), i, sep = ": "))
            resultSet[[myName]] <- data.frame(
              "Interval" = unique(nameList),
              "Observed Frequency" = this.myhist$frequency,
              "Relative Frequency" = this.myhist$frequency /
                sum(this.myhist$frequency)
            )
          }
          
        } else{
          base <- ggplot(data = myhist, aes(x = factor(interval), y = frequency)) +
            do.call(geom_histogram, allmapping) +
            theme(axis.text.x = element_text(
              angle = 90,
              vjust = 0.5,
              hjust = 1
            )) +
            labs(x = attr) +
            scale_x_discrete(labels = nameList) +
            scale_fill_manual(values = rep("black", bins * length(attr))) +
            guides(colour = "none",
                   alpha = "none",
                   fill = "none")
          
          #make table
          resultSet[[paste0("Table ", unique(myhist$group))]] <- data.frame(
            "Interval" = nameList,
            "Observed Frequency" = myhist$frequency,
            "Relative Frequency" = myhist$frequency /
              sum(myhist$frequency)
          )
          
        }
      }
      
      resultSet[["plot"]] <- base
      return(as_ggInterval_hist_result(resultSet))
      
    } else{
      stop(paste0("ERROR : Unrecognize method : ", method, " ."))
    }
  })
}

print.ggInterval_hist_result <- function(x, ...) {
  if (!is.null(x$plot) && inherits(x$plot, "ggplot")) {
    print(x$plot)
    return(invisible(x))
  }
  print(unclass(x))
  invisible(x)
}

as_ggInterval_hist_result <- function(x) {
  class(x) <- c("ggInterval_hist_result", "list")
  x
}


stackFreq <- function(data = NULL, myGroup = NULL) {
  n <- dim(data)[1] / length(unique(data$group)) / length(myGroup)
  final <- NULL
  for (var in unique(data$group)) {
    work.data <- dplyr::filter(data, data$group == var)
    times <- n
    temp <- 0
    for (i in (length(myGroup) - 1):1) {
      temp <- temp + c(rep(0, times), work.data[1:(i * n), "frequency"])
      times <- times + n
    }
    final <- c(final, temp)
  }
  
  data[, "ystart"] <- final
  data[, "yend"] <- final + data$frequency
  return(data)
}

#' @name ggInterval_radarplot
#' @title Radar plot for symbolic data
#' @description Visualize interval-valued and mixed symbolic data with a
#' radar-style display. Interval-valued variables are represented through
#' their lower and upper bounds, while modal multivalued variables can be
#' shown with stacked bars. When \code{type = "quantile"}, the function
#' summarizes all observations of each interval-valued variable through
#' nested empirical quantile bands.
#' @import ggplot2 ggforce
#' @importFrom gridExtra marrangeGrob
#' @importFrom RSDA is.sym.interval
#' @importFrom tidyr separate
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame(not recommended),which will be automatically convert to ggInterval
#' data.
#' @param layerNumber Number of concentric guide layers in the radar display.
#' @param inOneFig Logical. If \code{TRUE}, draw all selected observations in
#' one figure. Otherwise, generate separate figures for the selected
#' observations.
#' @param showLegend Logical. Whether to show the legend.
#' @param showXYLabs Logical. Whether to show the x- and y-axis labels.
#' @param plotPartial Numeric vector giving the row indices to plot. If
#' \code{NULL}, all observations are used. This argument is ignored when
#' \code{type = "quantile"} because the quantile plot summarizes all
#' observations.
#' @param base_lty Line type used in the base figure.
#' @param base_circle Logical. If \code{TRUE}, add inner guide circles.
#' @param alpha Alpha transparency for filled elements.
#' @param addText Logical. Whether to add interval-valued labels to the plot.
#' @param type Radar representation. Use \code{"default"} for polygon
#' intervals, \code{"rect"} for rectangle intervals, or \code{"quantile"}
#' for the quantile-radar display.
#' @param quantileNum Number of quantile layers when
#' \code{type = "quantile"}.
#' @param Drift Drift term controlling where the radar values begin.
#' @param addText_modal Logical. Whether to add labels for modal
#' multivalued variables.
#' @param addText_modal.p Logical. Whether to add modal percentages.
#' @usage ggInterval_radarplot(data=NULL,layerNumber=3,
#' inOneFig=TRUE,showLegend=TRUE,showXYLabs=FALSE,
#' plotPartial=NULL,
#' alpha=0.5,
#' base_circle=TRUE,
#' base_lty=2,
#' addText=TRUE,
#' type="default",
#' quantileNum=4,
#' Drift=0.5,
#' addText_modal=TRUE,
#' addText_modal.p=FALSE)
#' @examples
#' \donttest{
#' # must specify plotPartial to the rows you want to plot
#' Environment.n <- Environment[, 5:17]
#' ggInterval_radarplot(Environment.n,
#'   plotPartial = 2,
#'   showLegend = FALSE,
#'   base_circle = TRUE,
#'   base_lty = 2,
#'   addText = FALSE
#') +
#'   labs(title = "") +
#'   scale_fill_manual(values = c("gray50")) +
#'   scale_color_manual(values = c("red"))
#'
#' ggInterval_radarplot(Environment,
#'   plotPartial = 2,
#'   showLegend = FALSE,
#'   base_circle = FALSE,
#'   base_lty = 1,
#'   addText = TRUE,
#'   type = "rect"
#') +
#'   labs(title = "") +
#'   scale_fill_manual(values = c("gray50")) +
#'   scale_color_manual(values = c("gray50"))
#'
#' ggInterval_radarplot(
#'   facedata,
#'   base_circle = FALSE,
#'   base_lty = 1,
#'   type = "quantile",
#'   quantileNum = 5,
#'   showLegend = TRUE,
#'   Drift = 0
#' ) +
#'   scale_fill_brewer(palette = "Greys") +
#'   labs(title = "", fill = "Quantiles")
#' }
#'
#' @export
ggInterval_radarplot <- function(data = NULL,
                             layerNumber = 3,
                             inOneFig = TRUE,
                             showLegend = TRUE,
                             showXYLabs = FALSE,
                             plotPartial = NULL,
                             alpha = 0.5,
                             base_circle = TRUE,
                             base_lty = 2,
                             addText = TRUE,
                             type = "default",
                             quantileNum = 4,
                             Drift = 0.5,
                             addText_modal = TRUE,
                             addText_modal.p = FALSE) {
  #globalVariables("varLevels", add = F)
  #utils::globalVariables(c("varLevels"), add=FALSE)
  varLevels <- NULL
  extendUnit <- Drift
  if (extendUnit < 0 | extendUnit > 1) {
    stop("Drift must between 0 to 1.")
  }
  
  #extend plot original xyLimits = 1.25, extendUnit=0, colnamesDrift = 0.1
  colnamesDrift <- 0.18
  xyLimits <- extendUnit + 1.25 + colnamesDrift
  textShift <- 0.065
  textSize <- 4
  textCol <- "gray20"
  #
  
  fillBetween = TRUE #not fix complete
  #notes
  if (dim(data)[1] <= 1) {
    stop("ERROR IN data : data must be full data with observations larger than 1.")
  }
  #setting parameter
  nL <- layerNumber
  if (is.null(plotPartial)) {
    #test data to iData
    ggSymData <- testData(data)
    iData <- ggSymData$intervalData
    indNum <- dim(iData)[1]
    plotPartial <- c(1:indNum)
  } else{
    #checking input format
    if (!"symbolic_tbl" %in% class(data)) {
      stop("ERROR : data must be symbolic data,when plotPartial is on.")
    }
    if (!is.numeric(plotPartial)) {
      stop(
        "ERROR IN plotPartial : the input data must be numeric vector as the row index in full data."
      )
    }
    if (!all(plotPartial <= nrow(data)) || !all(plotPartial > 0)) {
      stop(
        "ERROR IN plotPartial : the input data must be numeric vector as the row index in full data."
      )
    }
    iData <- data
    indNum <- length(plotPartial)
  }
  
  if (!(type %in% c("default", "rect", "quantile"))) {
    warning(paste(
      "There is no type called ",
      type,
      ", automatically set default type."
    ))
    type <- "default"
  }
  
  #data preprocessing
  numericData <- unlist(lapply(iData[, 1:dim(data)[2]] , FUN = RSDA::is.sym.interval))
  
  rawiData <- iData
  allnP <- dim(rawiData)[2]
  propData <- iData[, !numericData]
  iData <- iData[, numericData]
  if ("symbolic_tbl" %in% class(iData)) {
    if (dim(iData)[2] == 0) {
      #all data are symbolic modal
      nP <- 0
    } else{
      #normal type , there are some symbolic interval data (len>1)
      nP <- dim(iData)[2]
    }
  } else{
    #only one symbolic interval data
    nP <- 1
    iData <- rawiData[, c(which(numericData), which(!numericData))]
  }
  
  #someday i will fix this bug ,maybe
  # if(nP < 2){
  #   stop("number of numerical (symbolic interval) data must greater than 2")
  # }
  
  
  #setting quantile argument
  if (type == "quantile") {
    #warning("inOneFig will be unused in type=='quantile'.")
    inOneFig <- TRUE
    indNum <- dim(iData)[1]
    if (!quantileNum %in% c(2:10)) {
      stop("the number of quantile must be in the range [2,10].")
    }
    if (nP < 3) {
      stop("numerical variables must greater than 3 in type=='quantile'")
    }
    if (allnP != nP) {
      warning("nominal variables will be ignore in type=='quantile'")
      allnP <- nP
    }
    plotPartial <- NULL
  }
  
  #test ERROR
  if (type != "quantile") {
    if (indNum > 50 & inOneFig == F) {
      stop("Out of time limits")
    } else if (indNum > 10 & inOneFig == T) {
      stop("Suggest set inOneFig to FALSE when the observations are large.")
    } else if (indNum > 5 & inOneFig == T) {
      warning("Suggest set inOneFig to FALSE when the observations are large.")
    }
  }
  
  #setting original plot
  #p<-generateCircle(nL)+coord_fixed(ratio = 1)
  
  if (base_circle) {
    p <- generateCircle(nL + 1, extendUnit) + coord_fixed(ratio = 1)
  } else{
    p <- ggplot() + coord_fixed(ratio = 1)
  }
  
  if (allnP != nP) {
    d <- generatePoint(allnP, nL, extendUnit)
    d <- d[(allnP * nL - allnP + 1):(allnP * nL), ]
    rawiData <- cbind(iData, propData)
    propData <- propData[plotPartial, ]
  } else{
    rawiData <- iData
    d <- generatePoint(nP, nL, extendUnit)
    d <- d[(nP * nL - nP + 1):(nP * nL), ]
  }
  p <- p + geom_point(data = d, aes(x = d$x, y = d$y))
  #get all variables min max data
  if (nP != 0) {
    allData <- iData[, 1:nP]
    
    
    if (!is.null(plotPartial)) {
      iData <- iData[plotPartial, 1:nP]
    }
    dataList <- lapply(
      1:nP,
      FUN = function(x)
        data.frame(allData[[x]])
    )
    iDataList <- lapply(
      1:nP,
      FUN = function(x)
        data.frame(iData[[x]])
    )
    maxList <- lapply(
      1:nP,
      FUN = function(x)
        max(dataList[[x]])
    )
    minList <- lapply(
      1:nP,
      FUN = function(x)
        min(dataList[[x]])
    )
    minminList <- minList
    maxmaxList <- maxList
    
    #quantile #this will over write iDataList
    if (type == "quantile") {
      indNum <- quantileNum + 1
      figQuaNum <- quantileNum + 3
      sepQua <- seq(0, 1, 1 / figQuaNum)
      for (i in 1:nP) {
        datatmp <- c(iDataList[[i]]$min, iDataList[[i]]$max)
        #extend adjust
        datatmp <- round(quantile(datatmp, sepQua), 2)[-(figQuaNum + 1)][-1] + extendUnit
        #end adjust
        tempMin <- datatmp[1:indNum]
        tempMax <- datatmp[2:(figQuaNum - 1)]
        iDataList[[i]] <- data.frame(min = tempMin, max = tempMax)
      }
    }
    
    if (type == "quantile") {
      maxList <- lapply(
        1:nP,
        FUN = function(x)
          max(iDataList[[x]][, "max"])
      )
      minList <- lapply(
        1:nP,
        FUN = function(x)
          min(iDataList[[x]][, "min"])
      )
      minminList <- minList
      maxmaxList <- maxList
    }
    
    #normalize data to 0,1
    #extend adjust
    normData <- lapply(
      1:nP ,
      FUN = function(x) {
        sapply(
          iDataList[[x]],
          FUN = function(elem) {
            ((elem - minList[[x]]) / (maxList[[x]] - minList[[x]])) + extendUnit
          }
        )
      }
    )
    
    
    
    
    #rescale dataframe to input form which data2Vec need
    minDF <- sapply(
      1:nP,
      FUN = function(x)
        matrix(normData[[x]], ncol = 2)[, 1]
    )
    maxDF <- sapply(
      1:nP,
      FUN = function(x)
        matrix(normData[[x]], ncol = 2)[, 2]
    )
    
    minDF <- matrix(minDF, ncol = nP)
    maxDF <- matrix(maxDF, ncol = nP)
    transMatrix <- d[1:nP, c(3, 4)]
    
    
    
    #convert data to match radar axis scale
    if (type == "quantile") {
      setRowNameDf <- iDataList[[1]]
    } else{
      setRowNameDf <- iData
    }
    plotMin <- data2Vec(
      iData = setRowNameDf,
      data = minDF,
      transMat = transMatrix,
      type,
      quantileNum
    )
    plotMax <- data2Vec(
      iData = setRowNameDf,
      data = maxDF,
      transMat = transMatrix,
      type,
      quantileNum
    )
    
    
    #making labels
    temp <- lapply(
      1:nP,
      FUN = function(x)
        paste(iDataList[[x]][, 1], iDataList[[x]][, 2], sep = ":")
    )
    temp <- lapply(
      1:nP,
      FUN = function(x)
        paste0("[", temp[[x]], "]")
    )
    temp <- matrix(unlist(temp), ncol = indNum, byrow = T)
    groupId <- rep(c(1:indNum), each = nP)
    myLabel = paste(groupId, paste(colnames(iData), temp), sep = " : ")
    
    
    #new a labels variable
    plotMin <- data.frame(plotMin, Variables = myLabel)
    plotMax <- data.frame(plotMax, Variables = myLabel)
    
    #make text in plot
    # minList<-unlist(lapply(1:nP,FUN=function(x) iDataList[[x]][,1]))
    # maxList<-unlist(lapply(1:nP,FUN=function(x) iDataList[[x]][,2]))
    #
    newDf <- data.frame(NULL)
    for (i in 1:indNum) {
      for (u in 1:nP) {
        newDf <- rbind(newDf, iDataList[[u]][i, ])
      }
    }
    
    textMin <- as.data.frame(cbind(plotMin, min = newDf$min))
    textMax <- as.data.frame(cbind(plotMax, max = newDf$max))
    textMin$cos <- textMin$cos + textShift
    textMin$sin <- textMin$sin + textShift
    textMax$cos <- textMax$cos + textShift
    textMax$sin <- textMax$sin + textShift
    
    plotMin$group <- as.factor(plotMin$group)
  }#end if np != 0
  
  #calculate nominal variable (point)
  if (allnP != nP) {
    # preparing the nominal data set
    tmpDf <- data.frame(NULL)
    varNum <- 1
    for (var in colnames(propData)) {
      propDf <- data.frame(NULL)
      counter <- 1
      for (ele in 1:indNum) {
        varPro <- unlist(propData[ele, var])
        propDf[counter:(counter + (length(varPro) / 2) - 1), "varName"] <- rep(var, length(varPro) /
                                                                                 2)
        propDf[counter:(counter + (length(varPro) / 2) - 1), "varLevels"] <-
          paste(varPro[1:(length(varPro) / 2)], round(as.numeric(varPro[(length(varPro) /
                                                                           2 + 1):length(varPro)]), 2), sep = ":")
        propDf[counter:(counter + (length(varPro) / 2) - 1), "prop"] <- as.numeric(varPro[(length(varPro) /
                                                                                             2 + 1):length(varPro)])
        propDf[counter:(counter + (length(varPro) / 2) - 1), "groupid"] <- rep(ele, length(varPro) /
                                                                                 2)
        #extendUnit adjust  quantile(seq(0,1,1/3),seq(0,1,1/3))[2:(3+1)]
        #pos<-seq(0,1,1/((length(varPro)+2)/2))
        pos <- as.numeric(quantile(seq(0, 1 + extendUnit, 1 / ((length(varPro) +
                                                                  2) / 2
        )), seq(0, 1, 1 / ((length(varPro) + 2) / 2
        )))[2:(length(varPro) / 2 + 1)])
        #pos <- pos[-length(pos)][-1]
        #end adjust
        propDf[counter:(counter + (length(varPro) / 2) - 1), "x"] <- d[nP +
                                                                         varNum, 3] * pos
        propDf[counter:(counter + (length(varPro) / 2) - 1), "y"] <- d[nP +
                                                                         varNum, 4] * pos
        counter <- counter + length(varPro) / 2
      }
      tmpDf <- rbind(tmpDf, propDf)
      varNum <- varNum + 1
    }
    propDf <- as.data.frame(tmpDf)
    propDf$groupid <- as.factor(propDf$groupid)
    
    
    # preparing the symbolic(rectangle) to represent nominal data
    totalRectDf <- data.frame(NULL)
    obsG <- 1
    #lastRect <- NULL
    heiList <- list(NULL)
    for (i in 1:indNum) {
      heiList[[i]] <- 0
    }
    for (ele in levels(propDf$groupid)) {
      propDf.ele <- dplyr::filter(propDf, propDf$groupid == ele)
      varG <- 1
      tempRectDf <- data.frame(NULL)
      for (i in unique(as.factor(propDf$varName))) {
        rectDf <- data.frame(NULL)
        propDf.temp <- dplyr::filter(propDf.ele, propDf.ele$varName == i)
        varL <- 1
        heiVec <- NULL
        for (u in 1:dim(propDf.temp)[1]) {
          #re range 0~1 to -0.9~0.5 (1 = 撟喲)
          #print(paste0(ele,i,u))
          #print(propDf.temp)
          thisHei <- reRange(-0.99, -0.8, propDf.temp[u, "prop"])
          heiVec <- cbind(heiVec, thisHei)
          rect <- build3DRect(-0.95, -0.95, thisHei, g = paste0(i, u, ele))
          #print(rect)
          distx <- mean(rect$newx - propDf.temp[u, "x"])
          disty <- mean(rect$newy[13:16] - propDf.temp[u, "y"])
          rect$newx <- rect$newx - distx
          rect$newy <- rect$newy - disty
          
          #add in height
          if (obsG >= 2 && inOneFig) {
            rect$newy <- rect$newy + unlist(heiList[[obsG - 1]][varG])[u]
          }
          
          rect[, "varLevels"] <- varL
          rectDf <- rbind(rectDf, rect)
          varL <- varL + 1
        }#inner for
        heiList[[obsG]][varG] <- list(1 + heiVec)
        if (obsG >= 2) {
          heiList[[obsG]][varG] <- list(unlist(heiList[[obsG - 1]][varG]) + unlist(heiList[[obsG]][varG]))
        }
        rectDf[, "varGroup"] <- varG
        tempRectDf <- rbind(tempRectDf, rectDf)
        varG <- varG + 1
      }# sec for
      tempRectDf[, "obsGroup"] <- obsG
      obsG <- obsG + 1
      # if(!is.null(lastRect)){
      #   tempRectDf$newy <- lastRect$newy+tempRectDf$newy
      # }
      totalRectDf <- rbind(totalRectDf, tempRectDf)
      #lastRect<-tempRectDf
      #print(lastRect)
    }# first for
    
    
    #totalRectDf<-as.data.frame(totalRectDf)
    totalRectDf$varLevels <- as.factor(totalRectDf$varLevels)
    totalRectDf$varGroup <- as.factor(totalRectDf$varGroup)
    totalRectDf$obsGroup <- as.factor(totalRectDf$obsGroup)
    if (nP != 0) {
      levels(propDf$groupid) <- levels(plotMin$group)
    }
  }
  
  #generate cut line for type==rect && build nominal rect
  if (type == "rect") {
    if (nP != 0) {
      cutDf <- c(
        getCutDf(plotMin, transMatrix, nP, indNum),
        getCutDf(plotMax, transMatrix, nP, indNum)
      )
    }
  }
  
  #plot
  if (inOneFig) {
    if (type == "default" || type == "quantile") {
      #if(fillBetween){
      myPolyData <- data.frame(NULL)
      if (nP != 0) {
        for (i in levels(as.factor(plotMin$group))) {
          plotMin.temp <- dplyr::filter(plotMin, plotMin$group == i)
          plotMax.temp <- dplyr::filter(plotMax, plotMin$group == i)
          myPathData <- data.frame(
            x1 = plotMin.temp$cos,
            y1 = plotMin.temp$sin,
            x2 = plotMax.temp$cos,
            y2 = plotMax.temp$sin
          )
          if (allnP == nP) {
            plotMin.rev <- plotMin.temp[nrow(plotMin.temp):1, ]
            newTemp <- rbind(plotMax.temp,
                             plotMax.temp[1, ],
                             plotMin.temp[1, ],
                             plotMin.rev)
            myPolyData <- rbind(myPolyData, newTemp)
          } else{
            #add nominal
            tmp2 <- NULL
            j <- 1
            # print(levels(as.factor(propDf$varName)))
            # print(propDf)
            for (u in unique(as.factor(propDf$varName))) {
              # propDf.temp <- dplyr::filter(propDf,propDf$varName==u) %>%
              #   dplyr::filter(groupid==i)
              
              propDf.temp <- dplyr::filter(propDf, propDf$varName == u)
              propDf.temp <- dplyr::filter(propDf.temp, propDf.temp$groupid ==
                                             i)
              tmp <- propDf.temp[propDf.temp$prop == max(propDf.temp$prop), ]
              tmp <- tmp[1, ]
              tmp2 <- rbind(tmp2, plotMin.temp[1, ])
              tmp2[j, c("cos", "sin")] <- c(tmp$x, tmp$y)
              myPathData <- rbind(myPathData,
                                  data.frame(
                                    x1 = tmp$x,
                                    y1 = tmp$y,
                                    x2 = tmp$x,
                                    y2 = tmp$y
                                  ))
              j <- j + 1
            }
            #propDf.temp <- dplyr::filter(propDf,groupid==i)
            #tmp<-propDf[propDf.temp$prop==max(propDf.temp$prop),]
            #tmp<-tmp[1,] #????銝甈⊿?????瘜? ????洵銝??
            #tmp2<-plotMin.temp[1,]
            #tmp2[,c("cos","sin")]<-c(tmp$x,tmp$y)
            #myPathData<-rbind(myPathData,data.frame(x1=tmp$x,y1=tmp$y,x2=tmp$x,y2=tmp$y))
            
            plotMin.rev <- plotMin.temp[nrow(plotMin.temp):1, ]
            tmp2.rev <- tmp2[nrow(tmp2):1, ]
            newTemp <- rbind(plotMax.temp,
                             tmp2,
                             plotMax.temp[1, ],
                             plotMin.temp[1, ],
                             tmp2.rev,
                             plotMin.rev)
            myPolyData <- rbind(myPolyData, newTemp)
          }
          p <- p + geom_path(data = myPathData,
                             aes(x = myPathData$x1, y = myPathData$y1),
                             lty = 0) +
            geom_path(data = myPathData,
                      aes(x = myPathData$x2, y = myPathData$y2),
                      lty = 0)
        }
      }
      
      if (allnP != nP) {
        if (nP != 0) {
          tmpN <- dim(myPolyData)[1] / indNum
          myPolyData[, "obsGroup"] <- as.factor(rep(1:indNum, each = tmpN))
          tempPolyDf <- myPolyData[, c(1, 2, 3, 5)]
        }
        
        tempRectDf <- totalRectDf[, c(1, 2, 3, 6)]
        
        colnames(tempRectDf) <- c("cos", "sin", "group", "obsGroup")
        #colnames(tempRectDf)<-colnames(tempPolyDf)
        if (nP != 0) {
          polyDf <- as.data.frame(rbind(tempPolyDf, tempRectDf))
        } else{
          polyDf <- tempRectDf
        }
        
        p <- p + geom_polygon(
          data = polyDf,
          aes(
            x = polyDf$cos,
            y = polyDf$sin,
            group = polyDf$group,
            fill = polyDf$obsGroup,
            col = polyDf$obsGroup
          ),
          alpha = alpha
        )
        if (nP != 0) {
          p <- p +  geom_point(data = plotMin, aes(x = plotMin$cos, y = plotMin$sin)) +
            geom_point(data = plotMax, aes(x = plotMax$cos, y = plotMax$sin))
        }
        
      } else{
        if (type == "quantile") {
          p <- p + geom_polygon(
            data = myPolyData,
            aes(
              x = myPolyData$cos,
              y = myPolyData$sin,
              fill = myPolyData$group,
              col = myPolyData$group
            ),
            alpha = alpha
          )
        } else{
          p <- p + geom_polygon(
            data = myPolyData,
            aes(
              x = myPolyData$cos,
              y = myPolyData$sin,
              fill = myPolyData$group,
              col = myPolyData$group
            ),
            alpha = alpha,
            lty = 0
          )
        }
        
        if (type != "quantile") {
          p <- p + geom_point(data = plotMin, aes(x = plotMin$cos, y = plotMin$sin)) +
            geom_point(data = plotMax, aes(x = plotMax$cos, y = plotMax$sin))
        }
        
      }
      
    }
    #else{ #this is else for if(fillbetween)
    else if (type == "rect") {
      #add cut segment
      if (nP != 0) {
        rectPolyData <- data.frame(NULL)
        gId <- 1
        for (g in levels(as.factor(cutDf[[1]]$obsGroup))) {
          cutDf.temp <- lapply(
            1:4,
            FUN = function(x) {
              as.data.frame(dplyr::filter(cutDf[[x]], cutDf[[x]]$obsGroup == g))
            }
          )
          #print(cutDf.temp)
          for (i in 1:nP) {
            tmpDf <- data.frame(
              x = c(
                cutDf.temp[[1]]$x2[i],
                cutDf.temp[[2]]$x2[i],
                cutDf.temp[[4]]$x2[i],
                cutDf.temp[[3]]$x2[i]
              ),
              y = c(
                cutDf.temp[[1]]$y2[i],
                cutDf.temp[[2]]$y2[i],
                cutDf.temp[[4]]$y2[i],
                cutDf.temp[[3]]$y2[i]
              ),
              varGroup = as.factor(rep(gId, 4)),
              obsGroup = as.factor(g)
            )
            rectPolyData <- rbind(rectPolyData, tmpDf)
            gId <- gId + 1
          }
        }
      }
      #print(cutDf)
      #print(rectPolyData)
      #numeric
      if (allnP != nP) {
        tempRectDf <- totalRectDf[, c(1, 2, 3, 6)]
        #colnames(tempRectDf) <- colnames(rectPolyData)
        colnames(tempRectDf) <- c("x", "y", "varGroup", "obsGroup")
        if (nP != 0) {
          levels(tempRectDf$obsGroup) <- levels(rectPolyData$obsGroup)
          rectPolyData <- as.data.frame(rbind(tempRectDf, rectPolyData))
        } else{
          rectPolyData <- tempRectDf
        }
        
      }
      p <- p + geom_polygon(
        data = rectPolyData,
        aes(
          x = .data$x,
          y = .data$y,
          group = .data$varGroup,
          fill = .data$obsGroup,
          col = .data$obsGroup
        ),
        alpha = alpha
      )
      
      
    }
    #nominal
    if (allnP != nP) {
      newPropDf <- as.data.frame(tidyr::separate(
        propDf,
        varLevels,
        into = c("varLevels", "gar"),
        sep = ":",
        convert = F
      ))
      newPropDf <- as.data.frame(dplyr::filter(
        newPropDf,
        newPropDf$groupid == levels(newPropDf$groupid)[1]
      ))
      
      if (addText_modal) {
        p <- p + geom_text(
          data = newPropDf,
          aes(
            x = newPropDf$x + textShift,
            y = newPropDf$y + textShift,
            label = newPropDf$varLevels
          ),
          vjust = 2.15,
          hjust = 0.5,
          size = textSize,
          col = textCol
        )
      }
      #print(totalRectDf)
      
      if (addText_modal.p) {
        tempd <- data.frame(NULL)
        for (i in unique(totalRectDf$varGroup)) {
          temp.varG <- dplyr::filter(totalRectDf, totalRectDf$varGroup == i)
          for (u in levels(temp.varG$obsGroup)) {
            temp.obsG <- dplyr::filter(temp.varG, temp.varG$obsGroup == u)
            for (k in unique(temp.obsG$varLevels)) {
              temp.varL <- dplyr::filter(temp.obsG, temp.obsG$varLevels == k)
              tempd <- rbind(tempd, data.frame(
                x = mean(temp.varL$newx),
                y = mean(temp.varL$newy)
              ))
            }
          }
        }
        propTextDf <- data.frame(tempd, prop = propDf$prop)
        #print(propTextDf)
        p <- p + geom_text(
          data = propTextDf,
          aes(
            x = propTextDf$x + textShift,
            y = propTextDf$y,
            label = round(propTextDf$prop, 2)
          ),
          size = textSize,
          col = textCol
        )
      }
    }
    #print("5")
    # if(allnP!=nP){
    #   p<-p+geom_polygon(data=totalRectDf,aes(x=newx,y=newy,group=group,fill=obsGroup),
    #                     col="black",
    #                     alpha=alpha)+
    #     geom_text(data=propDf,aes(x=propDf$x+textShift,y=propDf$y+textShift,label=varLevels),
    #               vjust=2.75)
    # }
    
    #colnames shift out of figure 0.1 unit (colnamesDrift)
    newD <- shift(d, unit = colnamesDrift)
    p <- p + geom_segment(
      data = d,
      aes(
        x = 0,
        y = 0,
        xend = d$x,
        yend = d$y
      ),
      lty = base_lty,
      alpha = 0.6,
      col = "gray50"
    )
    if (nP != 0) {
      p <- p + geom_point(data = plotMax, aes(
        x = 0,
        y = 0,
        alpha = plotMax$Variables
      )) +
        geom_text(data = newD, aes(
          x = newD$x,
          y = newD$y,
          label = c(colnames(rawiData))
        ))
      
    }
    
    
    
    if (addText && type != "quantile" && nP != 0) {
      p <- p + geom_text(
        data = textMin,
        aes(
          x = textMin$cos,
          y = textMin$sin,
          label = textMin$min
        ),
        size = textSize,
        col = textCol
      ) +
        geom_text(
          data = textMax,
          aes(
            x = textMax$cos,
            y = textMax$sin,
            label = textMax$max
          ),
          size = textSize,
          col = textCol
        )
    }
    if (!showXYLabs) {
      p <- p + scale_x_continuous(labels = NULL,
                                  limits = c(-xyLimits, xyLimits)) +
        scale_y_continuous(labels = NULL,
                           limits = c(-xyLimits, xyLimits)) + xlab(NULL) + ylab(NULL)
    } else{
      #adds scale y continuous label
      yLabelPos <- seq(-xyLimits, xyLimits, (xyLimits - (-xyLimits)) / (nP +
                                                                          1))
      yLabelPos <- yLabelPos[-1]
      yLabelTitle <- paste0(colnames(iData), ": [")
      yLabels <- paste0(yLabelTitle, paste(unlist(minminList), unlist(maxmaxList), sep =
                                             ","), "]")
      yLabels <- c(yLabels, expression(bold(
        "The range of each variable show as follows :"
      )))
      p <- p + scale_x_continuous(limits = c(-xyLimits, xyLimits)) +
        scale_y_continuous(
          breaks = yLabelPos,
          labels = yLabels,
          limits = c(-xyLimits, xyLimits)
        ) + xlab(NULL) + ylab(NULL) +
        scale_x_continuous(labels = NULL,
                           limits = c(-xyLimits, xyLimits))
    }
    #showLegend
    if (showLegend) {
      p <- p + guides(col = "none")
    } else{
      p <- p + guides(fill = "none",
                      col = "none",
                      alpha = "none")
    }
    if (type == "quantile") {
      myTitle <- "Radar : quantile plot"
    } else{
      #myTitle<-paste0("Radar : ",paste(sapply(rownames(iData), paste, collapse=":"), collapse=","))
      myTitle = "Radar plot"
    }
    p <- p + labs(title = myTitle) +
      scale_colour_discrete(name = "Group") +
      scale_alpha_discrete(name = "Interval value of each observation") +
      scale_fill_discrete(name = "Group") +
      theme_bw()
    
    #make circle interpret
    if (base_circle) {
      a <- cos(120 * pi / 180)
      b <- sin(120 * pi / 180)
      #extend adjust quantile(seq(0,myRange,1/nLayer),seq(0,1,1/nLayer))[2:(nLayer+1)]
      tmp <- quantile(seq(0 + extendUnit, 1 + extendUnit, 1 / layerNumber),
                      seq(0, 1, 1 / layerNumber))
      #tmp<-seq(0,1,1/layerNumber)[-1]
      #end adjust
      tmp <- data.frame(
        x = a * tmp,
        y = b * tmp,
        label = paste0(as.character(round((tmp - extendUnit) * 100
        )), "%")
      )
      p <- p + geom_text(data = tmp, aes(
        x = tmp$x,
        y = tmp$y,
        label = tmp$label
      ))
    }
  } else{
    plotList <- NULL
    u <- 1
    for (i in levels(as.factor(plotMin$group))) {
      #print(i)
      if (nP == 0) {
        stop(
          "Cannot allows discrete variables only, under the parameter inOneFig = FALSE,
             please adjust inOneFig to TRUE."
        )
      }
      plotMin.temp <- dplyr::filter(plotMin, plotMin$group == i)
      plotMax.temp <- dplyr::filter(plotMax, plotMax$group == i)
      textMin.temp <- dplyr::filter(textMin, textMin$group == i)
      textMax.temp <- dplyr::filter(textMax, textMax$group == i)
      if (allnP != nP) {
        propDf.temp <- dplyr::filter(propDf, propDf$groupid == levels(propDf$groupid)[u])
        totalRectDf.temp <- dplyr::filter(totalRectDf,
                                          totalRectDf$obsGroup == levels(totalRectDf$obsGroup)[u])
      } else{
        propDf.temp <- 0
        totalRectDf.temp <- 0
      }
      
      if (type == "rect") {
        cutDf.temp <- list(NULL)
        for (k in 1:4) {
          cutDf[[k]]$obsGroup <- as.factor(cutDf[[k]]$obsGroup)
          cutDf.temp[[k]] <- dplyr::filter(cutDf[[k]], cutDf[[k]]$obsGroup ==
                                             levels(cutDf[[k]]$obsGroup)[u])
        }
      } else{
        cutDf.temp <- 0
      }
      
      
      
      #print(paste(u,plotMin.temp))
      base <- plotFun(
        p,
        iData,
        plotMin.temp,
        plotMax.temp,
        d,
        showXYLabs,
        showLegend,
        fillBetween,
        base_circle,
        layerNumber,
        textMin.temp,
        textMax.temp,
        rawiData,
        propDf,
        propDf.temp,
        allnP,
        nP,
        type,
        cutDf.temp,
        totalRectDf.temp,
        addText,
        textShift,
        base_lty,
        alpha,
        xyLimits,
        extendUnit,
        addText_modal,
        addText_modal.p
      )
      
      
      base <- base + labs(title = paste0("Radar : ", rownames(iData)[u])) +
        labs(title = paste0("Radar : ", rownames(iData)[u])) + scale_colour_discrete(name = "Group") +
        scale_alpha_discrete(name = "Interval")
      plotList[[u]] <- base
      u <- u + 1
    }
    
    #print(plotList[[1]])
    #p<-gridExtra::marrangeGrob(plotList,ncol=2,nrow=1)
    myRow <- round(sqrt(indNum))
    myCol <- ceiling(indNum / myRow)
    p <- gridExtra::marrangeGrob(plotList, ncol = myCol, nrow = myRow)
    
  }
  
  if (type == "quantile") {
    colShift <- 4
    myValues <- rev(grey.colors(quantileNum + 1 + colShift))[-c(1:colShift)]
    p <- p + guides(alpha = "none") +
      scale_fill_manual(name = "quantile percentage", values = myValues) +
      scale_colour_manual(values = myValues)
  }
  return(p)
}




plotFun <- function(p,
                    iData,
                    plotMin.temp,
                    plotMax.temp,
                    d,
                    showXYLabs,
                    showLegend,
                    fillBetween,
                    base_circle,
                    layerNumber,
                    textMin.temp,
                    textMax.temp,
                    rawiData,
                    propDf,
                    propDf.temp,
                    allnP,
                    nP,
                    type,
                    cutDf,
                    totalRectDf.temp,
                    addText,
                    textShift,
                    base_lty,
                    alpha,
                    xyLimits,
                    extendUnit,
                    addText_modal,
                    addText_modal.p) {
  #if(fillBetween){
  varLevels <- NULL
  if (type == "default") {
    myPolyData <- data.frame(NULL)
    myPathData <- data.frame(
      x1 = plotMin.temp$cos,
      y1 = plotMin.temp$sin,
      x2 = plotMax.temp$cos,
      y2 = plotMax.temp$sin
    )
    
    if (allnP == nP) {
      plotMin.rev <- plotMin.temp[nrow(plotMin.temp):1, ]
      myPolyData <- rbind(plotMax.temp,
                          plotMax.temp[1, ],
                          plotMin.temp[1, ],
                          plotMin.rev)

    } else{
      #add nominal
      tmp2 <- NULL
      j <- 1
      # print(levels(as.factor(propDf$varName)))
      # print(propDf)
      for (u in unique(as.factor(propDf.temp$varName))) {
        propDf.temp2 <- dplyr::filter(propDf.temp, propDf.temp$varName == u)
        tmp <- propDf.temp2[propDf.temp2$prop == max(propDf.temp2$prop), ]
        tmp <- tmp[1, ]
        tmp2 <- rbind(tmp2, plotMin.temp[1, ])
        tmp2[j, c("cos", "sin")] <- c(tmp$x, tmp$y)
        myPathData <- rbind(myPathData,
                            data.frame(
                              x1 = tmp$x,
                              y1 = tmp$y,
                              x2 = tmp$x,
                              y2 = tmp$y
                            ))
        j <- j + 1
      }
      plotMin.rev <- plotMin.temp[nrow(plotMin.temp):1, ]
      tmp2.rev <- tmp2[nrow(tmp2):1, ]
      newTemp <- rbind(plotMax.temp,
                       tmp2,
                       plotMax.temp[1, ],
                       plotMin.temp[1, ],
                       tmp2.rev,
                       plotMin.rev)
      myPolyData <- rbind(myPolyData, newTemp)
    }
    base <- p + geom_path(data = myPathData, aes(x = myPathData$x1, y =
                                                   myPathData$y1)) +
      geom_path(data = myPathData, aes(x = myPathData$x2, y = myPathData$y2))
    
    
    
    if (allnP != nP) {
      #tmpN<-dim(myPolyData)[1]
      
      this.obs <- totalRectDf.temp$obsGroup[1]
      myPolyData[, "obsGroup"] <- as.factor(this.obs)
      tempRectDf <- totalRectDf.temp[, c(1, 2, 3, 6)]
      
      tempPolyDf <- myPolyData[, c(1, 2, 3, 5)]
      colnames(tempRectDf) <- colnames(tempPolyDf)
      polyDf <- as.data.frame(rbind(tempPolyDf, tempRectDf))
      
      base <- base +
        geom_point(data = plotMin.temp, aes(x = plotMin.temp$cos, y = plotMin.temp$sin)) +
        geom_point(data = plotMax.temp, aes(x = plotMax.temp$cos, y = plotMax.temp$sin))
      
      
      
      
      base <- base + geom_polygon(
        data = polyDf,
        aes(
          x = polyDf$cos,
          y = polyDf$sin,
          group = polyDf$group,
          fill = polyDf$obsGroup,
          col = polyDf$obsGroup
        ),
        alpha = alpha
      )
      
      
      
    } else{
      base <- base + geom_polygon(
        data = myPolyData,
        aes(
          x = myPolyData$cos,
          y = myPolyData$sin,
          fill = myPolyData$group,
          col = myPolyData$group
        ),
        alpha = alpha
      ) +
        geom_point(data = plotMin.temp, aes(x = plotMin.temp$cos, y = plotMin.temp$sin)) +
        geom_point(data = plotMax.temp, aes(x = plotMax.temp$cos, y = plotMax.temp$sin))
    }
    
    # base<-base+geom_polygon(data=myPolyData,aes(x=myPolyData$cos,y=myPolyData$sin,fill=group),alpha = alpha,col="black")
  } else if (type == "rect") {
    #add cut segment
    rectPolyData <- data.frame(NULL)
    gId <- 1
    for (g in levels(as.factor(cutDf[[1]]$obsGroup))) {
      cutDf.temp <- lapply(
        1:4,
        FUN = function(x) {
          as.data.frame(dplyr::filter(cutDf[[x]], cutDf[[x]]$obsGroup == g))
        }
      )
      #print(cutDf.temp)
      for (i in 1:nP) {
        tmpDf <- data.frame(
          x = c(
            cutDf.temp[[1]]$x2[i],
            cutDf.temp[[2]]$x2[i],
            cutDf.temp[[4]]$x2[i],
            cutDf.temp[[3]]$x2[i]
          ),
          y = c(
            cutDf.temp[[1]]$y2[i],
            cutDf.temp[[2]]$y2[i],
            cutDf.temp[[4]]$y2[i],
            cutDf.temp[[3]]$y2[i]
          ),
          varGroup = as.factor(rep(gId, 4)),
          obsGroup = as.factor(g)
        )
        rectPolyData <- rbind(rectPolyData, tmpDf)
        gId <- gId + 1
      }
      
    }
    #print(cutDf)
    #print(rectPolyData)
    #numeric
    if (allnP != nP) {
      tempRectDf <- totalRectDf.temp[, c(1, 2, 3, 6)]
      colnames(tempRectDf) <- colnames(rectPolyData)
      levels(tempRectDf$obsGroup) <- levels(rectPolyData$obsGroup)
      rectPolyData <- as.data.frame(rbind(tempRectDf, rectPolyData))
    }
    base <- p + geom_polygon(
      data = rectPolyData,
      aes(
        x = .data$x,
        y = .data$y,
        group = .data$varGroup,
        fill = .data$obsGroup,
        col = .data$obsGroup
      ),
      alpha = alpha
    )
    
  }#end rect
  
  #add nominal text
  if (allnP != nP) {
    newPropDf <- as.data.frame(tidyr::separate(
      propDf,
      varLevels,
      into = c("varLevels", "gar"),
      sep = ":",
      convert = F
    ))
    newPropDf <- as.data.frame(dplyr::filter(
      newPropDf,
      newPropDf$groupid == levels(newPropDf$groupid)[1]
    ))
    if (addText_modal) {
      base <- base + geom_text(
        data = newPropDf,
        aes(
          x = newPropDf$x + textShift,
          y = newPropDf$y + textShift,
          label = newPropDf$varLevels
        ),
        vjust = 2.75
      )
    }
    
    #print(totalRectDf.temp)
    if (addText_modal.p) {
      tempd <- data.frame(NULL)
      for (i in levels(totalRectDf.temp$varGroup)) {
        temp.varG <- dplyr::filter(totalRectDf.temp, totalRectDf.temp$varGroup ==
                                     i)
        for (k in unique(temp.varG$varLevels)) {
          temp.varL <- dplyr::filter(temp.varG, temp.varG$varLevels == k)
          tempd <- rbind(tempd, data.frame(
            x = mean(temp.varL$newx),
            y = mean(temp.varL$newy)
          ))
        }
      }
      propTextDf <- data.frame(tempd, prop = propDf.temp$prop)
      #print(propTextDf)
      base <- base + geom_text(data = propTextDf,
                               aes(
                                 x = propTextDf$x + textShift,
                                 y = propTextDf$y,
                                 label = round(propTextDf$prop, 2)
                               ))
    }
  }
  
  newD <- shift(d, 0.1)
  base <- base + geom_segment(
    data = d,
    aes(
      x = 0,
      y = 0,
      xend = d$x,
      yend = d$y
    ),
    lty = base_lty,
    alpha = 0.6,
    col = "gray50"
  ) +
    geom_point(data = plotMax.temp, aes(
      x = 0,
      y = 0,
      alpha = plotMax.temp$Variables
    )) +
    geom_text(data = newD, aes(
      x = newD$x,
      y = newD$y,
      label = c(colnames(rawiData))
    ))
  
  if (addText) {
    base <- base + geom_text(data = textMin.temp,
                             aes(
                               x = textMin.temp$cos,
                               y = textMin.temp$sin,
                               label = textMin.temp$min
                             )) +
      geom_text(data = textMax.temp,
                aes(
                  x = textMax.temp$cos,
                  y = textMax.temp$sin,
                  label = textMax.temp$max
                ))
  }
  
  if (!showXYLabs) {
    base <- base + scale_x_continuous(labels = NULL,
                                      limits = c(-xyLimits, xyLimits)) +
      scale_y_continuous(labels = NULL,
                         limits = c(-xyLimits, xyLimits)) + xlab(NULL) + ylab(NULL)
  } else{
    base <- base + scale_x_continuous(limits = c(-xyLimits, xyLimits)) +
      scale_y_continuous(limits = c(-xyLimits, xyLimits))
  }#showLegend
  if (showLegend) {
    base <- base + guides(fill = "none", col = "none")
  } else{
    base <- base + guides(fill = "none",
                          col = "none",
                          alpha = "none")
  }
  
  if (base_circle) {
    a <- cos(120 * pi / 180)
    b <- sin(120 * pi / 180)
    
    #extend adjust quantile(seq(0,myRange,1/nLayer),seq(0,1,1/nLayer))[2:(nLayer+1)]
    tmp <- quantile(seq(0 + extendUnit, 1 + extendUnit, 1 / layerNumber),
                    seq(0, 1, 1 / layerNumber))
    #tmp<-seq(0,1,1/layerNumber)[-1]
    #end adjust
    tmp <- data.frame(
      x = a * tmp,
      y = b * tmp,
      label = paste0(as.character(round((tmp - extendUnit) * 100
      )), "%")
    )
    base <- base + geom_text(data = tmp, aes(
      x = tmp$x,
      y = tmp$y,
      label = tmp$label
    ))
  }
  base <- base + theme_bw() + guides(alpha = "none")
  return(base)
}
data2Vec <- function(iData = NULL,
                     data = NULL,
                     transMat = NULL,
                     type = NULL,
                     quantileNum = NULL) {
  if (dim(data)[2] != dim(transMat)[1]) {
    stop("ERROR : Cannot match data and transMat")
  }
  result = data.frame(NULL)
  if (type == "quantile") {
    for (i in 1:dim(data)[1]) {
      result <- rbind(result, data.frame(data[i, ] * transMat, group = paste(
        paste("Group", as.factor(letters[i])), rownames(iData)[i], sep = " : "
      )))
    }
  } else{
    for (i in 1:dim(data)[1]) {
      result <- rbind(result, data.frame(data[i, ] * transMat, group = paste(
        paste("Group", as.factor(i)), rownames(iData)[i], sep = " : "
      )))
    }
  }
  return(result)
}
generateCircle <- function(nLayer = NULL,
                           extendUnit = 0.5) {
  if (is.null(nLayer) || nLayer <= 0 || nLayer > 30) {
    stop("Illegal layerNumber input. Recommended value will between 2 and 10.")
  }
  circles <- data.frame(
    x0 = rep(0, nLayer),
    y0 = rep(0, nLayer),
    #r = round(seq(0,myRange,1/nLayer)[2:(nLayer+1)],2)
    #r = quantile(seq(0,myRange,1/nLayer),seq(0,1,1/nLayer))[2:(nLayer+1)]
    r = quantile(seq(
      0 + extendUnit, 1 + extendUnit, 1 / (nLayer - 1)
    ), seq(0, 1, 1 / (nLayer - 1)))
  )
  p <- ggplot() +
    ggforce::geom_circle(
      aes(
        x0 = circles$x0,
        y0 = circles$y0,
        r = circles$r
      ),
      data = circles,
      col = "gray50"
    )
  return(p)
}
generatePoint <- function(nPoly = NULL,
                          nLayer = NULL,
                          extendUnit = 0.5) {
  if (is.null(nPoly) || nPoly <= 0 || nPoly > 360) {
    stop("ERROR : Illegal parameter input in nPoly")
  }
  if (is.null(nLayer) || nLayer <= 0 || nLayer > 30) {
    stop("ERROR : Illegal parameter input in nLayer")
  }
  #rList = round(seq(0,1,1/nLayer)[2:(nLayer+1)],2)
  #rList = quantile(seq(0,myRange,1/nLayer),seq(0,1,1/nLayer))[2:(nLayer+1)]
  rList = quantile(seq(0 + extendUnit, 1 + extendUnit, 1 / (nLayer - 1)), seq(0, 1, 1 /
                                                                                (nLayer - 1)))
  degreeUnit <- round(360 / nPoly)
  allPoint <- data.frame(NULL)
  for (r in rList) {
    allPoint <- rbind(allPoint, data.frame(
      x = 0,
      y = r,
      cos = 0,
      sin = 1
    ))
    for (i in 1:(nPoly - 1)) {
      degree <- 90 + degreeUnit * i
      x1 <- cos(degree * pi / 180) * r
      y1 <- sin(degree * pi / 180) * r
      allPoint <- rbind(allPoint, data.frame(
        x = x1,
        y = y1,
        cos = cos(degree * pi / 180),
        sin = sin(degree * pi / 180)
      ))
    }
  }
  return(allPoint)
}
getCutLine <- function(startPoint, len, perp) {
  if (perp[1] == 0 && perp[2] == 0) {
    stop("(0,0) vector cannot find a Perpendicular")
  } else if (perp[2] == 0) {
    if (perp[1] < 0) {
      return(c(startPoint[1] - len, startPoint[2]))
    }
    return(c(startPoint[1] + len, startPoint[2]))
  } else if (perp[1] == 0) {
    return(c(startPoint[1], startPoint[2] + len))
  } else{
    r <- perp[1] / perp[2]
  }
  #adjust <- abs(len/(perp[1]))
  adjust <- (len^2 / ((perp[1]^2) * (1 + (1 / r)^2)))^0.5
  result <-  c(perp[1] * adjust + startPoint[1] , perp[1] * adjust * (1 /
                                                                        r) + startPoint[2])
  return(result)
}
getPerpendicular <- function(v = NULL, changeSide = TRUE) {
  if (changeSide) {
    if (is.vector(v) && length(v) == 2) {
      if (v[1] == 0 && v[2] == 0) {
        stop("cannot find perp. with a point")
      } else if (v[1] == 0) {
        return(c(-1, 0))
      } else if (v[2] == 0) {
        return(c(0, -1))
      } else{
        return(c(-1, v[1] / v[2]))
      }
    } else{
      stop("generate perpendicular vector error, check data type and dimension.")
    }
  } else{
    if (is.vector(v) && length(v) == 2) {
      if (v[1] == 0 && v[2] == 0) {
        stop("cannot find perp. with a point")
      } else if (v[1] == 0) {
        return(c(1, 0))
      } else if (v[2] == 0) {
        return(c(0, 1))
      } else{
        return(c(1, -v[1] / v[2]))
      }
    } else{
      stop("generate perpendicular vector error, check data type and dimension.")
    }
  }
}
getCutDf <- function(tempDf, transMatrix, nP, indNum) {
  startDf <- data.frame(x = tempDf$cos,
                        y = tempDf$sin,
                        group = tempDf$group)
  perp1 <- lapply(
    1:nP,
    FUN = function(x)
      getPerpendicular(unlist(transMatrix[x, ]), changeSide = T)
  )
  perp2 <- lapply(
    1:nP,
    FUN = function(x)
      getPerpendicular(unlist(transMatrix[x, ]), changeSide = F)
  )
  
  cutPos1 <- sapply(
    1:(nP * indNum),
    FUN = function(x) {
      u <- x %% nP
      if (u == 0) {
        u <- nP
      }
      getCutLine(startDf[x, ], 0.025, perp1[[u]])
    }
  )
  cutPos2 <- sapply(
    1:(nP * indNum),
    FUN = function(x) {
      u <- x %% nP
      if (u == 0) {
        u <- nP
      }
      getCutLine(startDf[x, ], 0.025, perp2[[u]])
    }
  )
  #print(as.data.frame(t(cutPos1)))
  cutDf1 <- cbind(startDf, as.data.frame(t(cutPos1)))
  cutDf2 <- cbind(startDf, as.data.frame(t(cutPos2)))
  colnames(cutDf1) <- c("x1", "y1", "obsGroup", "x2", "y2")
  colnames(cutDf2) <- c("x1", "y1", "obsGroup", "x2", "y2")
  
  cutDf1$x2 <- as.numeric(cutDf1$x2)
  cutDf1$y2 <- as.numeric(cutDf1$y2)
  cutDf2$x2 <- as.numeric(cutDf2$x2)
  cutDf2$y2 <- as.numeric(cutDf2$y2)
  return(list(data.frame(cutDf1), data.frame(cutDf2)))
}
build3DRect <- function(len = NULL,
                        wid = NULL,
                        hei = NULL,
                        g = NULL) {
  if (is.null(g)) {
    g <- as.factor("g")
  }
  vertice <- get_vertice(3)
  vertice[c(3, 4, 7:8), 2] <- vertice[c(3, 4, 7:8), 2] + hei
  vertice[c(5:8), 1] <- vertice[c(5:8), 1] + wid
  vertice[c(2, 4, 6, 8), 3] <- vertice[c(2, 4, 6, 8), 3] + len
  vertice <- as.data.frame(vertice)
  vertice <- point3Dto2D(vertice)
  #rearrange
  vertice[, 6:11] <- 0
  vertice[c(1, 2, 4, 3), 6] <- paste0(g, 1)
  vertice[c(3, 4, 8, 7), 7] <- paste0(g, 2)
  vertice[c(5, 6, 8, 7), 8] <- paste0(g, 3)
  vertice[c(1, 2, 6, 5), 9] <- paste0(g, 4)
  vertice[c(1, 3, 7, 5), 10] <- paste0(g, 5)
  vertice[c(2, 4, 8, 6), 11] <- paste0(g, 6)
  d <- vertice[, c(4, 5)]
  d <- rbind(d[c(1, 2, 4, 3), ], d[c(3, 4, 8, 7), ], d[c(5, 6, 8, 7), ], d[c(1, 2, 6, 5), ], d[c(1, 3, 7, 5), ], d[c(2, 4, 8, 6), ])
  group <- matrix(c(vertice[, 6], vertice[, 7], vertice[, 8], vertice[, 9], vertice[, 10], vertice[, 11]),
                  ncol = 1)
  group <- group[group != 0]
  d <- cbind(d, group)
  d$group <- as.factor(d$group)
  return(d)
}
reRange <- function(min, max, data) {
  dist <- max - min
  return(data * dist - abs(min))
}
shift <- function(data, unit) {
  unit <- 1 + unit
  data$x <- round(data$x, 10)
  data$y <- round(data$y, 10)
  data$x <- data$x * unit
  data$y <- data$y * unit
  return(data)
}

#' @name ggInterval_2Dhist
#' @title Visualize a 2-dimension histogram for interval-valued data
#' @description Visualize the joint distribution of two continuous
#' interval-valued variables by dividing the x axis and y axis into
#' rectangles and calculating the frequency of each observation interval
#' in every rectangle.
#' @import tidyverse rlang ggplot2
#' @importFrom RSDA is.sym.interval
#' @param data A ggInterval object. It can also be either an RSDA object or
#' a classical data frame, which will be automatically converted to
#' ggInterval data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit.aes = TRUE (the default), it is combined with
#' the default mapping at the top level of the plot. You must supply
#' mapping if there is no plot mapping. It is the same as the mapping of
#' ggplot2.
#' @param method Histogram partition method. Use \code{"equal-bin"} for
#' equally spaced bins or \code{"unequal-bin"} for non-equidistant bins
#' defined by the observed interval endpoints.
#' @param xBins Number of x-axis bins used when \code{method = "equal-bin"}.
#' @param yBins Number of y-axis bins used when \code{method = "equal-bin"}.
#' @param display Metric shown in the cells. Use \code{"p"} for relative
#' frequency, \code{"f"} for weighted frequency, or \code{"h"} for density.
#' @param palette ColorBrewer palette passed to \code{scale_fill_distiller()}.
#' @param direction Direction passed to \code{scale_fill_distiller()}.
#' @param tau Non-negative tolerance used when \code{method = "unequal-bin"}.
#' Consecutive breakpoints whose gaps are smaller than \code{tau} are merged.
#' @param removeZero Whether remove data whose frequency is equal to zero.
#' @param cell_labels Logical. If \code{TRUE}, add frequency labels to cells.
#' @param label_rule Rule used when \code{cell_labels = TRUE}. The default
#' \code{"above-mean"} shows only cells whose frequencies exceed the mean
#' cell frequency. Alternatives are \code{"nonzero"} and \code{"all"}.
#' @param addFreq Deprecated alias for \code{cell_labels}.
#' @return Return a list containing a ggplot2 object and the corresponding
#' frequency table.
#' @usage ggInterval_2Dhist(data = NULL, mapping = aes(NULL),
#' method = "equal-bin", xBins = 14, yBins = 16,
#' display = "p", palette = "Blues", direction = 1, tau = 0,
#' removeZero = FALSE,
#' cell_labels = FALSE, label_rule = "above-mean", addFreq = NULL)
#' @examples
#' ggInterval_2Dhist(oils, aes(x = GRA, y = FRE),
#'   xBins = 5, yBins = 5,
#'   display = "p",
#'   palette = "Blues",
#'   cell_labels = TRUE)
#'
#' ggInterval_2Dhist(oils, aes(x = GRA, y = FRE),
#'   method = "unequal-bin",
#'   display = "p",
#'   palette = "Blues")
#'
#' @export
ggInterval_2Dhist <- function(data = NULL,
                              mapping = aes(NULL),
                              method = "equal-bin",
                              xBins = 14,
                              yBins = 16,
                              display = "p",
                              palette = "Blues",
                              direction = 1,
                              tau = 0,
                              removeZero = FALSE,
                              cell_labels = FALSE,
                              label_rule = "above-mean",
                              addFreq = NULL) {
  method <- match_hist2d_method(method)
  display <- match_hist2d_display(display)
  label_rule <- match.arg(label_rule, c("above-mean", "nonzero", "all"))
  if (!is.null(addFreq)) {
    cell_labels <- addFreq
  }

  if (method == "equal-bin" && xBins + yBins > 200) {
    stop("ERROR : Bins are too large to calculate. Suggest both axes use fewer than 100 bins.")
  }

  args_num <- length(mapping)
  args <- lapply(mapping[1:args_num], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y

  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  plot_data <- if ("ggInterval" %in% class(data)) iData else data
  p <- dim(plot_data)[2]
  n <- dim(iData)[1]
  resultSet <- NULL

  with(plot_data, {
    attr1 <- which(unlist(lapply(
      plot_data[, 1:p], FUN = identical, x = eval(this.x)
    )))
    attr1 <- names(attr1)
    attr2 <- which(unlist(lapply(
      plot_data[, 1:p], FUN = identical, x = eval(this.y)
    )))
    attr2 <- names(attr2)

    if (length(attr1) == 0 || length(attr2) == 0) {
      stop("ERROR : Missing attributes x or y in data frame.")
    }

    if (all((!is.numeric(plot_data[[attr1]])), !RSDA::is.sym.interval(plot_data[[attr1]])) ||
      all((!is.numeric(plot_data[[attr2]])), !RSDA::is.sym.interval(plot_data[[attr2]]))) {
      stop("ERROR : Variables in 2D histogram can only be numeric.")
    }

    hist2d_data <- build_hist2d_data(
      iData = iData,
      attr1 = attr1,
      attr2 = attr2,
      xBins = xBins,
      yBins = yBins,
      method = method,
      tau = tau
    )

    work_size <- n * hist2d_data$xBins * hist2d_data$yBins
    if (work_size >= 250000 && work_size < 1000000) {
      warning("The number of observations and bins is not suggested be too large.")
    } else if (work_size >= 1000000) {
      stop("The number of observations and bins are too large and will exceed the time limit.")
    }

    freq.matrix <- hist2d_assign_display(hist2d_data$data, display = display)
    table_data <- hist2d_data

    if (removeZero) {
      freq.matrix <- freq.matrix[freq.matrix$display_value != 0, , drop = FALSE]
    }
    if (nrow(freq.matrix) == 0) {
      stop("All histogram cells have zero frequency after removeZero = TRUE.")
    }

    border_linewidth <- if (method == "unequal-bin") 0.15 else 0.3
    usermapping <- mapping[setdiff(names(mapping), c("x", "y"))]
    mymapping <- list(
      data = freq.matrix,
      mapping = aes(
        xmin = x1,
        xmax = x2,
        ymin = y1,
        ymax = y2,
        fill = display_value,
        alpha = 0.5
      ),
      linewidth = border_linewidth
    )
    allmapping <- as.list(structure(as.expression(c(
      usermapping, mymapping
    )), class = "uneval"))

    base <- ggplot(freq.matrix, aes(x1, y1)) +
      do.call(geom_rect, allmapping) +
      scale_fill_distiller(
        palette = palette,
        direction = direction,
        limits = c(0, max(freq.matrix$display_value)),
        name = hist2d_legend_name(display)
      ) +
      labs(
        x = attr1,
        y = attr2,
        title = if (method == "equal-bin") "2D hist." else "2D hist. (Non-equidistant-bin)"
      ) +
      guides(alpha = "none")

    if (cell_labels) {
      label_data <- switch(
        label_rule,
        "above-mean" = freq.matrix[freq.matrix$display_value > mean(freq.matrix$display_value), , drop = FALSE],
        "nonzero" = freq.matrix[freq.matrix$display_value != 0, , drop = FALSE],
        "all" = freq.matrix
      )
      if (nrow(label_data) != 0) {
        label_data$cell_label <- format_hist2d_labels(label_data$display_value)
        base <- base + geom_text(
          data = label_data,
          aes(
            x = xmid,
            y = ymid,
            label = cell_label
          ),
          inherit.aes = FALSE
        )
      }
    }

    resultSet[["plot"]] <- base
    resultSet[[paste0("Table (", attr1, ", ", attr2, ")")]] <- build_hist2d_table(
      hist2d_data = table_data,
      attr1 = attr1,
      attr2 = attr2,
      display = display
    )
    return(resultSet)
  })
}

#' @name ggInterval_PCA
#' @title Vertices PCA for interval data
#' @description ggInterval_PCA performs a principal components
#' analysis on the given numeric interval data and returns the
#' results of princomp, a ggplot object, and interval scores.
#' @import rlang ggplot2 stats
#' @importFrom RSDA is.sym.interval
#' @importFrom gtools odd
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame, which will be automatically convert to ggInterval
#' data.
#' @param mapping  Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' It is the same as the mapping of ggplot2.
#' @param plot Boolean variable,Auto plot (if TRUE).It can also plot by
#' its inner object
#' @param concepts_group color with each group of concept
#' @param poly if plot a poly result
#' @param adjust adjust sign of the principal component
#' @param showLabels Logical. If \code{TRUE}, label interval scores in the plot.
#' @param labelSize Numeric text size used when \code{showLabels = TRUE}.
#' @param checkOverlap Logical. Passed to \code{geom_text()} to suppress
#' overlapping labels when needed.
#' @return A ggplot object for PC1, PC2, and interval scores with related outputs.
#' \itemize{
#'   \item scores_interval - The interval scores after PCA.
#'   \item ggplotPCA - a ggplot object with x-axis and y-axis are PC1 and
#'   PC2.
#'   \item others - others are the returns values of princomp.
#' }
#' @usage ggInterval_PCA(data = NULL,mapping = aes(NULL),plot=TRUE,
#'                       concepts_group=NULL, poly = FALSE, adjust = TRUE,
#'                       showLabels = TRUE, labelSize = 3,
#'                       checkOverlap = FALSE)
#' @examples
#' \donttest{
#' Subjects <- substr(rownames(facedata), 1, 3)
#' p <- ggInterval_PCA(facedata, plot = FALSE, concepts_group = Subjects)
#' p$ggplotPCA
#' p$scores_interval
#' ggInterval_PCA(facedata, poly = TRUE, concepts_group = Subjects)
#' }
#'
#' @export
ggInterval_PCA <- function(data = NULL,
                           mapping = aes(NULL),
                           plot = TRUE,
                           concepts_group = NULL,
                           poly = FALSE,
                           adjust = TRUE,
                           showLabels = TRUE,
                           labelSize = 3,
                           checkOverlap = FALSE) {
  #data preparing
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  temp <- c(which(names(args) == "x"), which(names(args) == "y"))
  if (length(temp) > 0) {
    args.noXY <- args[-temp]
  } else{
    args.noXY <- args
  }
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  myRowNames <- rownames(iData)
  
  #preparing data
  temp.1 <- ggSymData$statisticsDF$min
  temp.2 <- ggSymData$statisticsDF$max
  n <- dim(temp.1)[1]
  p <- dim(temp.1)[2]
  if (!is.null(concepts_group)) {
    if (length(concepts_group) != n) {
      stop(paste0("Length of concepts group must be equal to data (", n, ")."))
    }
    concepts_group <- as.factor(concepts_group)
  }
  
  
  #test big o
  if (n > 2000) {
    stop("Out of time limits.")
  }
  
  if (p < 2) {
    stop("ERROR : Variables need at least 2.")
  }
  vertice <- cbind(get_vertice(p), 2 * p + 1)
  
  #build PCA data
  pcaData <- lapply(
    1:p,
    FUN = function(x)
      cbind(temp.1[, x], temp.2[, x])
  )
  pcaData <- data.frame(pcaData, c(1:n))
  pcaData <- as.matrix(pcaData)
  
  #Meger all pca data into a large matrix m
  m <- NULL
  for (i in 1:n) {
    temp <- t(sapply(
      1:2^p,
      FUN = function(x)
        rbind(NULL, pcaData[i, vertice[x, ]])
    ))
    temp <- cbind(temp, 1:2^p)
    m <- rbind(m, temp)
  }
  
  #remove repeat value
  m <- m[!duplicated(as.data.frame(m[, 1:p])), ]
  m[, 1:p] <- apply(m[, 1:p], 2, scale)
  
  #start PCA
  mypca <- stats::princomp(m[, 1:p])
  if (adjust) {
    mypca$scores[, 1] <- mypca$scores[, 1] * -1
    mypca$scores[, 3] <- mypca$scores[, 3] * -1
  }
  
  PCscores <- data.frame(mypca$scores, u = m[, p + 1])
  
  #plot Poly PCA
  #plot Dim.1 and Dim.2 cause no.comp : number of component = 2
  if (poly) {
    polyList <- build_PCA_poly_data(mypca$scores, m, n, p, no.comp = 2, group = concepts_group)
    d <- polyList$cpData
    usermapping <- structure(as.expression(args.noXY), class = "uneval") #Aesthetic without x,y
    mymapping <- list(
      d,
      mapping = aes(
        x = .data$x1,
        y = .data$y1,
        xend = .data$x2,
        yend = .data$y2,
        col = .data$Concepts_Group
      )
    )
    
    allmapping <- as.list(structure(as.expression(c(
      usermapping, mymapping
    )), class = "uneval"))
    
    
    base <- ggplot(d, aes(.data$x1, .data$y1)) +
      do.call(geom_segment, allmapping)
    
    
  }
  
  
  #build PCA interval data
  PCscores_interval <- matrix(0, nrow = n, ncol = 2 * p)
  for (obs in 1:n) {
    for (var in 1:p) {
      PCscores_interval[obs, 2 * var - 1] <- min(PCscores[PCscores$u == obs, var])
      PCscores_interval[obs, 2 * var] <- max(PCscores[PCscores$u == obs, var])
    }
  }
  PCscores_interval <- as.data.frame(PCscores_interval)
  
  
  myname <- NULL
  for (i in 1:p) {
    a <- paste0("PC", i, ".min")
    b <- paste0("PC", i, ".max")
    myname <- append(myname, append(a, b))
  }
  colnames(PCscores_interval) <- myname
  mypca$scores_interval <- adjustToRSDA(PCscores_interval)
  mypca$scores_interval[, "rowname"] <- rownames(iData)
  mypca$scores_interval <- tibble::column_to_rownames(mypca$scores_interval, var = "rowname")
  class(mypca$scores_interval) <- c(class(mypca$scores_interval), "symbolic_tbl")
  PCscores_interval$rowname_label <- myRowNames
  #蝑??ㄐ ?隞半uild rownames ????rownames??絲靘??
  #build Aesthetic (mapping)
  if (!poly) {
    usermapping <- structure(as.expression(args.noXY), class = "uneval") #Aesthetic without x,y
    
    if (!is.null(concepts_group)) {
      mymapping <- list(
        PCscores_interval,
        mapping = aes(
          xmin = .data$PC1.min,
          xmax = .data$PC1.max,
          ymin = .data$PC2.min,
          ymax = .data$PC2.max,
          fill = concepts_group,
          alpha = 0.5
        ),
        col = "black"
      )
    } else{
      mymapping <- list(
        PCscores_interval,
        mapping = aes(
          xmin = .data$PC1.min,
          xmax = .data$PC1.max,
          ymin = .data$PC2.min,
          ymax = .data$PC2.max,
          fill = grDevices::gray.colors(n),
          alpha = 0.5
        ),
        col = "black"
      )
    }
    allmapping <- as.list(structure(as.expression(c(
      usermapping, mymapping
    )), class = "uneval"))
    
    #plot
    base <- ggplot(PCscores_interval, aes(.data$PC1.min, .data$PC2.min)) +
      do.call(geom_rect, allmapping) +
      guides(colour = "none", alpha = "none")
    
    if (showLabels) {
      base <- base +
        geom_text(
          aes(label = .data$rowname_label),
          size = labelSize,
          check_overlap = checkOverlap
        )
    }
    
    if (is.null(concepts_group)) {
      base <- base + scale_fill_manual(name = "Concept",
                                       values = gray.colors(n),
                                       labels = myRowNames)
    }
  }
  
  if (is.null(concepts_group)) {
    base <- base + guides(fill = "none",
                          colour = "none")
  }
  base <- base + labs(x = "PC1", y = "PC2")
  mypca$ggplotPCA <- base
  if (plot) {
    plot(base)
  }
  return(mypca)
}

get_vertice <- function(p) {
  origin <- matrix(which(gtools::odd(1:(2 * p))), nrow = 1)
  for (i in p:1) {
    temp <- origin
    temp[, i] <- temp[, i] + 1
    origin <- rbind(origin, temp)
  }
  return(origin)
}
adjustToRSDA <- function(data) {
  size <- dim(data)[1] * dim(data)[2]
  p <- dim(data)[2] / 2
  tempd <- matrix(0, nrow = size / p, ncol = p)
  odd <- unlist(as.list(matrix(which(
    gtools::odd(1:nrow(tempd))
  ), nrow = 1)))
  even <- odd + 1
  for (i in 1:p) {
    tempd[odd, i] <- data[, i * 2 - 1]
    tempd[even, i] <- data[, i * 2]
  }
  tempd <- cbind(tempd, rep(1:(size / (p * 2)), each = 2))
  tempd <- as.data.frame(tempd)
  tempd <- RSDA::classic.to.sym(tempd, concept = colnames(tempd)[dim(tempd)[2]])
  colnames(tempd) <- paste0("PC_", 1:p)
  return(tempd)
}


build_PCA_poly_data <- function(scores, m, n, p, no.comp, group) {
  polyData <- scores[, 1:no.comp]
  cp <- connectPoint(m[, 1:p], n, p)
  Ni <- cp[length(cp)]
  num <- dim(cp)[1]
  cpData <- NULL
  if (is.null(group)) {
    group <- rep(1, n)
  }
  for (i in 1:num) {
    Hi <- scores[((cp[i, 1] - 1) * Ni + 1):(cp[i, 1] * Ni), ]
    cpData <- rbind(cpData,
                    data.frame(
                      x1 = Hi[cp[i, 2], 1],
                      y1 = Hi[cp[i, 2], 2],
                      x2 = Hi[cp[i, 3], 1],
                      y2 = Hi[cp[i, 3], 2],
                      Concepts_Group = group[cp[i, 1]]
                    ))
  }
  cpData$Concepts_Group <- as.factor(cpData$Concepts_Group)
  polyData <- as.data.frame(polyData)
  colnames(polyData) <- paste0("PC", 1:no.comp)
  return(list(polyData = polyData, cpData = cpData))
}


connectPoint <- function(vdata, m, p) {
  Ni <- 2^p
  tempMatrix <- matrix(0, nrow = m * Ni * p, ncol = 3)
  count <- 0
  for (k in 1:m) {
    Hi <- vdata[((k - 1) * Ni + 1):(k * Ni), ]
    for (i in 1:(Ni - 1)) {
      for (j in (i + 1):Ni) {
        if (length(which(Hi[i, ] - Hi[j, ] == 0)) == (p - 1)) {
          count = count + 1
          tempMatrix[count, 1] <- k
          tempMatrix[count, 2] <- i
          tempMatrix[count, 3] <- j
        }
        
      }
    }
    
  }
  Cmatrix <- tempMatrix[1:count, ]
  Cmatrix
  #Cmatrix[k i j],
  #k=1,..., m;
  #Hi: i=1,...,Ni;
  #j: the points that connect to i
  
}

#' @name ggInterval_3Dscatterplot
#' @title 3D scatter plot for interval data
#' @description Visualize the three continuous variable distribution
#' by collecting all vertices in each interval to form a shape of
#' cube.Also show the difference between each group.
#' @param data A ggInterval object. It can also be either an RSDA object or
#' a classical data frame, which will be automatically converted to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' It is the same as the mapping of ggplot2.
#' @param scale A boolean variable. \code{TRUE} standardizes the data and
#' \code{FALSE} leaves the data unchanged.
#' If variance is too large(or small) or the difference between
#' two variables are too large,it will be distorted or difficult to see, which
#' may happen when variables are measured in different units.
#' @return Return a ggplot2 object (It will still be 2-Dimension).
#' @usage ggInterval_3Dscatterplot(data = NULL,mapping = aes(NULL),scale=FALSE)
#' @examples
#' ggInterval_3Dscatterplot(facedata[1:5, ], aes(x = BC, y = EH, z = GH))
#'
#' @export
ggInterval_3Dscatterplot <- function(data = NULL,
                                 mapping = aes(NULL),
                                 scale = FALSE) {
  #data preparing
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  if (is.null(args$z)) {
    args$z <- args[[3]]
    args <- args[-3]
  }
  this.z <- args$z
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  p <- dim(data)[2]
  n <- dim(iData)[1]
  
  #test big O
  if (n > 1000) {
    stop("Out of time limits.")
  }
  
  with(data, {
    #get attrs
    attr1 <- which(unlist(lapply(
      data[, 1:p], FUN = identical, x = eval(this.x)
    )))
    attr1 <- names(attr1)
    attr3 <- which(unlist(lapply(
      data[, 1:p], FUN = identical, x = eval(this.y)
    )))
    attr3 <- names(attr3)
    attr2 <- which(unlist(lapply(
      data[, 1:p], FUN = identical, x = eval(this.z)
    )))
    attr2 <- names(attr2)
    #if cannot find attr
    if (length(attr1) == 0 || length(attr2) == 0 ||
        length(attr3) == 0) {
      stop("ERROR : Missing attributes x,y or z in data frame.")
    }
    
    #test attribute illegal
    if (all((!is.numeric(data[[attr1]])) , !RSDA::is.sym.interval(data[[attr1]])) ||
        all((!is.numeric(data[[attr3]])) , !RSDA::is.sym.interval(data[[attr3]]))
        ||
        all((!is.numeric(data[[attr2]])), !RSDA::is.sym.interval(data[[attr2]]))) {
      stop("ERROR : Variables in Scatter Plot can only be numeric.")
    }
    
    #Test adjust
    if (scale) {
      iData <- scaleInterval(iData[, c(attr1, attr2, attr3)])
    }
    
    #build ggplot data.frame
    d <- data.frame(NULL)
    for (i in 1:dim(iData)[1]) {
      tmp <- data.frame(iData[[attr1]][i], iData[[attr2]][i], iData[[attr3]][i])
      d <- rbind(d, buildPolyGroup(tmp, g = i))
    }
    
    #build vision in plot
    sy <- sd(d$newy)
    sy <- ifelse(is.na(sy), 0, sy)
    sy <- ifelse(sy > 10, 10, sy)
    
    sx <- sd(d$newx)
    sx <- ifelse(is.na(sx), 0, sx)
    sx <- ifelse(sx > 10, 10, sx)
    
    m1 <- min(c(min(d$newx), max(d$newx), min(d$newy), max(d$newy)))
    m2 <- max(c(min(d$newx), max(d$newx), min(d$newy), max(d$newy)))
    
    #build Aesthetic (mapping)
    usermapping <- mapping[-c(1, 2, 3)] #Aesthetic without x,y,z
    mymapping <- list(
      data = d,
      mapping = aes(group = group, fill = cluster),
      alpha = 0.2,
      col = "black",
      lwd = 0.8
    )
    allmapping <- as.list(structure(as.expression(c(
      usermapping, mymapping
    )), class = "uneval"))
    
    #plot
    ggplot(d, aes(newx, newy)) +
      geom_point() +
      do.call(geom_polygon, allmapping) +
      geom_segment(aes(
        x = m1 - sx,
        xend = m2,
        y = m1 - sy,
        yend = m2
      ),
      arrow = arrow(length = unit(0.3, "cm"))) + #z
      geom_segment(aes(
        x = m1 - sx,
        xend = m2,
        y = m1 - sy,
        yend = m1 - sy
      ),
      arrow = arrow(length = unit(0.3, "cm"))) + #x
      geom_segment(aes(
        x = m1 - sx,
        xend = m1 - sx,
        y = m1 - sy,
        yend = m2
      ),
      arrow = arrow(length = unit(0.3, "cm"))) + #y
      coord_fixed(ratio = 1) + labs(x = attr1, y = attr2, title = "3D scatter plot") +
      annotate(
        geom = "text",
        x = m2 - sx / 2,
        y = m2 - sy / 2,
        label = attr3
      )
  })
}
point3Dto2D <- function(data = NULL) {
  data <- as.data.frame(data)
  z <- data[, 3]
  z <- round(((z^2) / 2)^(0.5), 6)
  data[, "newx"] <- data.frame(z + data[, 1])
  data[, "newy"] <- data.frame(z + data[, 2])
  return(data)
}
buildPolyGroup <- function(newPoint, g = NULL) {
  #Sort data
  vertice <- as.factor(get_vertice(3))
  temp <- newPoint
  levels(vertice) <- as.character(newPoint)
  newPoints <- matrix(as.numeric(as.character(vertice)),
                      nrow = 8,
                      ncol = 3)
  
  #3D to 2D
  newPoints <- point3Dto2D(newPoints)
  
  #make group
  newPoints[, 6:11] <- 0
  newPoints[c(1, 2, 3, 4), 6] <- paste0(g, 1)
  newPoints[c(3, 4, 7, 8), 7] <- paste0(g, 2)
  newPoints[c(5, 6, 7, 8), 8] <- paste0(g, 3)
  newPoints[c(1, 2, 5, 6), 9] <- paste0(g, 4)
  newPoints[c(1, 3, 5, 7), 10] <- paste0(g, 5)
  newPoints[c(2, 4, 6, 8), 11] <- paste0(g, 6)
  
  d <- newPoints[, c(4, 5)]
  d <- rbind(d[c(1, 2, 4, 3), ], d[c(3, 4, 8, 7), ], d[c(5, 6, 8, 7), ], d[c(1, 2, 6, 5), ], d[c(1, 3, 7, 5), ], d[c(2, 4, 8, 6), ])
  group <- matrix(c(
    newPoints[, 6],
    newPoints[, 7],
    newPoints[, 8],
    newPoints[, 9],
    newPoints[, 10],
    newPoints[, 11]
  ),
  ncol = 1)
  group <- group[group != 0]
  d <- cbind(d, group)
  d$group <- as.factor(d$group)
  d[, "cluster"] <- g
  d$cluster <- as.factor(d$cluster)
  return(d)
}

scaleInterval <- function(data = NULL) {
  if (dim(data)[2] > 3) {
    stop("ERROR : input data must be 3 attributes (x,y,z)")
  }
  temp <- data.frame(data[[1]], data[[2]], data[[3]])
  size <- dim(temp)[1] * dim(temp)[2]
  tempd <- matrix(0, nrow = size / 3, ncol = 3)
  odd <- unlist(as.list(matrix(which(
    gtools::odd(1:nrow(tempd))
  ), nrow = 1)))
  even <- odd + 1
  tempd[odd, 1] <- temp[, 1]
  tempd[even, 1] <- temp[, 2]
  tempd[odd, 2] <- temp[, 3]
  tempd[even, 2] <- temp[, 4]
  tempd[odd, 3] <- temp[, 5]
  tempd[even, 3] <- temp[, 6]
  tempd <- scale(tempd)
  tempd <- cbind(tempd, rep(1:(size / 6), each = 2))
  tempd <- as.data.frame(tempd)
  tempd <- RSDA::classic.to.sym(tempd, concept = "V4")
  colnames(tempd) <- colnames(data)
  return(tempd)
}

match_hist2d_method <- function(method) {
  match.arg(method, c("equal-bin", "unequal-bin"))
}

match_hist2d_display <- function(display) {
  match.arg(display, c("p", "f", "h"))
}

hist2d_legend_name <- function(display) {
  switch(
    display,
    f = "weighted freq",
    p = "relative freq",
    h = "density"
  )
}

hist2d_padding <- function(value) {
  if (!is.finite(value) || value == 0) {
    return(0.5)
  }
  max(abs(value) * 0.05, 0.5)
}

merge_hist2d_breaks <- function(breaks, tau) {
  tau <- as.numeric(tau)
  if (length(tau) != 1 || !is.finite(tau) || tau < 0) {
    stop("ERROR : tau must be a single non-negative number.")
  }
  if (tau == 0 || length(breaks) <= 2) {
    return(breaks)
  }

  groups <- list()
  current_group <- breaks[1]
  for (idx in 2:length(breaks)) {
    if ((breaks[idx] - breaks[idx - 1]) < tau) {
      current_group <- c(current_group, breaks[idx])
    } else {
      groups[[length(groups) + 1]] <- current_group
      current_group <- breaks[idx]
    }
  }
  groups[[length(groups) + 1]] <- current_group

  merged <- vapply(
    seq_along(groups),
    FUN.VALUE = numeric(1),
    FUN = function(i) {
      group <- groups[[i]]
      if (i == 1) {
        min(group)
      } else if (i == length(groups)) {
        max(group)
      } else {
        mean(group)
      }
    }
  )
  merged <- sort(unique(merged))

  if (length(merged) < 2) {
    return(c(min(breaks), max(breaks)))
  }
  merged
}

build_hist2d_breaks <- function(interval_column, bins, method, tau = 0) {
  mins <- interval_column$min
  maxs <- interval_column$max
  lower <- min(mins)
  upper <- max(maxs)

  if (method == "equal-bin") {
    if (bins < 1) {
      stop("ERROR : The number of bins must be at least 1.")
    }
    if (lower == upper) {
      pad <- hist2d_padding(lower)
      return(seq(lower - pad, upper + pad, length.out = bins + 1))
    }
    return(seq(lower, upper, length.out = bins + 1))
  }

  breaks <- sort(unique(c(mins, maxs)))
  breaks <- merge_hist2d_breaks(breaks, tau = tau)
  if (length(breaks) < 2) {
    pad <- hist2d_padding(lower)
    breaks <- c(lower - pad, upper + pad)
  }
  breaks
}

interval_overlap_ratio <- function(lower, upper, breaks) {
  left <- breaks[-length(breaks)]
  right <- breaks[-1]

  if (upper - lower == 0) {
    return(as.numeric(lower >= left & lower <= right))
  }

  overlap <- pmin(right, upper) - pmax(left, lower)
  overlap[overlap < 0] <- 0
  overlap / (upper - lower)
}

build_hist2d_frequency_matrix <- function(x_interval, y_interval, x_breaks, y_breaks) {
  x_cells <- length(x_breaks) - 1
  y_cells <- length(y_breaks) - 1
  n <- length(x_interval$min)
  freq_rectangle <- matrix(0, nrow = x_cells, ncol = y_cells)

  for (obs in seq_len(n)) {
    fx <- interval_overlap_ratio(
      lower = x_interval$min[obs],
      upper = x_interval$max[obs],
      breaks = x_breaks
    )
    fy <- interval_overlap_ratio(
      lower = y_interval$min[obs],
      upper = y_interval$max[obs],
      breaks = y_breaks
    )

    if (any(fx > 0) && any(fy > 0)) {
      freq_rectangle <- freq_rectangle + tcrossprod(fx, fy)
    }
  }

  freq_rectangle
}

build_hist2d_frame <- function(freq_rectangle, x_breaks, y_breaks, n) {
  grid <- expand.grid(
    x_index = seq_len(length(x_breaks) - 1),
    y_index = seq_len(length(y_breaks) - 1),
    KEEP.OUT.ATTRS = FALSE
  )

  grid$f <- c(freq_rectangle)
  grid$x1 <- x_breaks[grid$x_index]
  grid$x2 <- x_breaks[grid$x_index + 1]
  grid$y1 <- y_breaks[grid$y_index]
  grid$y2 <- y_breaks[grid$y_index + 1]
  grid$area <- (grid$x2 - grid$x1) * (grid$y2 - grid$y1)
  grid$p <- grid$f / n
  grid$h <- ifelse(grid$area > 0, grid$p / grid$area, 0)
  grid$xmid <- (grid$x1 + grid$x2) / 2
  grid$ymid <- (grid$y1 + grid$y2) / 2

  grid[, c("f", "p", "h", "x1", "x2", "y1", "y2", "area", "xmid", "ymid")]
}

build_hist2d_data <- function(iData, attr1, attr2, xBins, yBins, method, tau = 0) {
  x_breaks <- build_hist2d_breaks(iData[[attr1]], xBins, method, tau = tau)
  y_breaks <- build_hist2d_breaks(iData[[attr2]], yBins, method, tau = tau)
  freq_rectangle <- build_hist2d_frequency_matrix(
    x_interval = iData[[attr1]],
    y_interval = iData[[attr2]],
    x_breaks = x_breaks,
    y_breaks = y_breaks
  )

  list(
    xBreaks = x_breaks,
    yBreaks = y_breaks,
    xBins = length(x_breaks) - 1,
    yBins = length(y_breaks) - 1,
    freq = freq_rectangle,
    data = build_hist2d_frame(freq_rectangle, x_breaks, y_breaks, n = nrow(iData))
  )
}

hist2d_assign_display <- function(hist2d_frame, display) {
  display <- match_hist2d_display(display)
  hist2d_frame$display_value <- hist2d_frame[[display]]
  hist2d_frame$display_metric <- display
  hist2d_frame
}

format_hist2d_labels <- function(values) {
  gsub(
    pattern = "^0",
    replacement = "",
    x = format(round(values, 2), trim = TRUE, nsmall = 2)
  )
}

build_hist2d_table <- function(hist2d_data, attr1, attr2, display = "f") {
  display <- match_hist2d_display(display)
  metric_matrix <- matrix(
    round(hist2d_data$data[[display]], 3),
    nrow = hist2d_data$xBins,
    ncol = hist2d_data$yBins
  )

  c_names <- paste0(
    "[",
    paste(
      round(hist2d_data$yBreaks[-length(hist2d_data$yBreaks)], 2),
      round(hist2d_data$yBreaks[-1], 2),
      sep = ":"
    ),
    "]"
  )
  r_names <- paste0(
    "[",
    paste(
      round(hist2d_data$xBreaks[-length(hist2d_data$xBreaks)], 2),
      round(hist2d_data$xBreaks[-1], 2),
      sep = ":"
    ),
    "]"
  )

  if (display == "h") {
    rownames(metric_matrix) <- r_names
    colnames(metric_matrix) <- c_names
    return(as.data.frame(metric_matrix))
  }

  s1 <- apply(metric_matrix, 1, sum)
  s2 <- apply(metric_matrix, 2, sum)
  total_value <- sum(metric_matrix)
  metric_matrix <- cbind(metric_matrix, s1)
  metric_matrix <- rbind(metric_matrix, c(s2, total_value))

  metric_label <- hist2d_legend_name(display)
  rownames(metric_matrix) <- c(r_names, paste0(metric_label, " of ", attr1))
  colnames(metric_matrix) <- c(c_names, paste0(metric_label, " of ", attr2))

  as.data.frame(metric_matrix)
}

#' @name facedata
#' @aliases facedata
#' @title Face Data Example
#' @description
#' Symbolic data matrix with all the variables of interval type.
#' @usage data('facedata')
#' @format
#' $I;AD;AD;$I;BC;BC;......... \cr
#'
#' HUS1;$I;168.86;172.84;$I;58.55;63.39;.........\cr
#' HUS2;$I;169.85;175.03;$I;60.21;64.38;.........\cr
#' HUS3;$I;168.76;175.15;$I;61.4;63.51;.........\cr
#' INC1;$I;155.26;160.45;$I;53.15;60.21;.........\cr
#' INC2;$I;156.26;161.31;$I;51.09;60.07;.........\cr
#' INC3;$I;154.47;160.31;$I;55.08;59.03;.........\cr
#' ISA1;$I;164;168;$I;55.01;60.03;.........\cr
#' ISA2;$I;163;170;$I;54.04;59;.........\cr
#' ISA3;$I;164.01;169.01;$I;55;59.01;.........\cr
#' JPL1;$I;167.11;171.19;$I;61.03;65.01;.........\cr
#' JPL2;$I;169.14;173.18;$I;60.07;65.07;.........\cr
#' JPL3;$I;169.03;170.11;$I;59.01;65.01;.........\cr
#' KHA1;$I;149.34;155.54;$I;54.15;59.14;.........\cr
#' KHA2;$I;149.34;155.32;$I;52.04;58.22;.........\cr
#' KHA3;$I;150.33;157.26;$I;52.09;60.21;.........\cr
#' LOT1;$I;152.64;157.62;$I;51.35;56.22;.........\cr
#' LOT2;$I;154.64;157.62;$I;52.24;56.32;.........\cr
#' LOT3;$I;154.83;157.81;$I;50.36;55.23;.........\cr
#' PHI1;$I;163.08;167.07;$I;66.03;68.07;.........\cr
#' PHI2;$I;164;168.03;$I;65.03;68.12;.........\cr
#' PHI3;$I;161.01;167;$I;64.07;69.01;.........\cr
#' ROM1;$I;167.15;171.24;$I;64.07;68.07;.........\cr
#' ROM2;$I;168.15;172.14;$I;63.13;68.07;.........\cr
#' ROM3;$I;167.11;171.19;$I;63.13;68.03;.........\cr
#' @references
#' Billard, Lynne and Diday, Edwin (2006).
#' \emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
#' Chichester, UK: John Wiley and Sons.
#' @examples
#' \donttest{
#' data(facedata)
#' ggInterval_hist(facedata, aes(x = AD))
#' }
#' @keywords datasets
#' @source Adapted from \code{RSDA::facedata}; see
#'   \url{https://CRAN.R-project.org/package=RSDA}.
"facedata"


#' @name Cardiological
#' @title Cardiological data example
#' @description Cardiological interval data example.
#' @usage data(Cardiological)
#' @references Billard, Lynne and Diday, Edwin (2006).
#' \emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
#' Chichester, UK: John Wiley and Sons.
#' @examples
#' data(Cardiological)
#' ggInterval_indexplot(Cardiological, aes(x = Syst))
#' @keywords datasets
#' @source Adapted from \code{RSDA::Cardiological}; see
#'   \url{https://CRAN.R-project.org/package=RSDA}.
"Cardiological"


#' @name abalone.i
#' @title abalone.i data example
#' @description abalone.i interval data example.
#' @usage data(abalone.i)
#' @examples
#' data(abalone.i)
#' ggInterval_indexplot(abalone.i, aes(x = Length))
#' @keywords datasets
#' @source Adapted from \code{MAINT.Data::AbaloneIdt}; the underlying
#'   Abalone data are from the UCI Machine Learning Repository.
"abalone.i"

#' @name blood.i
#' @title blood.i data example
#' @description blood.i interval data example.
#' @usage data(blood.i)
#' @references Billard, Lynne and Diday, Edwin (2006).
#' \emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
#' Chichester, UK: John Wiley and Sons.
#' @examples
#' data(blood.i)
#' ggInterval_MMplot(blood.i, aes(x = Hematocrit))
#' @keywords datasets
#' @source Adapted from \code{HistDAWass::BLOOD}; see
#'   \url{https://CRAN.R-project.org/package=HistDAWass}.
"blood.i"

#' @name oils
#' @title oils data example
#' @description oils interval data example.
#' @usage data(oils)
#' @references Cazes, Pierre; Chouakria, Assia; Diday, Edwin; and
#' Schektman, Youri (1997). Extension de l'analyse en composantes principales
#' a des donnees de type intervalle. \emph{Revue de Statistique Appliquee},
#' 45(3), 5--24.
#' @examples
#' data(oils)
#' \donttest{
#' ggInterval_scatterplot(oils, aes(x = GRA, y = IOD))
#' }
#' @keywords datasets
#' @source Adapted from \code{RSDA::oils}; see
#'   \url{https://CRAN.R-project.org/package=RSDA}.
"oils"

#' @name mushroom
#' @title mushroom data example
#' @description mushroom interval data example.
#' @usage data(mushroom)
#' @references Billard, Lynne and Diday, Edwin (2006).
#' \emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
#' Chichester, UK: John Wiley and Sons.
#' @examples
#' data(mushroom)
#' \donttest{
#' ggInterval_scatterplot(mushroom, aes(x = Cap.Widths, y = Stipe.Lengths))
#' }
#' @keywords datasets
#' @source Adapted from \code{RSDA::mushroom}; see
#'   \url{https://CRAN.R-project.org/package=RSDA}.
"mushroom"

#' @name Environment
#' @title Environment data example
#' @description Environment interval and modal data example.
#' @usage data(Environment)
#' @examples
#' data(Environment)
#' ggInterval_radarplot(Environment,
#'  plotPartial = 2,
#'  showLegend = FALSE,
#'  base_circle = TRUE,
#'  base_lty = 2,
#'  addText = FALSE)
#' @keywords datasets
"Environment"

#' @name iris.i
#' @title iris.i data example
#' @description iris.i interval data example.
#' @usage data(iris.i)
#' @examples
#' data(iris.i)
#' ggInterval_indexplot(iris.i, aes(x = Sepal.Length))
#' @keywords datasets
"iris.i"

#' @name mtcars.i
#' @title mtcars.i data example
#' @description mtcars.i interval and modal data example.
#' @usage data(mtcars.i)
#' @examples
#' data(mtcars.i)
#' ggInterval_indexplot(mtcars.i, aes(x = mpg))
#' @keywords datasets
"mtcars.i"


#' @name Cardiological2
#' @title Cardiological data example
#' @description Cardiological interval data example.
#' @usage data(Cardiological2)
#' @references Billard, Lynne and Diday, Edwin (2006).
#' \emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
#' Chichester, UK: John Wiley and Sons.
#' @examples
#' data(Cardiological2)
#' ggInterval_indexplot(Cardiological2, aes(x = Syst))
#' @keywords datasets
#' @source Adapted from \code{RSDA::Cardiological2}; see
#'   \url{https://CRAN.R-project.org/package=RSDA}.
"Cardiological2"

#' @name ggInterval_scatterMatrix
#' @title Scatterplot matrix for interval-valued data
#' @description Visualize all continuous interval-valued variables with
#' a matrix of pairwise interval scatterplots. Each off-diagonal panel
#' shows interval rectangles for one variable pair, whereas each diagonal
#' panel displays the variable name. This function automatically filters
#' out non-interval variables and plots all remaining continuous interval
#' variables, so explicit \code{x} and \code{y} mappings are not required.
#' It is not recommended to apply the function to too many variables
#' because the full pairwise matrix becomes computationally expensive and
#' visually crowded.
#' @import rlang ggplot2 tidyverse
#' @importFrom grDevices gray.colors
#' @importFrom grDevices grey.colors
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame,which will be automatically convert to ggInterval
#' data.
#' @param mapping Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' @param showLegend whether show the legend.
#' @param borderLinewidth Numeric border width used for the interval rectangles.
#' @return Return a ggplot2 object.
#' @usage ggInterval_scatterMatrix(data = NULL, mapping = aes(NULL),
#' showLegend = FALSE, borderLinewidth = 0.08)
#' @examples
#' mydata <- ggInterval::facedata
#' ggInterval_scatterMatrix(mydata[, 1:3])
#' ggInterval_scatterMatrix(
#'   mydata[, 1:3],
#'   aes(fill = "black", alpha = 0.2),
#'   borderLinewidth = 0.15
#' )
#' @export
ggInterval_scatterMatrix <- function(data = NULL,
                                 mapping = aes(NULL),
                                 showLegend = FALSE,
                                 borderLinewidth = 0.08) {
  #data preparing
  . <- NULL
  if (!is.numeric(borderLinewidth) || length(borderLinewidth) != 1 ||
      !is.finite(borderLinewidth) || borderLinewidth < 0) {
    stop("ERROR : borderLinewidth must be a single non-negative number.")
  }
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  #remove user's x,y input,remain Aesthetic
  if ((!is.null(this.x)) && (!is.null(this.y))) {
    #both have value
    usermapping <- mapping[-c(1, 2)]
  } else if ((!is.null(this.x)) || (!is.null(this.y))) {
    #only one value
    usermapping <- mapping[-1]
  }
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  myRowNames <- rownames(iData)
  #preparing data
  p <- dim(iData)[2]
  n <- dim(iData)[1]
  numericData <- unlist(lapply(iData[, 1:p] , FUN = RSDA::is.sym.interval))
  iData <- iData[, numericData, drop = FALSE]
  
  #test big o
  if (p > 8 & p <= 20) {
    warning("It is not recommended number of variables are greater than 8.")
  } else if (p > 20) {
    stop(
      "The number of variables are too large to visualize clearly.
         Suggested input variables less than 4. ex. data[,1:4]"
    )
  }
  if (showLegend == T && n > 50) {
    stop("Suggest set showLegend to FALSE in the situation
         that your observations are large.")
  } else if (showLegend == F && n > 3000) {
    stop("Out of time limits.")
  }
  
  
  #now iData only have numeric data
  if (dim(iData)[2] < dim(data)[2]) {
    p <- dim(iData)[2]
    n <- dim(iData)[1]
    warning("Ignore non-numeric data.")
  }
  
  plotData <- NULL
  for (i in 1:p) {
    for (u in 1:p) {
      if (i != u) {
        plotData <- rbind(
          plotData,
          data.frame(
            x1 = iData[[i]]$min,
            x2 = iData[[i]]$max,
            y1 = iData[[u]]$min,
            y2 = iData[[u]]$max,
            xv = colnames(iData)[i],
            yv = colnames(iData)[u],
            Concepts = myRowNames,
            isPlot = T,
            textXY = 0
          )
        )
      } else{
        plotData <- rbind(
          plotData,
          data.frame(
            x1 = 0,
            x2 = 0,
            y1 = 0,
            y2 = 0,
            xv = colnames(iData)[i],
            yv = colnames(iData)[u],
            Concepts = "no concepts",
            isPlot = F,
            textXY = (max(iData[[i]]$max) + min(iData[[i]]$min)) /
              2
          )
        )
      }
    }
  }
  
  #build Aesthetic (mapping)
  xyLocation <- c(which(names(mapping) == "x"), which(names(mapping) == "y"))
  if (length(xyLocation) != 0) {
    usermapping <- mapping[-xyLocation] #Aesthetic without x,y
  } else{
    usermapping <- mapping
  }
  
  mymapping <- list(
    data = . %>% dplyr::filter(.data$isPlot),
    mapping = aes(
      xmin = .data$x1,
      xmax = .data$x2,
      ymin = .data$y1,
      ymax = .data$y2,
      fill = .data$Concepts,
      alpha = 0.5
    ),
    col = "black",
    linewidth = borderLinewidth
  )
  allmapping <- as.list(structure(as.expression(c(
    usermapping, mymapping
  )), class = "uneval"))
  
  
  
  #start plot
  result <- ggplot(data = plotData, aes(.data$x1, .data$y1)) +
    do.call(geom_rect, allmapping) +
    scale_fill_manual(name = "Concept",
                      values = gray.colors(n),
                      labels = myRowNames) +
    geom_text(
      data = . %>% dplyr::filter(!.data$isPlot),
      aes(
        x = .data$textXY,
        y = .data$textXY,
        label = .data$xv
      ),
      size = 12
    ) +
    guides(colour = "none", alpha = "none") +
    facet_grid(.data$yv ~ .data$xv, scales = "free") +
    labs(x = "", y = "")
  
  if (showLegend) {
    return(result)
  } else{
    return(result + theme(legend.position = "None"))
  }
}

#' @name ggInterval_indexImage
#' @title Index image plot for interval-valued data
#' @description Visualize interval-valued observations with color strips.
#' For a single variable, the display acts as a color-based analogue of an
#' index plot. When \code{plotAll = TRUE}, the function produces a
#' multivariate image plot across all continuous interval-valued variables.
#' @import ggplot2 ggthemes
#' @importFrom RSDA is.sym.interval
#' @importFrom ggpubr ggarrange
#' @param data A ggInterval object. It can also be either RSDA object or
#' classical data frame,which will be automatically convert to ggInterval
#' data.
#' @param mapping  Set of aesthetic mappings created by aes() or aes_().
#' If specified and inherit. aes = TRUE (the default),
#' it is combined with the default mapping at the top level of
#' the plot. You must supply mapping if there is no plot mapping.
#' @param column_condition Logical. If \code{TRUE}, the color scale is
#' applied separately within each variable (column condition). If
#' \code{FALSE}, one common color scale is applied to the whole matrix
#' (matrix condition).
#' @param full_strip Logical. If \code{TRUE}, each strip spans the full
#' plotting width. If \code{FALSE}, strip widths reflect the interval ranges.
#' @param plotAll Logical. If \code{TRUE}, produce a multivariate image plot
#' for all continuous interval-valued variables. The default is \code{FALSE}.
#' @return Return a ggplot2 object.
#' @usage ggInterval_indexImage(data = NULL, mapping = aes(NULL),
#' column_condition = TRUE, full_strip = FALSE, plotAll = FALSE)
#' @examples
#' mydata <- ggInterval::facedata
#' p <- ggInterval_indexImage(mydata, aes(x = AD),
#'   full_strip = TRUE, column_condition = TRUE)
#' # Recommend adding coord_flip() to make the single-variable display clearer.
#' p + coord_flip()
#' ggInterval_indexImage(
#'   mydata,
#'   plotAll = TRUE,
#'   full_strip = TRUE,
#'   column_condition = FALSE
#' ) +
#'   scale_colour_distiller(palette = "Blues", direction = 1)
#' ggInterval_indexImage(mydata, plotAll = TRUE, full_strip = FALSE)
#'
#' @export
ggInterval_indexImage <- function(data = NULL,
                                  mapping = aes(NULL),
                                  column_condition = TRUE,
                                  full_strip = FALSE,
                                  plotAll = FALSE) {
  #data preparing
  argsNum <- length(mapping)
  args <- lapply(mapping[1:argsNum], FUN = rlang::get_expr)
  this.x <- args$x
  this.y <- args$y
  
  #test data illegal
  ggSymData <- testData(data)
  iData <- ggSymData$intervalData
  myHeatMapNames <- rownames(iData)
  if (plotAll) {
    if (!is.null(this.x) | !is.null(this.y)) {
      warning("Using heatmap presentation cannot specify variables.")
    }
  } else{
    testXY(iData, this.x, this.y)
  }
  
  p <- dim(iData)[2]
  n <- dim(iData)[1]
  #test big o
  if (full_strip == FALSE & dim(iData)[1] > 200) {
    stop("Out of time limits")
  }
  if (full_strip == TRUE & dim(iData)[1] > 50) {
    stop("Out of time limits")
  }
  #start process
  datasetMin <- min(ggSymData$statisticsDF$min)
  datasetMax <- max(ggSymData$statisticsDF$max)
  #allDataMean <- mean(unlist(ggSymData$statisticsDF$min, ggSymData$statisticsDF$max))
  
  
  with(data, {
    #add heatmap
    if (plotAll) {
      #set adjust ggarrange width scale
      widthScale <- 7
      
      #get numerical data
      numericData <- unlist(lapply(data.frame(iData[1:dim(iData)[2]]) , FUN = is.sym.interval))
      iData <- iData[, which(numericData)]
      
      #scale
      if (column_condition) {
        iData <- scale_sym_table(iData, n, p)$intervalData
      }
      
      d <- data.frame(NULL)
      for (i in colnames(iData)) {
        a <- iData[[i]]$min
        b <- iData[[i]]$max
        adjustStrip <- min(b - a) / 50
        d <- rbind(d, data.frame(
          buildPlotData(a, b, adjustStrip, n, full_strip, datasetMax),
          group = factor(i)
        ))
      }
    } else{
      #end headmap
      
      #get attr
      if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.x)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.x)
        )))
        attr <- names(attr)
      } else if (any(unlist(lapply(
        as.data.frame(data[, 1:p]),
        FUN = identical,
        x = eval(this.y)
      )))) {
        attr <- which(unlist(lapply(
          as.data.frame(data[, 1:p]),
          FUN = identical,
          x = eval(this.y)
        )))
        attr <- names(attr)
      } else{
        stop("ERROR : Cannot find variables in aes(...)")
      }
      if (p == 1) {
        attr = colnames(data)
      }
      #test attribute illegal
      if (all(!is.numeric(data[[attr]]) , !RSDA::is.sym.interval(data[[attr]]))) {
        stop("ERROR : Variables in Min-Max Plot can only be numeric.")
      }
      
      #build data frame for ggplot
      a <- iData[[attr]]$min
      b <- iData[[attr]]$max
      
      #whether full strip (if partial strip)
      #debug for separate strip "adjustStrip"
      adjustStrip <- min(b - a) / 50
    }
    
    if (!full_strip) {
      if (!plotAll) {
        d <- buildPlotData(a, b, adjustStrip, n, full_strip, datasetMax)
      }
      #whether column condition
      if (!column_condition) {
        midp <- (datasetMin + datasetMax) / 2
        #midp <- allDataMean
        #midp<-(max(d$y)+min(d$y))/2
        NAME <- "Matrix Condition"
        myColScale <- "scale_color_gradient2(low = 'blue', mid = 'yellow', high = 'red',
                              midpoint = midp,
                              name = 'Value',
                              breaks=c(min(d$y),max(d$y)),
                              labels = c(round(min(d$y)),round(max(d$y))))"
        #breaks=c(min(d$y),max(d$y)),
        #labels = c(round(min(d$y)),round(max(d$y))))
        
        if (plotAll) {
          base <- ggplot(d, aes(
            x = x,
            xend = xend,
            y = y,
            yend = y,
            color = y
          )) +
            geom_segment(linewidth = 3) + facet_grid(. ~ group) +
            eval(parse(text = myColScale)) +
            coord_flip() + scale_x_continuous(breaks = 1:n, label = myHeatMapNames) +
            labs(x = "Concepts", y = "Values") +
            theme(legend.position = "bottom",
                  legend.title = element_blank())
          
          return(base)
        }
      } else{
        #column control
        #midp<-(max(d$y)+min(d$y))/2
        midp <- mean(d$y)
        #midp<-(datasetMin+datasetMax)/2
        NAME <- "Column Condition"
        myColScale <- "scale_color_gradient2(low = 'blue', mid = 'yellow', high = 'red',
                              midpoint = midp,
                              name = 'Value',
                              breaks=c(min(d$y),max(d$y)),
                              labels = c(round(min(d$y)),round(max(d$y))))"
        #c(round(datasetMin),round(datasetMax)))
        
        if (plotAll) {
          #midp<-(max(d$y)+min(d$y))/2
          midp <- 0
          p <- ggplot(d, aes(
            x = x,
            xend = xend,
            y = y,
            yend = yend,
            color = y
          ))
          
          p <- p + geom_segment(linewidth = 3) +
            scale_color_gradient2(
              low = 'blue',
              mid = 'yellow',
              high = 'red',
              midpoint = midp,
              breaks = c(min(d$y), max(d$y)),
              labels = c("min", "max")
            ) +
            scale_x_continuous(breaks = c(1:n), labels = myHeatMapNames) +
            coord_flip() +
            guides(fill = "none", alpha = "none") +
            facet_grid(. ~ group) +
            labs(x = "Concepts", y = "") +
            theme(legend.position = "bottom",
                  legend.title = element_blank()) +
            scale_y_continuous(breaks = NULL)
          return(p)
          
          # base <- ggplot(d, aes(x = x, xend = xend, y = y, yend = y,color=y)) +
          #   geom_segment(size = 3) + facet_grid(. ~ group) +
          #   eval(parse(text=myColScale))+
          #   coord_flip()+scale_x_continuous(breaks = 1:n, label=myHeatMapNames)+
          #   labs(x="Concepts",y="Values")+
          #   theme(legend.position="bottom",
          #         legend.title = element_blank())
          #
          # return(base)
          
          # u<-1
          # myp <- NULL
          # for(ele in levels(d$group)){
          #   tempData <- dplyr::filter(d, d$group == ele)
          #   myp[[u]] <- myplot(tempData, u, NAME, myHeatMapNames, n, ele,full_strip)
          #   u<-u+1
          # }
          #
          # #v <- seq(min(d$y),max(d$y),(max(d$y)-min(d$y))/(1000-1))
          # #adjust grid
          # maxChar <- max(unlist(lapply(myHeatMapNames, nchar)))
          # adjustWidth <- 1 + sqrt(maxChar)/widthScale
          # base <- ggarrange(plotlist=myp, nrow=1,
          #                   widths = c(adjustWidth,rep(1, p-1)))
          #
          # v <- seq(1,1000)
          # return(ggarrange(base, buildColLegend(v),nrow=2,heights=c(8,1)))
        }
      }
      
      #plot
      ggplot(d, aes(
        x = x,
        xend = xend,
        y = y,
        yend = yend,
        color = y
      )) +
        geom_segment(linewidth = 3) + eval(parse(text = myColScale)) +
        labs(y = attr,
             x = "",
             title = paste0("Index Image-", NAME)) +
        scale_x_continuous(breaks = c(1:n), labels = rownames(iData))
    }
    else{
      # if full strip
      #adjust
      #newN<-sort(runif((n)*2,1,n))
      if (!plotAll) {
        d <- buildPlotData(a, b, adjustStrip, n, full_strip, datasetMax)
      }
      
      #whether column condition
      if (!column_condition) {
        midp <- (datasetMin + datasetMax) / 2
        #midp <- allDataMean
        NAME <- "Matrix Condition"
        myColScale <- "scale_color_gradient2(low = 'blue', mid = 'yellow', high = 'red',
                              midpoint = midp,
                              name = 'Value',
                              breaks=c(min(d$value),max(d$value)),
                              labels = c(round(min(d$value)),round(max(d$value))))"
        
        if (plotAll) {
          base <- ggplot(d, aes(
            x = x,
            xend = xend,
            y = y,
            yend = y,
            color = value
          )) +
            geom_segment(linewidth = 3) + facet_grid(. ~ group) +
            eval(parse(text = myColScale)) +
            coord_flip() + scale_x_continuous(breaks = 1:n, label = myHeatMapNames) +
            labs(x = "Concepts", y = "Values") +
            theme(legend.position = "bottom",
                  legend.title = element_blank())
          return(base)
        }
        
      } else{
        # column condition
        midp <- (max(d$value) + min(d$value)) / 2
        NAME <- "Column Condition"
        myColScale <- "scale_color_gradient2(low = 'blue', mid = 'yellow', high = 'red',
                              midpoint = midp,
                              name = 'Value',
                              breaks=c(min(d$value),max(d$value)),
                              labels = c(round(min(d$value)),round(max(d$value))))"
        
        if (plotAll) {
          midp <- 0
          p <- ggplot(d, aes(
            x = x,
            xend = xend,
            y = y,
            yend = y,
            color = value
          ))
          
          p <- p + geom_segment(linewidth = 3) +
            scale_color_gradient2(
              low = 'blue',
              mid = 'yellow',
              high = 'red',
              midpoint = midp,
              breaks = c(min(d$value), max(d$value)),
              labels = c("min", "max")
            ) +
            scale_x_continuous(breaks = c(1:n), labels = myHeatMapNames) +
            coord_flip() +
            guides(fill = "none", alpha = "none") +
            facet_grid(. ~ group) +
            labs(x = "Concepts", y = "") +
            theme(legend.position = "bottom",
                  legend.title = element_blank()) +
            scale_y_continuous(breaks = NULL)
          return(p)
          
          
          
          
          # u<-1
          # myp <- NULL
          # for(ele in levels(d$group)){
          #   tempData <- dplyr::filter(d, d$group == ele)
          #   myp[[u]] <- myplot(tempData, u, NAME, myHeatMapNames, n, ele, full_strip)
          #   u<-u+1
          # }
          #
          # #v <- seq(min(d$y),max(d$y),(max(d$y)-min(d$y))/(1000-1))
          # #adjust grid
          # maxChar <- max(unlist(lapply(myHeatMapNames, nchar)))
          # adjustWidth <- 1 + sqrt(maxChar)/widthScale
          # base <- ggarrange(plotlist=myp, nrow=1,
          #                   widths = c(adjustWidth, rep(1, p-1)))
          #
          # v <- seq(1,1000)
          #
          # return(ggarrange(base, buildColLegend(v),nrow=2,heights=c(12,1)))
          
        }
        
      }
      
      #plot
      ggplot(data = d, aes(
        x = x,
        xend = xend,
        y = y,
        yend = y,
        color = value
      )) +
        geom_segment(linewidth = 3) +
        eval(parse(text = myColScale)) +
        labs(y = attr,
             x = "",
             title = paste0("Index Image-", NAME)) +
        scale_x_continuous(breaks = c(1:n), labels = rownames(iData)) +
        scale_y_continuous(breaks = NULL)
    }
  })
}


buildPlotData <- function(a,
                          b,
                          adjustStrip,
                          n,
                          full_strip,
                          datasetMax) {
  if (!full_strip) {
    val <- mapply(
      a,
      b,
      FUN = function(x, y)
        seq(x, y, by = adjustStrip)
    )
    y <- unlist(val)
    mid <- rep(1:n, lengths(val))
    d <- data.frame(
      x = mid - 0.4,
      xend = mid + 0.4,
      y = y,
      yend = y
    )
  } else{
    datasetMax <- ceiling(datasetMax)
    adjustCoef <- ifelse(datasetMax * n > 2000, 1, ceiling(2000 / (datasetMax *
                                                                     n)))
    
    myAdjust <- seq(1, datasetMax, (datasetMax - 1) / (datasetMax * adjustCoef -
                                                         1))
    #debug for unequal bin image
    d2 <- data.frame(
      x = rep(1:n, each = datasetMax * adjustCoef) - 0.5,
      xend = rep(1.5:(n + 0.5), each = datasetMax * adjustCoef),
      y = rep(myAdjust, n)
    )
    val2 <- mapply(
      a,
      b,
      FUN = function(x, y)
        sort(runif(datasetMax * adjustCoef, x, y))
    )
    val <- matrix(val2, ncol = 1)
    d <- data.frame(d2, value = val)
  }
  return(d)
}


# buildColLegend <- function(v){
#     newd<-data.frame(x = rep(1:1000),
#                      xend = rep(1:1000),
#                      y = rep(1, 1000),
#                      yend = rep(2, 1000),
#                      val = v)
#     p2 <- ggplot(newd,aes(x=x,xend=xend,y=y,yend=yend,col=val))+
#       geom_segment()+
#       scale_color_gradient2(low = 'blue', mid = 'yellow', high = 'red',
#                             midpoint = (min(newd$val)+max(newd$val))/2,
#                             name = 'Value',
#                             breaks=c(min(newd$val),max(newd$val)),
#                             labels = c(round(min(newd$val)),round(max(newd$val))))+
#       guides(color=F,y=F)+theme_tufte()+labs(y=NULL,x=NULL)+
#       scale_x_continuous(breaks=c(0,1000),labels = c("min","max"))
#
#     temp<-ggplot()+theme_void()
#     temp2<-ggarrange(temp,p2,temp,nrow=1,widths = c(2,1,2))
#     return(temp2)
# }


myplot <- function(tempData,
                   u,
                   NAME,
                   myHeatMapNames,
                   n,
                   ele,
                   full_strip) {
  if (full_strip) {
    midp <- (max(tempData$value) + min(tempData$value)) / 2
    p <- ggplot(
      tempData,
      aes(
        x = tempData$x,
        xend = tempData$xend,
        y = tempData$y,
        yend = tempData$y,
        color = tempData$value
      )
    )
  } else{
    midp <- (max(tempData$y) + min(tempData$y)) / 2
    p <- ggplot(
      tempData,
      aes(
        x = tempData$x,
        xend = tempData$xend,
        y = tempData$y,
        yend = tempData$yend,
        color = tempData$y
      )
    )
  }
  p <- p + geom_segment(linewidth = 3) +
    scale_color_gradient2(
      low = 'blue',
      mid = 'yellow',
      high = 'red',
      midpoint = midp
    ) +
    labs(y = "",
         x = "",
         title = paste0("Index Image-", NAME)) +
    scale_x_continuous(breaks = c(1:n), labels = myHeatMapNames) +
    coord_flip() +
    guides(fill = "none",
           col = "none",
           alpha = "none") +
    theme(plot.title = element_text(hjust = 0.5)) + ggtitle(ele)
  
  if (u != 1) {
    p <- p + theme(axis.text.y = element_blank())
  }
  
  return(p)
}
scale_sym_table <- function(d, n, p) {
  temp1 <- sapply(
    1:p,
    FUN = function(x)
      unlist(data.frame(d[[x]]))
  )
  temp2 <- apply(temp1, 2, scale)
  newd <- data.frame(temp2[1:n, ], temp2[(n + 1):(n * 2), ])
  myd <- classic2sym(newd,
                     groupby = "customize",
                     minData = temp2[1:n, ],
                     maxData = temp2[(n + 1):(n * 2), ])
  colnames(myd$intervalData) <- colnames(d)
  return(myd)
}

---
title: "Introduction to ggInterval"
subtitle: "A source-portable companion for interval-valued visualization"
author: "Bo-Syue Jiang and Han-Ming Wu"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Introduction to ggInterval}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  echo = TRUE,
  warning = FALSE,
  message = FALSE,
  fig.align = "center",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)

library(ggInterval)
library(ggplot2)
```

# Overview

`ggInterval` extends `ggplot2` for interval-valued data. The package supports
visualizations such as scatter plots, histograms, index plots, radar plots,
line plots, PCA displays, and correlation heatmaps, while keeping the
layered grammar familiar to `ggplot2` users.

This vignette focuses on source-portable examples that can be rebuilt directly
from the package. It uses built-in data, the bundled `data.csv` file for the
aggregation example, and optional sections for packages such as `HistDAWass`
and `MAINT.Data`.

# Built-in Data

```{r built_in_data}
data(facedata)
data(Environment)

facedata
Environment

Subjects <- substr(rownames(facedata), 1, 3)
Subjects
```

# Basic Usage

```{r ggplot2_workflow}
ggInterval_scatterplot(
  facedata,
  aes(x = AD, y = BC, fill = Subjects),
  showLabels = FALSE,
  col = "black"
) +
  scale_fill_brewer(palette = "Set1") +
  labs(fill = "Subjects")
```

# Aggregating Classical Data into Interval-Valued Data

```{r aggregation_examples}
breastData <- read.csv("data.csv")
breastData$X <- NULL
breastData <- dplyr::select(breastData, -id)

breastData_kmeans <- classic2sym(breastData, groupby = "kmeans", k = 5)
head(breastData_kmeans$intervalData[, 1:4])

breastData_diag <- classic2sym(breastData, groupby = "diagnosis")
head(breastData_diag$intervalData[, 1:4])
```

# Descriptive Statistics

```{r descriptive_statistics}
mean(facedata)
sd(facedata)

cov(facedata$AD, facedata$BC, method = "BD")
cor(facedata$AD, facedata$BC, method = "BD")

summary(facedata$AD)
summary(Environment[1:3, ])
```

# Univariate Plots

## Index Plots and Index Images

```{r index_plots}
ggInterval_indexplot(facedata, aes(x = AD))

ggInterval_indexplot(facedata, aes(x = AD, fill = Subjects)) +
  scale_fill_brewer(palette = "Set1") +
  geom_vline(xintercept = mean(facedata$AD), color = "darkgray") +
  labs(fill = "Subjects")

ggInterval_indexImage(facedata, aes(x = AD)) +
  coord_flip()

ggInterval_indexImage(facedata, aes(x = AD), full_strip = TRUE) +
  coord_flip()
```

```{r index_plot_ordering}
face.ip <- ggInterval_indexplot(facedata, aes(fill = Subjects), plotAll = TRUE) +
  scale_fill_brewer(palette = "Set1") +
  labs(x = "", y = "", fill = "Subjects")
face.ip

b <- ggInterval_indexplot(facedata, aes(fill = Subjects),
  plotAll = TRUE, row_order = "c") +
  scale_fill_brewer(palette = "Set1") +
  labs(x = "", y = "", fill = "Subjects")
b

cc <- ggInterval_indexplot(facedata, aes(fill = Subjects),
  plotAll = TRUE, row_order = "r") +
  scale_fill_brewer(palette = "Set1") +
  labs(x = "", y = "", fill = "Subjects")
cc
```

In the ordered `plotAll` displays, the outer y-axis labels are suppressed by
default. Here `row_order = "c"` reorders observations by interval centers
within each variable panel, while `row_order = "r"` reorders them by interval
ranges within each variable panel. Set `labels = TRUE` if you want the row
labels repeated inside each variable panel.

## Boxplots and Histograms

```{r boxplots_histograms}
ggInterval_boxplot(facedata, plotAll = TRUE, width_type = "violin-like") +
  theme(legend.position = "bottom", axis.text.x = element_blank())

ggInterval_boxplot(facedata, plotAll = TRUE, width_type = "side-by-side") +
  theme(legend.position = "bottom", axis.text.x = element_blank())

ggInterval_boxplot(facedata, aes(x = AD), width_type = "quantile-depth")

hist.obj.equal <- ggInterval_hist(facedata, plotAll = TRUE, bins = 10)
print(hist.obj.equal)
hist.obj.equal$`Table AD`

hist.obj.unequal <- ggInterval_hist(
  facedata,
  plotAll = TRUE,
  method = "unequal-bin"
)
print(hist.obj.unequal)

ggInterval_hist(facedata, aes(x = AD), method = "equal-bin", bins = 10)

ggInterval_hist(facedata, aes(x = AD), method = "unequal-bin")
```

## Line Plot

```{r lineplot_stocklike}
if (!requireNamespace("TTR", quietly = TRUE)) {
  stop("Package 'TTR' is required for this example.")
}

data("ttrc", package = "TTR")
stock.data <- subset(
  ttrc[, c("Date", "Close", "Low", "High")],
  format(Date, "%Y-%m") %in% c("1985-01", "1985-02", "1985-03")
)
stock.data$Month <- factor(
  month.abb[as.integer(format(stock.data$Date, "%m"))],
  levels = month.abb[1:3]
)
stock.data$Day <- as.integer(format(stock.data$Date, "%d"))

stock.data.LH <- classic2sym(
  stock.data,
  groupby = "customize",
  minData = stock.data$Low,
  maxData = stock.data$High
)

ggInterval_lineplot(stock.data.LH, aes(y = V1, x = Day), barWidth = 0.6) +
  geom_point(aes(y = Close), shape = 21, fill = "#D95F02",
             color = "black", size = 1.6, stroke = 0.2) +
  coord_cartesian(xlim = c(1, 31), expand = FALSE) +
  facet_wrap(~Month, ncol = 1, scales = "free_y") +
  scale_x_continuous(breaks = c(1, 8, 15, 22, 29)) +
  labs(title = "Interval-valued Line Plot",
       x = "Day of month", y = "Price") +
  ggthemes::theme_economist() +
  theme(strip.text = element_text(face = "bold"))
```

## Min-Max Plot

```{r minmax_plot}
mm.plot <- function(x) {
  plot.var <<- names(facedata)[x]
  ggInterval_MMplot(facedata, aes(facedata[[plot.var]], size = 2)) +
    coord_fixed(ratio = 1) +
    theme(legend.position = "none")
}

mm.plot.list <- lapply(seq_along(facedata), mm.plot)
gridExtra::marrangeGrob(mm.plot.list, nrow = 2, ncol = 3, top = "")
```

## Center-Range Plot

```{r centerrange_plot}
cr_aspect_ratio <- 5

ggInterval_CRplot(facedata, aes(size = 1.5), plotAll = TRUE) +
  coord_fixed(ratio = cr_aspect_ratio)

facedata.scale <- scale(facedata)$intervalData
ggInterval_CRplot(facedata.scale, aes(size = 1.5), plotAll = TRUE) +
  coord_fixed(ratio = cr_aspect_ratio)
```

# Bivariate Plots

## Scatter Plot with Label Controls

```{r scatterplot_labels}
ggInterval_scatterplot(
  facedata,
  aes(x = BC, y = AD, fill = Subjects),
  showLabels = TRUE,
  labelSize = 2.6,
  labelPosition = "topright",
  labelNudgeX = -0.2,
  labelNudgeY = 0.15,
  checkOverlap = FALSE,
  color = "black"
) +
  scale_fill_brewer(palette = "Set1") +
  scale_x_continuous(expand = expansion(mult = c(0.08, 0.12))) +
  scale_y_continuous(expand = expansion(mult = c(0.06, 0.10))) +
  labs(fill = "Subjects")
```

## 2D Histograms

```{r hist2d_equal}
face.2dh.equal <- ggInterval_2Dhist(
  facedata,
  aes(x = BC, y = AD, col = "white"),
  method = "equal-bin",
  xBins = 10,
  yBins = 10,
  display = "p",
  palette = "Blues",
  direction = 1,
  cell_labels = TRUE
)$plot

face.2dh.equal +
  coord_fixed(ratio = 1)
```

```{r hist2d_unequal}
face.2dh.unequal <- ggInterval_2Dhist(
  facedata,
  aes(x = BC, y = AD, col = "white"),
  method = "unequal-bin",
  display = "p",
  palette = "Blues",
  direction = 1,
  tau = 0.5
)$plot

face.2dh.unequal +
  coord_fixed(ratio = 1)
```

# Matrix Displays

## Scatter Matrix

```{r scatter_matrix}
ggInterval_scatterMatrix(facedata)

ggInterval_scatterMatrix(facedata, aes(fill = "steelblue", alpha = 0.3))
```

## 2D Histogram Matrices

```{r hist2d_matrix_equal}
ggInterval_2DhistMatrix(
  facedata,
  aes(col = "white"),
  method = "equal-bin",
  xBins = 10,
  yBins = 10,
  display = "p",
  removeZero = TRUE,
  palette = "Blues",
  direction = 1,
  cell_labels = FALSE
)
```

```{r hist2d_matrix_unequal}
ggInterval_2DhistMatrix(
  facedata,
  aes(col = "white"),
  method = "unequal-bin",
  display = "p",
  palette = "Blues",
  direction = 1,
  tau = 0.5,
  removeZero = TRUE,
  cell_labels = FALSE
)
```

## Image Plots

```{r image_plots}
ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = TRUE,
  column_condition = FALSE
) +
  scale_colour_distiller(palette = "Blues", direction = 1) +
  labs(x = "Subjects") +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank())

ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = TRUE
) +
  scale_colour_distiller(palette = "Spectral") +
  labs(x = "Subjects")

ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = FALSE,
  column_condition = FALSE
) +
  scale_colour_distiller(palette = "Blues", direction = 1) +
  labs(x = "Subjects") +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank())

ggInterval_indexImage(
  facedata,
  plotAll = TRUE,
  full_strip = FALSE
) +
  scale_colour_distiller(palette = "Spectral") +
  labs(x = "Subjects")
```

## Radar Plots

```{r radar_plots}
library(ggthemes)

ggInterval_radarplot(
  Environment,
  plotPartial = c(4, 6),
  showLegend = FALSE,
  base_circle = FALSE,
  base_lty = 1,
  addText = FALSE,
  addText_modal = FALSE
) +
  scale_fill_manual(values = c("darkred", "darkblue")) +
  scale_color_manual(values = c("darkred", "darkblue")) +
  labs(title = "") +
  theme_hc()

ggInterval_radarplot(
  Environment,
  plotPartial = c(4, 6),
  showLegend = FALSE,
  base_circle = FALSE,
  base_lty = 1,
  addText = FALSE,
  addText_modal = FALSE,
  type = "rect"
) +
  scale_fill_manual(values = c("darkred", "darkblue")) +
  scale_color_manual(values = c("darkred", "darkblue")) +
  labs(title = "") +
  theme_hc()

ggInterval_radarplot(
  facedata,
  base_circle = FALSE,
  base_lty = 1,
  type = "quantile",
  quantileNum = 5,
  showLegend = TRUE,
  Drift = 0
) +
  scale_fill_brewer(palette = "Greys") +
  labs(title = "", fill = "Quantiles") +
  theme_hc()

ggInterval_radarplot(
  Environment,
  base_circle = FALSE,
  base_lty = 1,
  type = "quantile",
  quantileNum = 5,
  showLegend = TRUE,
  Drift = 0
) +
  scale_fill_brewer(palette = "Greys") +
  labs(title = "", fill = "Quantiles") +
  theme_hc()
```

# Customization

```{r customization_examples}
set.seed(1234567890)
facedata.tmp <- facedata
facedata.tmp$cluster <- RSDA::sym.kmeans(facedata.tmp, k = 3)$cluster

ggInterval_indexplot(facedata.tmp, aes(y = AD, fill = Subjects)) +
  geom_text(
    aes(x = seq_len(nrow(facedata.tmp)), y = .data$AD$min,
        label = rownames(facedata.tmp)),
    vjust = 1.5,
    size = 2
  ) +
  scale_fill_brewer(palette = "Set1") +
  labs(title = "Customized Index Plot Using K-Means Clusters",
       fill = "Subjects") +
  facet_grid(cols = vars(cluster), scales = "free_x", space = "free_x") +
  theme_economist_white() +
  theme(axis.title.x = element_blank(),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank())
```

# Working with Other SDA Packages

## Histogram-Valued Data via HistDAWass

This example is evaluated only when the optional package `HistDAWass` is
installed.

```{r histdawass_example, eval = requireNamespace("HistDAWass", quietly = TRUE)}
BLOOD <- HistDAWass::BLOOD
blood.min <- HistDAWass::get.MatH.stats(BLOOD, stat = "min")
blood.max <- HistDAWass::get.MatH.stats(BLOOD, stat = "max")
blood <- data.frame(blood.min, blood.max)

myBLOOD <- classic2sym(
  blood,
  groupby = "customize",
  minData = blood[, 2:4],
  maxData = blood[, 6:8]
)

colnames(myBLOOD$intervalData) <- HistDAWass::get.MatH.main.info(BLOOD)$varnames
ggInterval_PCA(myBLOOD$intervalData, plot = FALSE)$ggplotPCA
```

## Midpoint-Range Data via MAINT.Data

This example is evaluated only when the optional package `MAINT.Data` is
installed.

```{r maint_data_example, eval = requireNamespace("MAINT.Data", quietly = TRUE)}
AbaloneIdt <- MAINT.Data::AbaloneIdt

AbaloneRange <- exp(AbaloneIdt@LogR)
AbaloneMid <- AbaloneIdt@MidP
AbaloneBounds <- data.frame(
  AbaloneMid - AbaloneRange / 2,
  AbaloneMid + AbaloneRange / 2
)

myAbalone <- classic2sym(
  AbaloneBounds,
  groupby = "customize",
  minData = AbaloneBounds[, 1:7],
  maxData = AbaloneBounds[, 8:14]
)

colnames(myAbalone$intervalData) <- AbaloneIdt@VarNames
ggInterval_PCA(myAbalone$intervalData, plot = FALSE)$ggplotPCA
```

# PCA and Correlation Heatmaps

```{r pca_example}
pca.results <- RSDA::sym.pca(facedata, method = "tops")
rownames(pca.results$Sym.Components) <- rownames(facedata)

ggInterval_scatterplot(
  pca.results$Sym.Components,
  aes(Dim.1, Dim.2, fill = as.factor(Subjects)),
  labelSize = 2.6,
  labelPosition = "topright",
  labelNudgeX = -0.2,
  labelNudgeY = 0.15,
  checkOverlap = TRUE,
  col = "black"
) +
  scale_fill_brewer(palette = "Set1") +
  scale_x_continuous(expand = expansion(mult = c(0.08, 0.12))) +
  scale_y_continuous(expand = expansion(mult = c(0.06, 0.10))) +
  coord_fixed(ratio = 1) +
  labs(x = "PCA-1", y = "PCA-2", fill = "Subjects")

ggInterval_PCA(facedata, poly = TRUE, concepts_group = as.factor(Subjects)) +
  coord_fixed(ratio = 1) +
  labs(x = "PCA-1", y = "PCA-2")
```

```{r corrplot_example}
ggInterval_corrplot(facedata, method = "BG", triangle = "lower")
```

# Citation

Use `citation("ggInterval")` after installation to retrieve the current
package citation metadata.

# Session Information

```{r session_info}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_tsplot.R
\name{ggInterval_lineplot}
\alias{ggInterval_lineplot}
\alias{ggInterval_tsplot}
\title{Interval-valued line plot}
\usage{
ggInterval_lineplot(data = NULL, mapping = aes(NULL),
barWidth = 0.5, add_line = TRUE)

ggInterval_tsplot(data = NULL, mapping = aes(NULL),
barWidth = 0.5, add_line = TRUE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame, which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
Must include x for the ordered variable and y for the interval variable.
Optional aesthetics include group and fill for multiple line series.
It is the same as the mapping of ggplot2.}

\item{barWidth}{The width of the crossbar or errorbar indicating
the interval range at each x position. Default is 0.5.}

\item{add_line}{Logical; if TRUE (default), connect interval centers with a
line. Set FALSE when adding a customized line layer with
\code{geom_line(...)}.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize interval-valued data along an ordered horizontal
axis. A line can connect the centers of intervals at each position, with
crossbars or errorbars indicating the interval range. When the horizontal
axis is time, the display becomes an interval-valued time-series plot.
}
\examples{
\donttest{
if (requireNamespace("TTR", quietly = TRUE)) {
  data("ttrc", package = "TTR")
  stock.data <- subset(
    ttrc[, c("Date", "Close", "Low", "High")],
    format(Date, "\%Y-\%m") \%in\% c("1985-01", "1985-02", "1985-03")
  )
  stock.data$Month <- factor(
    month.abb[as.integer(format(stock.data$Date, "\%m"))],
    levels = month.abb[1:3]
  )
  stock.data$Day <- as.integer(format(stock.data$Date, "\%d"))
  stock.i <- classic2sym(
    stock.data,
    groupby = "customize",
    minData = stock.data$Low,
    maxData = stock.data$High
  )
  ggInterval_lineplot(stock.i, aes(y = V1, x = Day), barWidth = 0.6) +
    geom_point(aes(y = Close), shape = 21, fill = "#D95F02",
               color = "black", size = 1.6, stroke = 0.2) +
    coord_cartesian(xlim = c(1, 31), expand = FALSE) +
    facet_wrap(~Month, ncol = 1, scales = "free_y") +
    scale_x_continuous(breaks = c(1, 8, 15, 22, 29)) +
    labs(x = "Day of month", y = "Price") +
    ggthemes::theme_economist() +
    theme(strip.text = element_text(face = "bold"))
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classic2sym.R
\name{classic2sym}
\alias{classic2sym}
\title{Convert classical data frame into a symbolic data.}
\usage{
classic2sym(data=NULL,groupby = "kmeans",k=5,minData=NULL,maxData=NULL,
modalData = NULL)
}
\arguments{
\item{data}{A classical data frame that you want to be converted into
interval-valued data}

\item{groupby}{A way to aggregate. It can be either a clustering method
or a variable name which exist in input data (necessary factor type)
. Default "kmeans".}

\item{k}{A number of group,which is used by clustering. Default k = 5.}

\item{minData}{if choose groupby parameter as 'customize',user need to define
which data is min data or max data.}

\item{maxData}{if choose groupby parameter as 'customize',user need to define
which data is min data or max data.}

\item{modalData}{list, each cell of list contain a set of column index of its
modal multi-valued data of the input data. the value of it is a proportion
presentation, and sum of each row in these column must be equal to 1.
ex 0,1,0  or  0.2,0.3,0.5. the input type of modalData for example is
modalData[[1]] = c(2, 3), modalData[[2]] = c(7:10), that 2, 3, 7, 8, 9, 10
columns are modal type of the data. Note: the option is only valid when
groupby == "customize".}
}
\value{
classic2sym returns an object of class "ggInterval",which
has interval-valued data and related outputs as follows.
\itemize{
  \item intervalData - The Interval data after converting also known
  as a RSDA object.
  \item rawData - Classical data that user input.
  \item clusterResult - Cluster results .If the groupby method is
  a clustering method then it will exist.
  \item statisticsDF - A list contains data frame including some
  typically statistics in each group.
}
}
\description{
A function for converting a classical data, which may
present as a data frame or a matrix with one entry one value, into
a symbolic data object, which is represented as an interval or a set in an entry.
Object after converting is ggInterval class containing interval
data and raw data(if it exist) and typically statistics.
}
\examples{
#classical data to symbolic data
classic2sym(iris)
classic2sym(mtcars, groupby = "kmeans", k = 10)
classic2sym(iris, groupby = "hclust", k = 7)
classic2sym(iris, groupby = "Species")

x1<-runif(10, -30, -10)
y1<-runif(10, -10, 30)
x2<-runif(10, -5, 5)
y2<-runif(10, 10, 50)
x3<-runif(10, -50, 30)
y3<-runif(10, 31, 60)

d<-data.frame(min1=x1,max1=y1,min2=x2,max2=y2,min3=x3,max3=y3)
classic2sym(d,groupby="customize",minData=d[,c(1,3,5)],maxData=d[,c(2,4,6)])
classic2sym(d,groupby="customize",minData=d$min1,maxData=d$min2)


#example for build modal data
#for the first modal data proportion
a1 <- runif(10, 0,0.4) \%>\% round(digits = 1)
a2 <- runif(10, 0,0.4) \%>\% round(digits = 1)

#for the second modal data proportion
b1 <- runif(10, 0,0.4) \%>\% round(digits = 1)
b2 <- runif(10, 0,0.4) \%>\% round(digits = 1)

#for interval-valued data
c1 <- runif(10, 10, 20) \%>\% round(digits = 0)
c2 <- runif(10, -50, -10) \%>\% round(digits = 0)

#build simulated data
d <- data.frame(a1 = a1, a2 = a2, a3 = 1-(a1+a2),
                c1 = c1, c2 = c2,
                b1 = b1, b2 = b2, b3 = 1-(b1+b2))

#transformation
classic2sym(d, groupby = "customize",
            minData = d$c2,
            maxData = d$c1,
            modalData = list(1:3, 6:8))#two modal data

#extract the data
symObj<-classic2sym(iris)
symObj$intervalData       #interval data
symObj$rawData            #raw data
symObj$clusterResult      #cluster result
symObj$statisticsDF       #statistics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mushroom}
\alias{mushroom}
\title{mushroom data example}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}, \code{symbolic_tbl}) with 23 rows and 3 columns.
}
\source{
Adapted from \code{RSDA::mushroom}; see
  \url{https://CRAN.R-project.org/package=RSDA}.
}
\usage{
data(mushroom)
}
\description{
mushroom interval data example.
}
\examples{
data(mushroom)
\donttest{
ggInterval_scatterplot(mushroom, aes(x = Cap.Widths, y = Stipe.Lengths))
}
}
\references{
Billard, Lynne and Diday, Edwin (2006).
\emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
Chichester, UK: John Wiley and Sons.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_2DhistMatrix.R
\name{ggInterval_2DhistMatrix}
\alias{ggInterval_2DhistMatrix}
\title{2-Dimension histogram matrix}
\usage{
ggInterval_2DhistMatrix(data = NULL, mapping = aes(NULL),
method = "equal-bin", xBins = 8, yBins = 8,
display = "p", palette = "Blues", direction = 1, tau = 0,
removeZero = FALSE, cell_labels = FALSE, label_rule = "above-mean",
addFreq = NULL)
}
\arguments{
\item{data}{A ggInterval object. It can also be either an RSDA object or
a classical data frame, which will be automatically converted to
ggInterval data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit.aes = TRUE (the default), it is combined with
the default mapping at the top level of the plot. It is the same as the
mapping of ggplot2. This function ignores \code{x} and \code{y} mappings
and plots all continuous interval-valued variables.}

\item{method}{Histogram partition method. Use \code{"equal-bin"} for
equally spaced bins or \code{"unequal-bin"} for non-equidistant bins
defined by the observed interval endpoints.}

\item{xBins}{Number of x-axis bins used when \code{method = "equal-bin"}.}

\item{yBins}{Number of y-axis bins used when \code{method = "equal-bin"}.}

\item{display}{Metric shown in the cells. Use \code{"p"} for relative
frequency, \code{"f"} for weighted frequency, or \code{"h"} for density.}

\item{palette}{ColorBrewer palette passed to \code{scale_fill_distiller()}.}

\item{direction}{Direction passed to \code{scale_fill_distiller()}.}

\item{tau}{Non-negative tolerance used when \code{method = "unequal-bin"}.
Consecutive breakpoints whose gaps are smaller than \code{tau} are merged.}

\item{removeZero}{Whether remove cells whose frequency is equal to zero.}

\item{cell_labels}{Logical. If \code{TRUE}, add cell labels.}

\item{label_rule}{Rule used when \code{cell_labels = TRUE}. The default
\code{"above-mean"} shows only cells whose displayed values exceed the
mean value within the matrix. Alternatives are \code{"nonzero"} and
\code{"all"}.}

\item{addFreq}{Deprecated alias for \code{cell_labels}.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize all continuous interval-valued variables with a
matrix of 2D histograms. Each off-diagonal panel shows a 2D histogram for
a pair of variables, and each diagonal panel displays the variable name.
}
\examples{
ggInterval_2DhistMatrix(
  oils,
  xBins = 5,
  yBins = 5,
  display = "p",
  palette = "Blues",
  cell_labels = TRUE
)

ggInterval_2DhistMatrix(
  oils,
  method = "unequal-bin",
  display = "p",
  palette = "Blues",
  tau = 0.5
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{scale}
\alias{scale}
\alias{scale.default}
\alias{scale.symbolic_tbl}
\alias{scale.symbolic_interval}
\title{scale for symbolic data table}
\usage{
scale(x, ...)

\method{scale}{default}(x, center = TRUE, scale = TRUE, ...)

\method{scale}{symbolic_tbl}(x, ...)

\method{scale}{symbolic_interval}(x, ...)
}
\arguments{
\item{x}{A ggInterval object. It can also be either RSDA object or
classical data frame, which will be automatically convert to ggInterval
data.}

\item{...}{Used by other R function.}

\item{center}{same as base::scale, either a logical value or numeric-alike vector of length equal to the number of columns of x, where nmeric-alike means that as.numeric(.) will be applied successfully if is.numeric(.) is not true.}

\item{scale}{same as base::scale, either a logical value or a numeric-alike vector of length equal to the number of columns of x.}
}
\value{
Return a scale ggInterval object.
}
\description{
scale for symbolic data table
}
\examples{

#For all interval-valued
scale(facedata)

#For both interval-valued and modal multi-valued
scale(mtcars.i)

}
\keyword{Symbolic}
\keyword{scale}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_3Dscatterplot.R
\name{ggInterval_3Dscatterplot}
\alias{ggInterval_3Dscatterplot}
\title{3D scatter plot for interval data}
\usage{
ggInterval_3Dscatterplot(data = NULL,mapping = aes(NULL),scale=FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either an RSDA object or
a classical data frame, which will be automatically converted to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{scale}{A boolean variable. \code{TRUE} standardizes the data and
\code{FALSE} leaves the data unchanged.
If variance is too large(or small) or the difference between
two variables are too large,it will be distorted or difficult to see, which
may happen when variables are measured in different units.}
}
\value{
Return a ggplot2 object (It will still be 2-Dimension).
}
\description{
Visualize the three continuous variable distribution
by collecting all vertices in each interval to form a shape of
cube.Also show the difference between each group.
}
\examples{
ggInterval_3Dscatterplot(facedata[1:5, ], aes(x = BC, y = EH, z = GH))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_indexImage.R
\name{ggInterval_indexImage}
\alias{ggInterval_indexImage}
\title{Index image plot for interval-valued data}
\usage{
ggInterval_indexImage(data = NULL, mapping = aes(NULL),
column_condition = TRUE, full_strip = FALSE, plotAll = FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame,which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.}

\item{column_condition}{Logical. If \code{TRUE}, the color scale is
applied separately within each variable (column condition). If
\code{FALSE}, one common color scale is applied to the whole matrix
(matrix condition).}

\item{full_strip}{Logical. If \code{TRUE}, each strip spans the full
plotting width. If \code{FALSE}, strip widths reflect the interval ranges.}

\item{plotAll}{Logical. If \code{TRUE}, produce a multivariate image plot
for all continuous interval-valued variables. The default is \code{FALSE}.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize interval-valued observations with color strips.
For a single variable, the display acts as a color-based analogue of an
index plot. When \code{plotAll = TRUE}, the function produces a
multivariate image plot across all continuous interval-valued variables.
}
\examples{
mydata <- ggInterval::facedata
p <- ggInterval_indexImage(mydata, aes(x = AD),
  full_strip = TRUE, column_condition = TRUE)
# Recommend adding coord_flip() to make the single-variable display clearer.
p + coord_flip()
ggInterval_indexImage(
  mydata,
  plotAll = TRUE,
  full_strip = TRUE,
  column_condition = FALSE
) +
  scale_colour_distiller(palette = "Blues", direction = 1)
ggInterval_indexImage(mydata, plotAll = TRUE, full_strip = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{iris.i}
\alias{iris.i}
\title{iris.i data example}
\format{
An object of class \code{data.frame} (inherits from \code{symbolic_tbl}) with 3 rows and 4 columns.
}
\usage{
data(iris.i)
}
\description{
iris.i interval data example.
}
\examples{
data(iris.i)
ggInterval_indexplot(iris.i, aes(x = Sepal.Length))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_2Dhist.R
\name{ggInterval_2Dhist}
\alias{ggInterval_2Dhist}
\title{Visualize a 2-dimension histogram for interval-valued data}
\usage{
ggInterval_2Dhist(data = NULL, mapping = aes(NULL),
method = "equal-bin", xBins = 14, yBins = 16,
display = "p", palette = "Blues", direction = 1, tau = 0,
removeZero = FALSE, cell_labels = FALSE, label_rule = "above-mean",
addFreq = NULL)
}
\arguments{
\item{data}{A ggInterval object. It can also be either an RSDA object or
a classical data frame, which will be automatically converted to
ggInterval data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit.aes = TRUE (the default), it is combined with
the default mapping at the top level of the plot. You must supply
mapping if there is no plot mapping. It is the same as the mapping of
ggplot2.}

\item{method}{Histogram partition method. Use \code{"equal-bin"} for
equally spaced bins or \code{"unequal-bin"} for non-equidistant bins
defined by the observed interval endpoints.}

\item{xBins}{Number of x-axis bins used when \code{method = "equal-bin"}.}

\item{yBins}{Number of y-axis bins used when \code{method = "equal-bin"}.}

\item{display}{Metric shown in the cells. Use \code{"p"} for relative
frequency, \code{"f"} for weighted frequency, or \code{"h"} for density.}

\item{palette}{ColorBrewer palette passed to \code{scale_fill_distiller()}.}

\item{direction}{Direction passed to \code{scale_fill_distiller()}.}

\item{tau}{Non-negative tolerance used when \code{method = "unequal-bin"}.
Consecutive breakpoints whose gaps are smaller than \code{tau} are merged.}

\item{removeZero}{Whether remove data whose frequency is equal to zero.}

\item{cell_labels}{Logical. If \code{TRUE}, add frequency labels to cells.}

\item{label_rule}{Rule used when \code{cell_labels = TRUE}. The default
\code{"above-mean"} shows only cells whose displayed values exceed the mean
cell value. Alternatives are \code{"nonzero"} and \code{"all"}.}

\item{addFreq}{Deprecated alias for \code{cell_labels}.}
}
\value{
Return a list containing a ggplot2 object and the corresponding
frequency table.
}
\description{
Visualize the joint distribution of two continuous
interval-valued variables by dividing the x axis and y axis into
rectangles and calculating the frequency of each observation interval
in every rectangle.
}
\examples{
ggInterval_2Dhist(oils, aes(x = GRA, y = FRE),
  xBins = 5, yBins = 5,
  display = "p",
  palette = "Blues",
  cell_labels = TRUE)

ggInterval_2Dhist(oils, aes(x = GRA, y = FRE),
  method = "unequal-bin",
  display = "p",
  palette = "Blues",
  tau = 0.5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_scatterMatrix.R
\name{ggInterval_scatterMatrix}
\alias{ggInterval_scatterMatrix}
\title{Scatterplot matrix for interval-valued data}
\usage{
ggInterval_scatterMatrix(data = NULL, mapping = aes(NULL),
showLegend = FALSE, borderLinewidth = 0.08)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame,which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.}

\item{showLegend}{whether show the legend.}

\item{borderLinewidth}{Numeric border width used for the interval rectangles.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize all continuous interval-valued variables with
a matrix of pairwise interval scatterplots. Each off-diagonal panel
shows interval rectangles for one variable pair, whereas each diagonal
panel displays the variable name. This function automatically filters
out non-interval variables and plots all remaining continuous interval
variables, so explicit \code{x} and \code{y} mappings are not required.
It is not recommended to apply the function to too many variables
because the full pairwise matrix becomes computationally expensive and
visually crowded.
}
\examples{
mydata <- ggInterval::facedata
ggInterval_scatterMatrix(mydata[, 1:3])
ggInterval_scatterMatrix(
  mydata[, 1:3],
  aes(fill = "black", alpha = 0.2),
  borderLinewidth = 0.15
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{sd}
\alias{sd}
\alias{sd.default}
\alias{sd.symbolic_tbl}
\alias{sd.symbolic_interval}
\title{Generic function for the standard deviation}
\usage{
sd(x, ...)

\method{sd}{default}(x, na.rm = FALSE, ...)

\method{sd}{symbolic_tbl}(
  x,
  method = c("billard", "centers", "interval", "B", "BD", "BG"),
  na.rm = FALSE,
  ...
)

\method{sd}{symbolic_interval}(
  x,
  method = c("billard", "centers", "interval", "B", "BD", "BG"),
  na.rm = FALSE,
  ...
)
}
\arguments{
\item{x}{First symbolic variables.}

\item{...}{As in R sd function.}

\item{na.rm}{As in R sd function.}

\item{method}{The method to use.}
}
\value{
Return a real number.
}
\description{
This function compute the symbolic standard deviation.
}
\details{
Supported interval-valued methods are:
\itemize{
  \item \code{"billard"}: standard deviation based on the Billard
  univariate variance formula.
  \item \code{"centers"}: standard deviation of interval centers.
  \item \code{"interval"}: interval-valued standard deviation obtained by
  standardizing lower and upper bounds separately.
  \item \code{"B"}: method-matched standard deviation defined by
  \eqn{\sqrt{C_B(X, X)}}.
  \item \code{"BD"}: method-matched standard deviation defined by
  \eqn{\sqrt{C_{BD}(X, X)}}.
  \item \code{"BG"}: method-matched standard deviation defined by
  \eqn{\sqrt{C_{BG}(X, X)}}.
}
}
\references{
Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
\emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
Springer.

Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.

Billard, Lynne (2008). Sample covariance functions for complex quantitative
data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
Yokohama, Japan.
}
\keyword{Symbolic}
\keyword{standard}
\keyword{deviation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{summary}
\alias{summary}
\alias{summary.default}
\alias{summary.symbolic_tbl}
\alias{summary.symbolic_interval}
\alias{summary.symbolic_modal}
\title{summary for symbolic data table}
\usage{
summary(object, ...)

\method{summary}{default}(object, ...)

\method{summary}{symbolic_tbl}(object, ...)

\method{summary}{symbolic_interval}(object, ...)

\method{summary}{symbolic_modal}(object, summary_fun = "mean", ...)
}
\arguments{
\item{object}{an object for which a summary is desired.}

\item{...}{additional arguments affecting the summary produced.}

\item{summary_fun}{only works when the symbolic_modal class input, it determine which summary function be applied for each modal.}
}
\value{
Return a summary table.
}
\description{
summary for symbolic data table
}
\examples{

#For all interval-valued
summary(facedata)

#For both interval-valued and modal multi-valued
summary(Environment)

summary(Environment$URBANICITY, summary_fun = "quantile")


}
\keyword{Symbolic}
\keyword{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_boxplot.R
\name{ggInterval_boxplot}
\alias{ggInterval_boxplot}
\title{A interval Box plot}
\usage{
ggInterval_boxplot(data = NULL,mapping = aes(NULL),plotAll=FALSE,
width_type = "violin-like")
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame, which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{plotAll}{booleans, if TRUE, plot all variable together}

\item{width_type}{Box-width style. Use \code{"violin-like"} (default) to let
the rectangle widths reflect local interval concentration,
\code{"quantile-depth"} for progressively narrower quantile boxes, or
\code{"side-by-side"} for conventional boxplots of the lower and upper
bounds shown next to each other.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize the one continuous variable distribution using one of
three interval-aware boxplot styles.
}
\examples{
mydata <- ggInterval::facedata
ggInterval_boxplot(mydata, aes(x = AD))
ggInterval_boxplot(mydata, aes(x = AD), width_type = "quantile-depth")
ggInterval_boxplot(mydata, plotAll = TRUE, width_type = "side-by-side")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mtcars.i}
\alias{mtcars.i}
\title{mtcars.i data example}
\format{
An object of class \code{symbolic_tbl} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}, \code{symbolic_tbl}) with 5 rows and 11 columns.
}
\usage{
data(mtcars.i)
}
\description{
mtcars.i interval and modal data example.
}
\examples{
data(mtcars.i)
ggInterval_indexplot(mtcars.i, aes(x = mpg))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_indexplot.R
\name{ggInterval_indexplot}
\alias{ggInterval_indexplot}
\title{Plot the range of each observations}
\usage{
ggInterval_indexplot(data = NULL,mapping = aes(NULL),
plotAll = FALSE, row_order = "o", user_order = NULL, labels = FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame,which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{plotAll}{plot all variables}

\item{row_order}{Row ordering used when \code{plotAll = TRUE}: \code{"o"}
for the original order, \code{"c"} for ascending interval centers within
each variable panel, \code{"r"} for ascending interval ranges within each
variable panel, and \code{"u"} for a user-defined permutation supplied
through \code{user_order}.}

\item{user_order}{User-defined row permutation used when
\code{row_order = "u"} and \code{plotAll = TRUE}. Supply either an integer
permutation of \code{1:nrow(data)} or a character permutation of the row
names.}

\item{labels}{Logical. When \code{TRUE} and \code{plotAll = TRUE}, repeat the
row labels inside each variable panel. This is especially useful for ordered
displays with \code{row_order = "c"} or \code{"r"}. For ordered displays,
the outer left-side row labels are suppressed by default.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize the range of the variables of each observations
by using a kind of margin bar that indicate the minimal and maximal of
observations.
}
\examples{
mydata <- ggInterval::facedata
Subjects <- substr(rownames(mydata), 1, 3)
ggInterval_indexplot(mydata, aes(x = AD))
ggInterval_indexplot(mydata, aes(x = AD, fill = Subjects))
ggInterval_indexplot(mydata, plotAll = TRUE, row_order = "c")
custom_order <- c(19:21, 22:24, 1:18, 25:27)
ggInterval_indexplot(mydata["AD"], plotAll = TRUE,
                     row_order = "u", user_order = custom_order)
ggInterval_indexplot(mydata, plotAll = TRUE, row_order = "c", labels = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_PCA.R
\name{ggInterval_PCA}
\alias{ggInterval_PCA}
\title{Vertices PCA for interval data}
\usage{
ggInterval_PCA(data = NULL,mapping = aes(NULL),plot=TRUE,
                      concepts_group=NULL, poly = FALSE, adjust = TRUE,
                      showLabels = TRUE, labelSize = 3,
                      checkOverlap = FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame, which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{plot}{Boolean variable,Auto plot (if TRUE).It can also plot by
its inner object}

\item{concepts_group}{color with each group of concept}

\item{poly}{if plot a poly result}

\item{adjust}{adjust sign of the principal component}

\item{showLabels}{Logical. If \code{TRUE}, label interval scores in the plot.}

\item{labelSize}{Numeric text size used when \code{showLabels = TRUE}.}

\item{checkOverlap}{Logical. Passed to \code{geom_text()} to suppress
overlapping labels when needed.}
}
\value{
A ggplot object for PC1, PC2, and interval scores with related outputs.
\itemize{
  \item scores_interval - The interval scores after PCA.
  \item ggplotPCA - a ggplot object with x-axis and y-axis are PC1 and
  PC2.
  \item others - others are the returns values of princomp.
}
}
\description{
ggInterval_PCA performs a principal components
analysis on the given numeric interval data and returns the
results of princomp, a ggplot object, and interval scores.
}
\examples{
\donttest{
Subjects <- substr(rownames(facedata), 1, 3)
p <- ggInterval_PCA(facedata, plot = FALSE, concepts_group = Subjects)
p$ggplotPCA
p$scores_interval
ggInterval_PCA(facedata, poly = TRUE, concepts_group = Subjects)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_MMplot.R
\name{ggInterval_MMplot}
\alias{ggInterval_MMplot}
\title{A min-max plot for interval data}
\usage{
ggInterval_MMplot(data = NULL,mapping = aes(NULL),
          scaleXY = "local",plotAll=FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame,which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.}

\item{scaleXY}{default "local", which means limits of x-axis and y-axis
depend on their own variable. "global" means limits of them depend on all
variables that user input.}

\item{plotAll}{booleans, if TRUE, plot all variable together}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize the range of the variables of each observations
by marking minimal and maximal point.
}
\examples{
mydata <- ggInterval::facedata
ggInterval_MMplot(mydata, aes(x = AD))
ggInterval_MMplot(mydata, aes(x = AD, size = 3))
ggInterval_MMplot(mydata, plotAll = TRUE, scaleXY = "global") +
  theme_classic()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_corrplot.R
\name{ggInterval_corrplot}
\alias{ggInterval_corrplot}
\title{Correlation heatmap for interval-valued data}
\usage{
ggInterval_corrplot(
  data = NULL,
  method = c("centers", "B", "BD", "BG"),
  triangle = c("lower", "upper", "full"),
  showValues = TRUE,
  digits = 2,
  showLegend = TRUE
)
}
\arguments{
\item{data}{A ggInterval object. It can also be either an RSDA object or
a classical data frame, which will be automatically converted to ggInterval
data.}

\item{method}{Correlation method for interval-valued variables. It must be
one of \code{"centers"} (interval centers), \code{"B"} (Billard),
\code{"BD"} (Billard-Diday), or \code{"BG"} (Bertrand-Goupil).}

\item{triangle}{Which part of the symmetric correlation matrix to display.
It can be \code{"lower"}, \code{"upper"}, or \code{"full"}.}

\item{showValues}{Logical. If \code{TRUE}, print the correlation values in
each cell.}

\item{digits}{Number of digits used when \code{showValues = TRUE}.}

\item{showLegend}{Logical. If \code{TRUE}, display the fill legend.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize pairwise symbolic correlations between
interval-valued variables with a heatmap. This plot is especially useful
for summarizing multivariate dependence structures before more formal
modeling or dimension reduction.
}
\examples{
ggInterval_corrplot(facedata)

ggInterval_corrplot(facedata, method = "BG", triangle = "full")
}
\references{
Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
\emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
Springer.

Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.

Billard, Lynne (2008). Sample covariance functions for complex quantitative
data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
Yokohama, Japan.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Cardiological2}
\alias{Cardiological2}
\title{Cardiological data example}
\format{
An object of class \code{symbolic_tbl} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 15 rows and 3 columns.
}
\source{
Adapted from \code{RSDA::Cardiological2}; see
  \url{https://CRAN.R-project.org/package=RSDA}.
}
\usage{
data(Cardiological2)
}
\description{
Cardiological interval data example.
}
\examples{
data(Cardiological2)
ggInterval_indexplot(Cardiological2, aes(x = Syst))
}
\references{
Billard, Lynne and Diday, Edwin (2006).
\emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
Chichester, UK: John Wiley and Sons.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Environment}
\alias{Environment}
\title{Environment data example}
\format{
An object of class \code{symbolic_tbl} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 14 rows and 17 columns.
}
\usage{
data(Environment)
}
\description{
Environment interval and modal data example.
}
\examples{
data(Environment)
ggInterval_radarplot(Environment,
 plotPartial = 2,
 showLegend = FALSE,
 base_circle = TRUE,
 base_lty = 2,
 addText = FALSE)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{blood.i}
\alias{blood.i}
\title{blood.i data example}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}, \code{symbolic_tbl}) with 14 rows and 3 columns.
}
\source{
Adapted from \code{HistDAWass::BLOOD}; see
  \url{https://CRAN.R-project.org/package=HistDAWass}.
}
\usage{
data(blood.i)
}
\description{
blood.i interval data example.
}
\examples{
data(blood.i)
ggInterval_MMplot(blood.i, aes(x = Hematocrit))
}
\references{
Billard, Lynne and Diday, Edwin (2006).
\emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
Chichester, UK: John Wiley and Sons.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{cor}
\alias{cor}
\alias{cor.default}
\alias{cor.symbolic_tbl}
\alias{cor.symbolic_interval}
\title{Generic function for the correlation}
\usage{
cor(x, ...)

\method{cor}{default}(
  x,
  y = NULL,
  use = "everything",
  method = c("pearson", "kendall", "spearman"),
  ...
)

\method{cor}{symbolic_tbl}(x, ...)

\method{cor}{symbolic_interval}(x, y, method = c("centers", "B", "BD", "BG"), ...)
}
\arguments{
\item{x}{First symbolic variables.}

\item{...}{As in R cor function.}

\item{y}{Second symbolic variables.}

\item{use}{an optional character string giving a method for computing
correlation in the presence of missing values. This must be (an abbreviation of)
 one of the strings 'everything', 'all.obs', 'complete.obs', 'na.or.complete',
 or 'pairwise.complete.obs'.}

\item{method}{The method to use.}
}
\value{
Return a real number in [-1, 1].
}
\description{
This function compute the symbolic correlation
}
\details{
Supported interval-valued methods are:
\itemize{
  \item \code{"centers"}: correlation of interval centers.
  \item \code{"B"}: Billard correlation.
  \item \code{"BD"}: Billard-Diday correlation.
  \item \code{"BG"}: Bertrand-Goupil correlation.
}
For \code{"B"}, \code{"BD"}, and \code{"BG"}, the denominator uses the
corresponding method-matched standard deviation.
}
\references{
Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
\emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
Springer.

Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.

Billard, Lynne (2008). Sample covariance functions for complex quantitative
data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
Yokohama, Japan.

Rodriguez-Rojas, Oldemar (2000). \emph{Classification et modeles lineaires
en analyse des donnees symboliques}. PhD thesis, Universite Paris IX
Dauphine.
}
\keyword{Symbolic}
\keyword{correlation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_radarplot.R
\name{ggInterval_radarplot}
\alias{ggInterval_radarplot}
\title{Radar plot for symbolic data}
\usage{
ggInterval_radarplot(data=NULL,layerNumber=3,
inOneFig=TRUE,showLegend=TRUE,showXYLabs=FALSE,
plotPartial=NULL,
alpha=0.5,
base_circle=TRUE,
base_lty=2,
addText=TRUE,
type="default",
quantileNum=4,
Drift=0.5,
addText_modal=TRUE,
addText_modal.p=FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame(not recommended),which will be automatically convert to ggInterval
data.}

\item{layerNumber}{Number of concentric guide layers in the radar display.}

\item{inOneFig}{Logical. If \code{TRUE}, draw all selected observations in
one figure. Otherwise, generate separate figures for the selected
observations.}

\item{showLegend}{Logical. Whether to show the legend.}

\item{showXYLabs}{Logical. Whether to show the x- and y-axis labels.}

\item{plotPartial}{Numeric vector giving the row indices to plot. If
\code{NULL}, all observations are used. This argument is ignored when
\code{type = "quantile"} because the quantile plot summarizes all
observations.}

\item{alpha}{Alpha transparency for filled elements.}

\item{base_circle}{Logical. If \code{TRUE}, add inner guide circles.}

\item{base_lty}{Line type used in the base figure.}

\item{addText}{Logical. Whether to add interval-valued labels to the plot.}

\item{type}{Radar representation. Use \code{"default"} for polygon
intervals, \code{"rect"} for rectangle intervals, or \code{"quantile"}
for the quantile-radar display.}

\item{quantileNum}{Number of quantile layers when
\code{type = "quantile"}.}

\item{Drift}{Drift term controlling where the radar values begin.}

\item{addText_modal}{Logical. Whether to add labels for modal
multivalued variables.}

\item{addText_modal.p}{Logical. Whether to add modal percentages.}
}
\description{
Visualize interval-valued and mixed symbolic data with a
radar-style display. Interval-valued variables are represented through
their lower and upper bounds, while modal multivalued variables can be
shown with stacked bars. When \code{type = "quantile"}, the function
summarizes all observations of each interval-valued variable through
nested empirical quantile bands.
}
\examples{
\donttest{
# must specify plotPartial to the rows you want to plot
Environment.n <- Environment[, 5:17]
ggInterval_radarplot(Environment.n,
  plotPartial = 2,
  showLegend = FALSE,
  base_circle = TRUE,
  base_lty = 2,
  addText = FALSE
) +
  labs(title = "") +
  scale_fill_manual(values = c("gray50")) +
  scale_color_manual(values = c("red"))

ggInterval_radarplot(Environment,
  plotPartial = 2,
  showLegend = FALSE,
  base_circle = FALSE,
  base_lty = 1,
  addText = TRUE,
  type = "rect"
) +
  labs(title = "") +
  scale_fill_manual(values = c("gray50")) +
  scale_color_manual(values = c("gray50"))

ggInterval_radarplot(
  facedata,
  base_circle = FALSE,
  base_lty = 1,
  type = "quantile",
  quantileNum = 5,
  showLegend = TRUE,
  Drift = 0
) +
  scale_fill_brewer(palette = "Greys") +
  labs(title = "", fill = "Quantiles")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_scatterplot.R
\name{ggInterval_scatterplot}
\alias{ggInterval_scatterplot}
\title{Scatter plot for two continuous interval variables}
\usage{
ggInterval_scatterplot(data = NULL,mapping = aes(NULL),
showLabels = TRUE, labelSize = 3,
labelPosition = "topright", labelNudgeX = 0, labelNudgeY = 0,
checkOverlap = FALSE, ...)
}
\arguments{
\item{data}{A ggInterval object.It can also be either RSDA object or
classical data frame, which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.}

\item{showLabels}{Logical. If \code{TRUE}, add row labels to the rectangles.}

\item{labelSize}{Numeric text size used when \code{showLabels = TRUE}.}

\item{labelPosition}{Character label position used when
\code{showLabels = TRUE}. One of \code{"topright"}, \code{"topleft"},
\code{"bottomright"}, or \code{"bottomleft"}.}

\item{labelNudgeX}{Numeric horizontal adjustment applied to label positions.}

\item{labelNudgeY}{Numeric vertical adjustment applied to label positions.}

\item{checkOverlap}{Logical. Passed to \code{geom_text()} to suppress
overlapping labels when needed.}

\item{...}{Others in ggplot2.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize the distribution of two continuous
interval-valued variables with rectangles whose widths and heights
represent the corresponding intervals.
}
\examples{
Subjects <- substr(rownames(facedata), 1, 3)
ggInterval_scatterplot(facedata, aes(x = AD, y = BC))
ggInterval_scatterplot(
  facedata,
  aes(x = AD, y = BC, fill = Subjects),
  showLabels = TRUE,
  labelSize = 2.6,
  labelPosition = "topright",
  labelNudgeX = 0.8,
  labelNudgeY = 0.15,
  checkOverlap = TRUE,
  col = "black"
)
p <- ggInterval_scatterplot(facedata[1:10, ], aes(x = AD, y = BC, alpha = 0.2))
p + scale_fill_manual(
  labels = rownames(facedata)[1:10],
  values = rainbow(10),
  name = "Group"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_hist.R
\name{ggInterval_hist}
\alias{ggInterval_hist}
\title{Histogram for symbolic data with equal-bin or unequal-bin.}
\usage{
ggInterval_hist(data = NULL,mapping = aes(NULL),method="equal-bin",bins=10,
 plotAll = FALSE, position = "identity", alpha = 0.5)
}
\arguments{
\item{data}{A ggInterval object.It can also be either RSDA object or
classical data frame,which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{method}{It can be \code{"equal-bin"} (default) or
\code{"unequal-bin"}. Equal-bin means the histogram uses equally spaced
intervals. Unequal-bin means the bin widths are determined by the data,
so the argument \code{bins} is unused in that case.}

\item{bins}{x axis bins,which mean how many partials the variable}

\item{plotAll}{boolean, whether plot all variables, default FALSE.
will be separate into.}

\item{position}{"stack" or "identity"}

\item{alpha}{fill alpha}
}
\value{
An object of class \code{ggInterval_hist_result}. A direct call
displays the histogram, and the returned object stores the ggplot object in
\code{$plot} together with the accompanying frequency tables.
}
\description{
Visualize the continuous variable distribution
by dividing the x axis into bins,and calculating the frequency
of observation interval in each bin.
}
\examples{
mydata <- ggInterval::facedata
ggInterval_hist(mydata, aes(x = AD), bins = 10)

hist_obj <- ggInterval_hist(mydata, plotAll = TRUE, bins = 10)
hist_obj
hist_obj$`Table AD`

ggInterval_hist(mydata, aes(x = AD), method = "unequal-bin")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{RSDA2sym}
\alias{RSDA2sym}
\title{RSDA object to symbolic object for ggplot}
\usage{
RSDA2sym(data=NULL,rawData=NULL)
}
\arguments{
\item{data}{an interval data, which may transfrom by RSDA::classic.to.sym
.Note:data is a necessary parameter,and must have symbolic_tbl class.}

\item{rawData}{rawData, which can be transformed to interval data,
must be a data frame and match to data.}
}
\value{
Return an object of class "ggInterval", which
has interval-valued data and related outputs as follows.
\itemize{
  \item intervalData - The Interval data after converting also known
  as a RSDA object.
  \item rawData - Classical data that user input.
  \item clusterResult - Cluster results .If the groupby method is
  a clustering method then it will exist.
  \item statisticsDF - A list contains data frame including some
  typically statistics in each group.
}#'
}
\description{
It will be a good way to unify all symbolic data
object in R that collects all useful symbolic analysis tools
such like RSDA into the same class for management. In this way,
user who wants to do some study in symbolic data will be more
convenient for searching packages.Thus,RSDA2sym collecting RSDA
object into ggInterval object will do for plot(ggplot) and
RSDA's analysis.
}
\examples{
r<-ggInterval::Cardiological
mySym<-RSDA2sym(r)
mySym$intervalData

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{oils}
\alias{oils}
\title{oils data example}
\format{
An object of class \code{symbolic_tbl} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 8 rows and 4 columns.
}
\source{
Adapted from \code{RSDA::oils}; see
  \url{https://CRAN.R-project.org/package=RSDA}.
}
\usage{
data(oils)
}
\description{
oils interval data example.
}
\examples{
data(oils)
\donttest{
ggInterval_scatterplot(oils, aes(x = GRA, y = IOD))
}
}
\references{
Cazes, Pierre; Chouakria, Assia; Diday, Edwin; and
Schektman, Youri (1997). Extension de l'analyse en composantes principales
a des donnees de type intervalle. \emph{Revue de Statistique Appliquee},
45(3), 5--24.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_CRplot.R
\name{ggInterval_CRplot}
\alias{ggInterval_CRplot}
\title{Center-range plot for interval data}
\usage{
ggInterval_CRplot(data = NULL,mapping = aes(NULL),plotAll=FALSE,
                         addEllipse = TRUE,
                         ellipseFill = "blue",
                         ellipseAlpha = 0.3)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame,which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{plotAll}{booleans, if TRUE, plot all variable together}

\item{addEllipse}{logical, if TRUE (default), add a shaded ellipse layer.}

\item{ellipseFill}{fill color of the ellipse layer. Default is "blue".}

\item{ellipseAlpha}{alpha level of the ellipse layer. Default is 0.3.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize the relation between interval centers and ranges,
with mean reference lines and an optional ellipse overlay.
}
\examples{
mydata <- ggInterval::facedata
ggInterval_CRplot(mydata, aes(x = AD, col = "blue", shape = 2))
ggInterval_CRplot(mydata, plotAll = TRUE)
ggInterval_CRplot(mydata, plotAll = TRUE, addEllipse = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval.R
\name{ggInterval}
\alias{ggInterval}
\title{A symbolic object by R6 class for interval analysis and ggplot}
\description{
This is an object that will be used to make a ggplot
object.A ggInterval object contains both classic data that user have
and interval data which we transform.More over,some basic statistics
from row data will also be recorded in this object,and the interval
data which is from RSDA transformation will still contain RSDA
properties.
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{rawData}}{the data from user.}

\item{\code{statisticsDF}}{contains min max mean median dataframe for each group of symbolic data}

\item{\code{intervalData}}{interval data from RSDA type}

\item{\code{clusterResult}}{clustering result}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ggInterval-new}{\code{ggInterval$new()}}
\item \href{#method-ggInterval-clone}{\code{ggInterval$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ggInterval-new"></a>}}
\if{latex}{\out{\hypertarget{method-ggInterval-new}{}}}
\subsection{Method \code{new()}}{
initialize all data, check whether satisfy theirs form
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ggInterval$new(
  rawData = NULL,
  statisticsDF = NULL,
  intervalData = NULL,
  clusterResult = NULL
)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ggInterval-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ggInterval-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ggInterval$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{abalone.i}
\alias{abalone.i}
\title{abalone.i data example}
\format{
An object of class \code{data.frame} (inherits from \code{symbolic_tbl}) with 24 rows and 7 columns.
}
\source{
Adapted from \code{MAINT.Data::AbaloneIdt}; the underlying
  Abalone data are from the UCI Machine Learning Repository.
}
\usage{
data(abalone.i)
}
\description{
abalone.i interval data example.
}
\examples{
data(abalone.i)
ggInterval_indexplot(abalone.i, aes(x = Length))
}
\keyword{datasets}
